/**************************************************************************

                    =====  JUNO codes  =====

   Copyright (C) 2019 California Institute of Technology (Caltech)
   All rights reserved.

   Written by Fabiano Oyafuso (fabiano@jpl.nasa.gov)
   Adapted from Mark Hostadter's H2S absorption code:

*Last Revised:  15 April 2008
*Mark Hofstadter, JPL
*This file contains information taken  from JPL's "Molecular
*Spectroscopy Home Page" website, formerly the Poynter and Pickett
*catalog.  It concatenates their directory and line information into
*a single file.  The directory information is not in the same format as
*on the website (it is space-deliminated here, I removed the "Name"
*character string, and I added a line for the temperature levels of
*QLOG), but the line info does follow their convention.

!Start directory information!
TAG   NLINE <-------------------- QLOG ----------------------------> VER
34002  1525 514.4470 334.5222 182.7622 65.5265 23.8654 8.6996 2.9106 3

<-------------- QTLOG ------------------>
300.0  225.0  150.0  75.0  37.5  18.75  9.375

from JPL catalog:

The catalog data files are composed of 80-character card images, with one card
image per spectral line.  The format of each card image is:
  FREQ, ERR, LGINT, DR,  ELO, GUP, TAG, QNFMT,  QN',  QN"
(F13.4,F8.4, F8.4,  I2,F10.4,  I3,  I7,    I4,  6I2,  6I2)

FREQ:  Frequency of the line in MHz.
ERR:   Estimated or experimental error of FREQ in MHz.
LGINT: Base 10 logarithm of the integrated intensity in units of nm^2 MHz at
       300 K.
DR:    Degrees of freedom in the rotational partition function (0 for atoms,
       2 for linear molecules, and 3 for nonlinear molecules).
ELO:   Lower state energy in cm^{-1} relative to the ground state.
GUP:   Upper state degeneracy.
TAG:   Species tag or molecular identifier.
       A negative value flags that the line frequency has
       been measured in the laboratory.  The absolute value of TAG is then the
       species tag and ERR is the reported experimental error.  The three most
       significant digits of the species tag are coded as the mass number of
       the species.
QNFMT: Identifies the format of the quantum numbers
QN':   Quantum numbers for the upper state.
QN":   Quantum numbers for the lower state.

**************************************************************************/

#ifndef HDR_ATMOSPHERE_H2S
#define HDR_ATMOSPHERE_H2S

struct H2SLineList {
  double FREQ;
  double ERR;
  double LGINT;
  int DR;
  double ELO;
  int GUP;
  int TAG;
  int QNFMT;
  int QNp[3];
  int QNpp[3];
};

static H2SLineList h2s_lines[] = {{35028.1500,
                                   0.0700,
                                   -7.9892,
                                   3,
                                   114.1722,
                                   7,
                                   -34002,
                                   303,
                                   {3, 3, 1},
                                   {4, 0, 4}},
                                  {89497.9900,
                                   0.0700,
                                   -6.8114,
                                   3,
                                   359.3662,
                                   45,
                                   -34002,
                                   303,
                                   {7, 1, 6},
                                   {6, 4, 3}},
                                  {102028.6731,
                                   1.4430,
                                   -8.7231,
                                   3,
                                   1418.4664,
                                   81,
                                   34002,
                                   303,
                                   {13, 5, 8},
                                   {14, 4, 11}},
                                  {119664.4200,
                                   0.0700,
                                   -7.0046,
                                   3,
                                   166.3442,
                                   9,
                                   -34002,
                                   303,
                                   {4, 2, 2},
                                   {5, 1, 5}},
                                  {131463.4790,
                                   0.0700,
                                   -7.5959,
                                   3,
                                   738.4675,
                                   21,
                                   -34002,
                                   303,
                                   {10, 2, 8},
                                   {9, 5, 5}},
                                  {145177.6412,
                                   0.0816,
                                   -8.3961,
                                   3,
                                   681.6867,
                                   19,
                                   34002,
                                   303,
                                   {9, 4, 6},
                                   {8, 7, 1}},
                                  {149662.6170,
                                   0.0500,
                                   -6.9991,
                                   3,
                                   737.8607,
                                   63,
                                   -34002,
                                   303,
                                   {10, 3, 8},
                                   {9, 4, 5}},
                                  {150964.8610,
                                   0.0700,
                                   -7.8500,
                                   3,
                                   870.5962,
                                   21,
                                   -34002,
                                   303,
                                   {10, 4, 6},
                                   {11, 3, 9}},
                                  {155595.7990,
                                   0.0700,
                                   -7.3484,
                                   3,
                                   870.5961,
                                   63,
                                   -34002,
                                   303,
                                   {10, 5, 6},
                                   {11, 2, 9}},
                                  {161438.4500,
                                   0.0700,
                                   -6.6103,
                                   3,
                                   452.4945,
                                   45,
                                   -34002,
                                   303,
                                   {7, 3, 4},
                                   {8, 2, 7}},
                                  {168762.7624,
                                   0.0100,
                                   -2.8376,
                                   3,
                                   13.7463,
                                   9,
                                   -34002,
                                   303,
                                   {1, 1, 0},
                                   {1, 0, 1}},
                                  {175009.5800,
                                   0.0700,
                                   -6.6633,
                                   3,
                                   356.5151,
                                   15,
                                   -34002,
                                   303,
                                   {7, 2, 6},
                                   {6, 3, 3}},
                                  {175528.3250,
                                   0.3906,
                                   -7.8224,
                                   3,
                                   1247.8137,
                                   81,
                                   34002,
                                   303,
                                   {13, 3, 10},
                                   {12, 6, 7}},
                                  {179587.0562,
                                   0.2944,
                                   -8.2780,
                                   3,
                                   1247.6784,
                                   27,
                                   34002,
                                   303,
                                   {13, 4, 10},
                                   {12, 5, 7}},
                                  {185099.8800,
                                   0.0700,
                                   -6.9782,
                                   3,
                                   452.4943,
                                   15,
                                   -34002,
                                   303,
                                   {7, 4, 4},
                                   {8, 1, 7}},
                                  {204140.1700,
                                   0.0700,
                                   -5.5914,
                                   3,
                                   107.3682,
                                   27,
                                   -34002,
                                   303,
                                   {4, 1, 4},
                                   {3, 2, 1}},
                                  {216710.4350,
                                   0.0100,
                                   -3.0212,
                                   3,
                                   51.1402,
                                   5,
                                   -34002,
                                   303,
                                   {2, 2, 0},
                                   {2, 1, 1}},
                                  {228556.2700,
                                   0.0700,
                                   -6.0463,
                                   3,
                                   166.3435,
                                   27,
                                   -34002,
                                   303,
                                   {4, 3, 2},
                                   {5, 0, 5}},
                                  {248214.0409,
                                   8.1043,
                                   -8.7481,
                                   3,
                                   1883.8150,
                                   99,
                                   34002,
                                   303,
                                   {16, 5, 12},
                                   {15, 6, 9}},
                                  {270489.5200,
                                   0.1000,
                                   -8.5209,
                                   3,
                                   1013.3421,
                                   23,
                                   -34002,
                                   303,
                                   {11, 5, 7},
                                   {10, 8, 2}},
                                  {293967.0900,
                                   0.1000,
                                   -7.9850,
                                   3,
                                   814.4658,
                                   57,
                                   -34002,
                                   303,
                                   {9, 7, 2},
                                   {10, 4, 7}},
                                  {300505.5600,
                                   0.0700,
                                   -2.3371,
                                   3,
                                   107.3682,
                                   21,
                                   -34002,
                                   303,
                                   {3, 3, 0},
                                   {3, 2, 1}},
                                  {314437.7900,
                                   0.0700,
                                   -5.9093,
                                   3,
                                   271.1060,
                                   13,
                                   -34002,
                                   303,
                                   {6, 1, 5},
                                   {5, 4, 2}},
                                  {325325.1700,
                                   0.1000,
                                   -7.0609,
                                   3,
                                   557.0299,
                                   51,
                                   -34002,
                                   303,
                                   {8, 4, 5},
                                   {7, 7, 0}},
                                  {340450.9796,
                                   0.0505,
                                   -7.2173,
                                   3,
                                   515.2971,
                                   45,
                                   34002,
                                   303,
                                   {7, 6, 1},
                                   {8, 3, 6}},
                                  {369101.4500,
                                   0.0700,
                                   -2.1302,
                                   3,
                                   95.0563,
                                   21,
                                   -34002,
                                   303,
                                   {3, 2, 1},
                                   {3, 1, 2}},
                                  {369126.9120,
                                   0.0700,
                                   -2.5921,
                                   3,
                                   170.3358,
                                   9,
                                   -34002,
                                   303,
                                   {4, 3, 1},
                                   {4, 2, 2}},
                                  {377111.8617,
                                   4.8866,
                                   -8.5016,
                                   3,
                                   1592.4247,
                                   29,
                                   34002,
                                   303,
                                   {14, 5, 9},
                                   {15, 4, 12}},
                                  {377410.9686,
                                   4.9396,
                                   -8.0239,
                                   3,
                                   1592.4247,
                                   87,
                                   34002,
                                   303,
                                   {14, 6, 9},
                                   {15, 3, 12}},
                                  {392617.8400,
                                   0.0700,
                                   -4.8956,
                                   3,
                                   58.3689,
                                   7,
                                   -34002,
                                   303,
                                   {3, 1, 3},
                                   {2, 2, 0}},
                                  {393450.4900,
                                   0.0700,
                                   -2.7398,
                                   3,
                                   38.0161,
                                   5,
                                   -34002,
                                   303,
                                   {2, 1, 1},
                                   {2, 0, 2}},
                                  {407676.7300,
                                   0.0700,
                                   -2.1386,
                                   3,
                                   263.7389,
                                   33,
                                   -34002,
                                   303,
                                   {5, 4, 1},
                                   {5, 3, 2}},
                                  {410377.9300,
                                   0.5000,
                                   -5.7596,
                                   3,
                                   610.7241,
                                   57,
                                   -34002,
                                   303,
                                   {9, 2, 7},
                                   {8, 5, 4}},
                                  {424314.8200,
                                   0.0700,
                                   -2.6899,
                                   3,
                                   182.6485,
                                   9,
                                   -34002,
                                   303,
                                   {4, 4, 0},
                                   {4, 3, 1}},
                                  {436373.3600,
                                   0.0700,
                                   -5.0218,
                                   3,
                                   195.6614,
                                   33,
                                   -34002,
                                   303,
                                   {5, 1, 4},
                                   {4, 4, 1}},
                                  {440698.3700,
                                   0.1000,
                                   -6.8248,
                                   3,
                                   1007.6266,
                                   69,
                                   -34002,
                                   303,
                                   {11, 4, 7},
                                   {12, 3, 10}},
                                  {441837.3600,
                                   0.1000,
                                   -7.3002,
                                   3,
                                   1007.6266,
                                   23,
                                   -34002,
                                   303,
                                   {11, 5, 7},
                                   {12, 2, 10}},
                                  {451946.9700,
                                   0.0700,
                                   -6.7372,
                                   3,
                                   281.6023,
                                   33,
                                   -34002,
                                   303,
                                   {5, 5, 0},
                                   {6, 2, 5}},
                                  {452390.3300,
                                   0.0700,
                                   -2.6154,
                                   3,
                                   0.0000,
                                   3,
                                   -34002,
                                   303,
                                   {1, 1, 1},
                                   {0, 0, 0}},
                                  {457114.8300,
                                   0.1000,
                                   -7.0439,
                                   3,
                                   1082.8075,
                                   25,
                                   -34002,
                                   303,
                                   {12, 3, 9},
                                   {11, 6, 6}},
                                  {461632.4300,
                                   0.2000,
                                   -5.6849,
                                   3,
                                   227.9450,
                                   33,
                                   -34002,
                                   303,
                                   {5, 2, 3},
                                   {6, 1, 6}},
                                  {470157.6214,
                                   0.0551,
                                   -6.4934,
                                   3,
                                   552.0139,
                                   17,
                                   34002,
                                   303,
                                   {8, 3, 5},
                                   {9, 2, 8}},
                                  {471791.1663,
                                   0.1293,
                                   -6.5337,
                                   3,
                                   1082.3184,
                                   75,
                                   34002,
                                   303,
                                   {12, 4, 9},
                                   {11, 5, 6}},
                                  {475148.2200,
                                   0.2000,
                                   -6.0822,
                                   3,
                                   608.5652,
                                   19,
                                   -34002,
                                   303,
                                   {9, 3, 7},
                                   {8, 4, 4}},
                                  {475701.4800,
                                   0.2000,
                                   -6.0082,
                                   3,
                                   552.0138,
                                   51,
                                   -34002,
                                   303,
                                   {8, 4, 5},
                                   {9, 1, 8}},
                                  {493085.0000,
                                   0.2000,
                                   -6.1192,
                                   3,
                                   227.9449,
                                   11,
                                   -34002,
                                   303,
                                   {5, 3, 3},
                                   {6, 0, 6}},
                                  {493362.1600,
                                   0.0700,
                                   -2.6630,
                                   3,
                                   375.4308,
                                   13,
                                   -34002,
                                   303,
                                   {6, 5, 1},
                                   {6, 4, 2}},
                                  {505565.2300,
                                   0.0700,
                                   -2.2026,
                                   3,
                                   38.2978,
                                   15,
                                   -34002,
                                   303,
                                   {2, 2, 1},
                                   {2, 1, 2}},
                                  {517392.9163,
                                   2.6952,
                                   -7.6333,
                                   3,
                                   1682.7300,
                                   93,
                                   34002,
                                   303,
                                   {15, 4, 11},
                                   {14, 7, 8}},
                                  {521140.2695,
                                   2.1911,
                                   -8.1027,
                                   3,
                                   1682.6052,
                                   31,
                                   34002,
                                   303,
                                   {15, 5, 11},
                                   {14, 6, 8}},
                                  {535528.8180,
                                   0.0700,
                                   -4.7838,
                                   3,
                                   263.7389,
                                   39,
                                   -34002,
                                   303,
                                   {6, 2, 5},
                                   {5, 3, 2}},
                                  {555253.9720,
                                   0.0700,
                                   -2.2184,
                                   3,
                                   487.2351,
                                   45,
                                   -34002,
                                   303,
                                   {7, 5, 2},
                                   {7, 4, 3}},
                                  {567079.6180,
                                   0.0700,
                                   -2.5112,
                                   3,
                                   356.5151,
                                   13,
                                   -34002,
                                   303,
                                   {6, 4, 2},
                                   {6, 3, 3}},
                                  {568050.5570,
                                   0.0700,
                                   -2.5010,
                                   3,
                                   96.3925,
                                   7,
                                   -34002,
                                   303,
                                   {3, 3, 1},
                                   {3, 2, 2}},
                                  {570519.7493,
                                   0.1738,
                                   -6.8931,
                                   3,
                                   856.7557,
                                   63,
                                   34002,
                                   303,
                                   {10, 5, 6},
                                   {9, 8, 1}},
                                  {579799.0420,
                                   0.0700,
                                   -2.1716,
                                   3,
                                   277.3376,
                                   33,
                                   -34002,
                                   303,
                                   {5, 5, 0},
                                   {5, 4, 1}},
                                  {593170.0000,
                                   0.0700,
                                   -2.8938,
                                   3,
                                   635.4082,
                                   17,
                                   -34002,
                                   303,
                                   {8, 6, 2},
                                   {8, 5, 3}},
                                  {611441.6290,
                                   0.0700,
                                   -1.8858,
                                   3,
                                   243.3434,
                                   33,
                                   -34002,
                                   303,
                                   {5, 3, 2},
                                   {5, 2, 3}},
                                  {626474.6250,
                                   0.0700,
                                   -2.2695,
                                   3,
                                   505.7564,
                                   45,
                                   -34002,
                                   303,
                                   {7, 6, 1},
                                   {7, 5, 2}},
                                  {645967.3759,
                                   0.1643,
                                   -7.2354,
                                   3,
                                   716.9203,
                                   19,
                                   34002,
                                   303,
                                   {9, 5, 5},
                                   {8, 8, 0}},
                                  {647079.1608,
                                   13.3314,
                                   -7.9994,
                                   3,
                                   1775.5177,
                                   93,
                                   34002,
                                   303,
                                   {15, 5, 10},
                                   {16, 4, 13}},
                                  {647161.4792,
                                   13.3568,
                                   -8.4764,
                                   3,
                                   1775.5177,
                                   31,
                                   34002,
                                   303,
                                   {15, 6, 10},
                                   {16, 3, 13}},
                                  {650374.1860,
                                   0.0700,
                                   -1.9795,
                                   3,
                                   173.9673,
                                   27,
                                   -34002,
                                   303,
                                   {4, 4, 1},
                                   {4, 3, 2}},
                                  {657148.8010,
                                   0.0700,
                                   -5.4737,
                                   3,
                                   493.3685,
                                   17,
                                   -34002,
                                   303,
                                   {8, 2, 6},
                                   {7, 5, 3}},
                                  {665393.7380,
                                   0.0700,
                                   -2.2864,
                                   3,
                                   148.1406,
                                   9,
                                   -34002,
                                   303,
                                   {4, 2, 2},
                                   {4, 1, 3}},
                                  {687303.4680,
                                   0.0700,
                                   -2.1815,
                                   3,
                                   15.0901,
                                   5,
                                   -34002,
                                   303,
                                   {2, 0, 2},
                                   {1, 1, 1}},
                                  {689120.1700,
                                   0.0700,
                                   -2.6266,
                                   3,
                                   801.2849,
                                   57,
                                   -34002,
                                   303,
                                   {9, 7, 2},
                                   {9, 6, 3}},
                                  {708470.4300,
                                   0.0700,
                                   -1.8846,
                                   3,
                                   71.4243,
                                   21,
                                   -34002,
                                   303,
                                   {3, 1, 2},
                                   {3, 0, 3}},
                                  {724591.8372,
                                   0.5234,
                                   -7.2580,
                                   3,
                                   1153.9248,
                                   25,
                                   34002,
                                   303,
                                   {12, 4, 8},
                                   {13, 3, 11}},
                                  {724871.2554,
                                   0.5269,
                                   -6.7807,
                                   3,
                                   1153.9248,
                                   75,
                                   34002,
                                   303,
                                   {12, 5, 8},
                                   {13, 2, 11}},
                                  {733032.8130,
                                   0.1000,
                                   -3.3055,
                                   3,
                                   960.9002,
                                   21,
                                   -34002,
                                   303,
                                   {10, 7, 3},
                                   {10, 6, 4}},
                                  {735812.6233,
                                   0.0800,
                                   -5.7350,
                                   3,
                                   927.1022,
                                   69,
                                   34002,
                                   303,
                                   {11, 3, 8},
                                   {10, 6, 5}},
                                  {736034.0960,
                                   0.0700,
                                   -1.5778,
                                   3,
                                   13.7463,
                                   15,
                                   -34002,
                                   303,
                                   {2, 1, 2},
                                   {1, 0, 1}},
                                  {746726.4919,
                                   0.0552,
                                   -2.5026,
                                   3,
                                   776.3768,
                                   57,
                                   34002,
                                   303,
                                   {9, 6, 3},
                                   {9, 5, 4}},
                                  {747301.8900,
                                   0.0700,
                                   -2.3338,
                                   3,
                                   71.4652,
                                   7,
                                   -34002,
                                   303,
                                   {3, 2, 2},
                                   {3, 1, 3}},
                                  {748241.4900,
                                   0.0700,
                                   -2.6877,
                                   3,
                                   391.8876,
                                   13,
                                   -34002,
                                   303,
                                   {6, 6, 0},
                                   {6, 5, 1}},
                                  {749432.2800,
                                   0.0700,
                                   -2.4919,
                                   3,
                                   271.1060,
                                   11,
                                   -34002,
                                   303,
                                   {5, 5, 1},
                                   {5, 4, 2}},
                                  {762084.0699,
                                   0.5299,
                                   -7.3245,
                                   3,
                                   1222.3933,
                                   75,
                                   34002,
                                   303,
                                   {12, 6, 7},
                                   {11, 9, 2}},
                                  {765937.9100,
                                   0.0700,
                                   -1.7421,
                                   3,
                                   148.4183,
                                   27,
                                   -34002,
                                   303,
                                   {4, 3, 2},
                                   {4, 2, 3}},
                                  {767222.3005,
                                   0.0733,
                                   -5.9236,
                                   3,
                                   660.8967,
                                   57,
                                   34002,
                                   303,
                                   {9, 3, 6},
                                   {10, 2, 9}},
                                  {768445.4710,
                                   0.1000,
                                   -6.3998,
                                   3,
                                   660.8967,
                                   19,
                                   -34002,
                                   303,
                                   {9, 4, 6},
                                   {10, 1, 9}},
                                  {777907.9700,
                                   0.1000,
                                   -5.9969,
                                   3,
                                   298.9696,
                                   13,
                                   -34002,
                                   303,
                                   {6, 2, 4},
                                   {7, 1, 7}},
                                  {782355.8880,
                                   0.1000,
                                   -3.1598,
                                   3,
                                   1162.1390,
                                   69,
                                   -34002,
                                   303,
                                   {11, 8, 3},
                                   {11, 7, 4}},
                                  {784758.2480,
                                   0.1000,
                                   -5.5149,
                                   3,
                                   298.9696,
                                   39,
                                   -34002,
                                   303,
                                   {6, 3, 4},
                                   {7, 0, 7}},
                                  {785745.6850,
                                   0.1000,
                                   -6.1354,
                                   3,
                                   925.4386,
                                   23,
                                   -34002,
                                   303,
                                   {11, 4, 8},
                                   {10, 5, 5}},
                                  {792123.0232,
                                   0.6179,
                                   -7.2691,
                                   3,
                                   1490.5152,
                                   29,
                                   34002,
                                   303,
                                   {14, 4, 10},
                                   {13, 7, 7}},
                                  {794222.4640,
                                   0.1000,
                                   -2.8815,
                                   3,
                                   655.1942,
                                   17,
                                   -34002,
                                   303,
                                   {8, 7, 1},
                                   {8, 6, 2}},
                                  {800852.0700,
                                   0.1000,
                                   -2.2468,
                                   3,
                                   244.3925,
                                   11,
                                   -34002,
                                   303,
                                   {5, 4, 2},
                                   {5, 3, 3}},
                                  {804636.1150,
                                   0.1000,
                                   -6.7735,
                                   3,
                                   1490.0984,
                                   87,
                                   -34002,
                                   303,
                                   {14, 5, 10},
                                   {13, 6, 7}},
                                  {804733.2320,
                                   0.1000,
                                   -2.6729,
                                   3,
                                   608.5652,
                                   17,
                                   -34002,
                                   303,
                                   {8, 5, 3},
                                   {8, 4, 4}},
                                  {827201.4730,
                                   0.1000,
                                   -4.5083,
                                   3,
                                   387.8848,
                                   45,
                                   -34002,
                                   303,
                                   {7, 2, 5},
                                   {6, 5, 2}},
                                  {827915.0460,
                                   0.1000,
                                   -4.3089,
                                   3,
                                   182.6485,
                                   11,
                                   -34002,
                                   303,
                                   {5, 2, 4},
                                   {4, 3, 1}},
                                  {839137.9526,
                                   0.0832,
                                   -3.3396,
                                   3,
                                   985.3515,
                                   21,
                                   34002,
                                   303,
                                   {10, 8, 2},
                                   {10, 7, 3}},
                                  {841277.6000,
                                   0.0521,
                                   -4.6784,
                                   3,
                                   487.2351,
                                   51,
                                   34002,
                                   303,
                                   {8, 3, 6},
                                   {7, 4, 3}},
                                  {854965.9500,
                                   0.1000,
                                   -1.8720,
                                   3,
                                   359.3662,
                                   39,
                                   -34002,
                                   303,
                                   {6, 5, 2},
                                   {6, 4, 3}},
                                  {860132.5590,
                                   0.1000,
                                   -2.1165,
                                   3,
                                   387.8848,
                                   39,
                                   -34002,
                                   303,
                                   {6, 6, 1},
                                   {6, 5, 2}},
                                  {863805.8370,
                                   0.1000,
                                   -4.8899,
                                   3,
                                   296.1044,
                                   13,
                                   -34002,
                                   303,
                                   {6, 2, 4},
                                   {5, 5, 1}},
                                  {869442.4224,
                                   12.2676,
                                   -8.0833,
                                   3,
                                   2179.1347,
                                   105,
                                   34002,
                                   303,
                                   {17, 5, 12},
                                   {16, 8, 9}},
                                  {873079.8839,
                                   10.9977,
                                   -8.5554,
                                   3,
                                   2179.0136,
                                   35,
                                   34002,
                                   303,
                                   {17, 6, 12},
                                   {16, 7, 9}},
                                  {880057.3740,
                                   0.1000,
                                   -1.9283,
                                   3,
                                   457.8796,
                                   45,
                                   -34002,
                                   303,
                                   {7, 4, 3},
                                   {7, 3, 4}},
                                  {885432.2437,
                                   0.0428,
                                   -7.1632,
                                   3,
                                   362.3528,
                                   13,
                                   34002,
                                   303,
                                   {6, 5, 1},
                                   {7, 2, 6}},
                                  {900350.7690,
                                   0.1000,
                                   -3.9773,
                                   3,
                                   1380.5116,
                                   25,
                                   -34002,
                                   303,
                                   {12, 9, 3},
                                   {12, 8, 4}},
                                  {903750.1058,
                                   4.3663,
                                   -8.1136,
                                   3,
                                   1652.5842,
                                   87,
                                   34002,
                                   303,
                                   {14, 7, 8},
                                   {13, 10, 3}},
                                  {908368.5893,
                                   1.2735,
                                   -3.8459,
                                   3,
                                   1586.4084,
                                   81,
                                   34002,
                                   303,
                                   {13, 9, 4},
                                   {13, 8, 5}},
                                  {908984.5868,
                                   0.4203,
                                   -3.8810,
                                   3,
                                   1350.1911,
                                   25,
                                   34002,
                                   303,
                                   {12, 8, 4},
                                   {12, 7, 5}},
                                  {910665.4342,
                                   0.1074,
                                   -8.3596,
                                   3,
                                   686.4885,
                                   51,
                                   34002,
                                   303,
                                   {8, 8, 1},
                                   {9, 3, 6}},
                                  {910665.7890,
                                   0.1000,
                                   -2.3296,
                                   3,
                                   526.6533,
                                   45,
                                   -34002,
                                   303,
                                   {7, 7, 0},
                                   {7, 6, 1}},
                                  {912490.8534,
                                   31.6971,
                                   -8.1523,
                                   3,
                                   1967.7170,
                                   99,
                                   34002,
                                   303,
                                   {16, 6, 11},
                                   {17, 3, 14}},
                                  {913106.1882,
                                   0.0355,
                                   -7.0531,
                                   3,
                                   166.3442,
                                   9,
                                   34002,
                                   303,
                                   {4, 4, 0},
                                   {5, 1, 5}},
                                  {922755.0114,
                                   0.0718,
                                   -7.6929,
                                   3,
                                   624.4145,
                                   17,
                                   34002,
                                   303,
                                   {8, 6, 2},
                                   {9, 3, 7}},
                                  {926544.4923,
                                   0.4034,
                                   -6.8743,
                                   3,
                                   896.1960,
                                   63,
                                   34002,
                                   303,
                                   {10, 6, 5},
                                   {9, 9, 0}},
                                  {928640.5350,
                                   0.1000,
                                   -2.5082,
                                   3,
                                   493.3685,
                                   15,
                                   -34002,
                                   303,
                                   {7, 6, 2},
                                   {7, 5, 3}},
                                  {930145.8320,
                                   0.1000,
                                   -3.1859,
                                   3,
                                   117.3920,
                                   27,
                                   -34002,
                                   303,
                                   {4, 2, 3},
                                   {3, 3, 0}},
                                  {938446.0760,
                                   0.5700,
                                   -7.2633,
                                   3,
                                   1051.5043,
                                   23,
                                   34002,
                                   303,
                                   {11, 6, 6},
                                   {10, 9, 1}},
                                  {947260.4700,
                                   0.1000,
                                   -2.2004,
                                   3,
                                   324.9179,
                                   13,
                                   -34002,
                                   303,
                                   {6, 3, 3},
                                   {6, 2, 4}},
                                  {970575.2800,
                                   0.1000,
                                   -2.9706,
                                   3,
                                   1129.7641,
                                   69,
                                   -34002,
                                   303,
                                   {11, 7, 4},
                                   {11, 6, 5}},
                                  {973853.4860,
                                   0.1000,
                                   -2.6008,
                                   3,
                                   824.2715,
                                   57,
                                   -34002,
                                   303,
                                   {9, 8, 1},
                                   {9, 7, 2}},
                                  {976182.6890,
                                   0.1000,
                                   -2.7566,
                                   3,
                                   524.3446,
                                   15,
                                   -34002,
                                   303,
                                   {7, 7, 1},
                                   {7, 6, 2}},
                                  {985693.1554,
                                   2.8888,
                                   -4.7665,
                                   3,
                                   1838.6480,
                                   29,
                                   34002,
                                   303,
                                   {14, 10, 4},
                                   {14, 9, 5}},
                                  {993101.7899,
                                   59.9585,
                                   -1.1747,
                                   3,
                                   38.2978,
                                   21,
                                   -34002,
                                   303,
                                   {3, 0, 3},
                                   {2, 1, 2}},
                                  {993101.7899,
                                   59.9585,
                                   -1.6119,
                                   3,
                                   210.2173,
                                   33,
                                   -34002,
                                   303,
                                   {5, 2, 3},
                                   {5, 1, 4}},
                                  {994980.9150,
                                   0.1000,
                                   -5.5429,
                                   3,
                                   781.2679,
                                   21,
                                   -34002,
                                   303,
                                   {10, 3, 7},
                                   {9, 6, 4}},
                                  {1002778.6980,
                                   0.1000,
                                   -1.6394,
                                   3,
                                   38.0161,
                                   7,
                                   -34002,
                                   303,
                                   {3, 1, 3},
                                   {2, 0, 2}},
                                  {1004224.9830,
                                   2.2068,
                                   -6.8928,
                                   3,
                                   1309.4694,
                                   81,
                                   34002,
                                   303,
                                   {13, 4, 9},
                                   {14, 3, 12}},
                                  {1004294.4939,
                                   2.2078,
                                   -7.3698,
                                   3,
                                   1309.4694,
                                   27,
                                   34002,
                                   303,
                                   {13, 5, 9},
                                   {14, 2, 12}},
                                  {1010398.5877,
                                   0.1307,
                                   -8.5024,
                                   3,
                                   951.6482,
                                   21,
                                   34002,
                                   303,
                                   {10, 7, 3},
                                   {11, 4, 8}},
                                  {1018347.2300,
                                   0.1000,
                                   -2.1436,
                                   3,
                                   114.1722,
                                   9,
                                   -34002,
                                   303,
                                   {4, 1, 3},
                                   {4, 0, 4}},
                                  {1019453.6480,
                                   0.1000,
                                   -2.2412,
                                   3,
                                   646.4593,
                                   51,
                                   -34002,
                                   303,
                                   {8, 7, 2},
                                   {8, 6, 3}},
                                  {1023122.5720,
                                   0.1000,
                                   -2.0710,
                                   3,
                                   210.2648,
                                   11,
                                   -34002,
                                   303,
                                   {5, 3, 3},
                                   {5, 2, 4}},
                                  {1024023.1150,
                                   0.1000,
                                   -3.1410,
                                   3,
                                   1188.2356,
                                   69,
                                   -34002,
                                   303,
                                   {11, 9, 2},
                                   {11, 8, 3}},
                                  {1025884.4170,
                                   0.1000,
                                   -1.6776,
                                   3,
                                   325.1463,
                                   39,
                                   -34002,
                                   303,
                                   {6, 4, 3},
                                   {6, 3, 4}},
                                  {1026511.1650,
                                   0.1000,
                                   -1.6615,
                                   3,
                                   114.1776,
                                   27,
                                   -34002,
                                   303,
                                   {4, 2, 3},
                                   {4, 1, 4}},
                                  {1040275.6000,
                                   0.1000,
                                   -2.3177,
                                   3,
                                   458.6686,
                                   15,
                                   -34002,
                                   303,
                                   {7, 5, 3},
                                   {7, 4, 4}},
                                  {1056278.0510,
                                   0.1000,
                                   -3.0024,
                                   3,
                                   681.6867,
                                   17,
                                   -34002,
                                   303,
                                   {8, 8, 0},
                                   {8, 7, 1}},
                                  {1059153.3580,
                                   0.1000,
                                   -6.4581,
                                   3,
                                   779.1273,
                                   21,
                                   -34002,
                                   303,
                                   {10, 3, 7},
                                   {11, 2, 10}},
                                  {1059420.3077,
                                   0.0981,
                                   -5.9808,
                                   3,
                                   779.1273,
                                   63,
                                   34002,
                                   303,
                                   {10, 4, 7},
                                   {11, 1, 10}},
                                  {1061034.8288,
                                   9.4976,
                                   -4.6776,
                                   3,
                                   2072.2132,
                                   93,
                                   34002,
                                   303,
                                   {15, 10, 5},
                                   {15, 9, 6}},
                                  {1063111.5870,
                                   0.1000,
                                   -3.0401,
                                   3,
                                   925.4386,
                                   21,
                                   -34002,
                                   303,
                                   {10, 6, 4},
                                   {10, 5, 5}},
                                  {1067447.9950,
                                   0.1000,
                                   -6.0675,
                                   3,
                                   1307.3605,
                                   81,
                                   -34002,
                                   303,
                                   {13, 4, 9},
                                   {12, 7, 6}},
                                  {1071313.6080,
                                   0.1000,
                                   -2.0598,
                                   3,
                                   610.7241,
                                   51,
                                   -34002,
                                   303,
                                   {8, 6, 3},
                                   {8, 5, 4}},
                                  {1072836.5480,
                                   0.1000,
                                   -1.2736,
                                   3,
                                   19.3756,
                                   15,
                                   -34002,
                                   303,
                                   {2, 2, 1},
                                   {1, 1, 0}},
                                  {1075532.4200,
                                   0.1000,
                                   -3.8655,
                                   3,
                                   1616.7083,
                                   81,
                                   -34002,
                                   303,
                                   {13, 10, 3},
                                   {13, 9, 4}},
                                  {1081348.2083,
                                   0.1726,
                                   -5.5154,
                                   3,
                                   379.4073,
                                   45,
                                   34002,
                                   303,
                                   {7, 2, 5},
                                   {8, 1, 8}},
                                  {1082688.6105,
                                   0.1715,
                                   -5.9920,
                                   3,
                                   379.4073,
                                   15,
                                   34002,
                                   303,
                                   {7, 3, 5},
                                   {8, 0, 8}},
                                  {1091253.5409,
                                   59.9585,
                                   -2.5011,
                                   3,
                                   680.4646,
                                   51,
                                   -34002,
                                   303,
                                   {8, 8, 1},
                                   {8, 7, 2}},
                                  {1094613.5859,
                                   17.9895,
                                   -5.6955,
                                   3,
                                   2357.7165,
                                   33,
                                   34002,
                                   303,
                                   {16, 11, 5},
                                   {16, 10, 6}},
                                  {1107634.2027,
                                   0.4725,
                                   -6.4971,
                                   3,
                                   1306.0224,
                                   27,
                                   34002,
                                   303,
                                   {13, 5, 9},
                                   {12, 6, 6}},
                                  {1109146.0230,
                                   4.4178,
                                   -4.6146,
                                   3,
                                   1801.6509,
                                   29,
                                   34002,
                                   303,
                                   {14, 9, 5},
                                   {14, 8, 6}},
                                  {1122139.0593,
                                   59.9585,
                                   -2.8031,
                                   3,
                                   781.2679,
                                   19,
                                   -34002,
                                   303,
                                   {9, 7, 3},
                                   {9, 6, 4}},
                                  {1122632.8175,
                                   59.9585,
                                   -2.9783,
                                   3,
                                   818.6985,
                                   19,
                                   -34002,
                                   303,
                                   {9, 8, 2},
                                   {9, 7, 3}},
                                  {1136399.9976,
                                   4.5625,
                                   -7.8107,
                                   3,
                                   1960.2475,
                                   33,
                                   34002,
                                   303,
                                   {16, 5, 11},
                                   {15, 8, 8}},
                                  {1138969.2379,
                                   5.4947,
                                   -4.7436,
                                   3,
                                   2107.6055,
                                   93,
                                   34002,
                                   303,
                                   {15, 11, 4},
                                   {15, 10, 5}},
                                  {1141879.5471,
                                   0.0714,
                                   -4.8798,
                                   3,
                                   776.3768,
                                   63,
                                   34002,
                                   303,
                                   {10, 4, 7},
                                   {9, 5, 4}},
                                  {1144070.0768,
                                   59.9585,
                                   -3.3425,
                                   3,
                                   1013.3421,
                                   21,
                                   -34002,
                                   303,
                                   {10, 9, 1},
                                   {10, 8, 2}},
                                  {1147682.9549,
                                   3.5348,
                                   -7.3207,
                                   3,
                                   1959.8719,
                                   99,
                                   34002,
                                   303,
                                   {16, 6, 11},
                                   {15, 7, 8}},
                                  {1154680.3314,
                                   59.9585,
                                   -2.1991,
                                   3,
                                   737.8607,
                                   57,
                                   -34002,
                                   303,
                                   {9, 5, 4},
                                   {9, 4, 5}},
                                  {1173493.2011,
                                   68.1868,
                                   -8.3932,
                                   3,
                                   2168.9926,
                                   105,
                                   34002,
                                   303,
                                   {17, 5, 12},
                                   {18, 4, 15}},
                                  {1173637.1571,
                                   1.0067,
                                   -7.7001,
                                   3,
                                   1094.5219,
                                   23,
                                   34002,
                                   303,
                                   {11, 7, 5},
                                   {10, 10, 0}},
                                  {1181530.5962,
                                   1.9861,
                                   -7.7703,
                                   3,
                                   1451.1036,
                                   27,
                                   34002,
                                   303,
                                   {13, 7, 7},
                                   {12, 10, 2}},
                                  {1182386.8506,
                                   59.9585,
                                   -2.7901,
                                   3,
                                   856.7557,
                                   57,
                                   -34002,
                                   303,
                                   {9, 9, 0},
                                   {9, 8, 1}},
                                  {1193100.8335,
                                   59.9585,
                                   -2.6354,
                                   3,
                                   970.3139,
                                   63,
                                   -34002,
                                   303,
                                   {10, 8, 3},
                                   {10, 7, 4}},
                                  {1196012.1180,
                                   59.9585,
                                   -1.4217,
                                   3,
                                   55.1616,
                                   21,
                                   -34002,
                                   303,
                                   {3, 1, 2},
                                   {2, 2, 1}},
                                  {1199939.6990,
                                   59.9585,
                                   -3.2562,
                                   3,
                                   856.1456,
                                   19,
                                   -34002,
                                   303,
                                   {9, 9, 1},
                                   {9, 8, 2}},
                                  {1200045.6124,
                                   0.0685,
                                   -4.5309,
                                   3,
                                   646.4593,
                                   57,
                                   34002,
                                   303,
                                   {9, 3, 6},
                                   {8, 6, 3}},
                                  {1201901.9185,
                                   0.0522,
                                   -4.3120,
                                   3,
                                   375.4308,
                                   15,
                                   34002,
                                   303,
                                   {7, 3, 5},
                                   {6, 4, 2}},
                                  {1209655.4684,
                                   1.6060,
                                   -3.6112,
                                   3,
                                   1546.0586,
                                   81,
                                   34002,
                                   303,
                                   {13, 8, 5},
                                   {13, 7, 6}},
                                  {1215944.2795,
                                   0.5446,
                                   -3.9527,
                                   3,
                                   1410.5441,
                                   25,
                                   34002,
                                   303,
                                   {12, 10, 2},
                                   {12, 9, 3}},
                                  {1218648.9210,
                                   32.9824,
                                   -5.7607,
                                   3,
                                   2658.7833,
                                   105,
                                   34002,
                                   303,
                                   {17, 12, 5},
                                   {17, 11, 6}},
                                  {1221574.4166,
                                   69.8043,
                                   -6.7590,
                                   3,
                                   2935.7265,
                                   37,
                                   34002,
                                   303,
                                   {18, 12, 6},
                                   {18, 11, 7}},
                                  {1225207.1068,
                                   59.9585,
                                   -2.3727,
                                   3,
                                   567.6967,
                                   17,
                                   -34002,
                                   303,
                                   {8, 4, 4},
                                   {8, 3, 5}},
                                  {1230377.7403,
                                   43.5745,
                                   -5.6518,
                                   3,
                                   2617.7424,
                                   105,
                                   34002,
                                   303,
                                   {17, 11, 6},
                                   {17, 10, 7}},
                                  {1232118.2223,
                                   59.9585,
                                   -2.8112,
                                   3,
                                   1010.1115,
                                   63,
                                   -34002,
                                   303,
                                   {10, 9, 2},
                                   {10, 8, 3}},
                                  {1238258.6573,
                                   0.0690,
                                   -4.2757,
                                   3,
                                   416.5757,
                                   45,
                                   34002,
                                   303,
                                   {7, 3, 4},
                                   {6, 6, 1}},
                                  {1247674.3340,
                                   0.0605,
                                   -8.3045,
                                   3,
                                   515.2886,
                                   15,
                                   34002,
                                   303,
                                   {7, 7, 1},
                                   {8, 2, 6}},
                                  {1253279.8601,
                                   1.8415,
                                   -6.9997,
                                   3,
                                   1265.5556,
                                   75,
                                   34002,
                                   303,
                                   {12, 7, 6},
                                   {11, 10, 1}},
                                  {1271191.0600,
                                   0.5000,
                                   -1.6660,
                                   3,
                                   415.4772,
                                   45,
                                   -34002,
                                   303,
                                   {7, 3, 4},
                                   {7, 2, 5}},
                                  {1279721.6023,
                                   1.2900,
                                   -4.7503,
                                   3,
                                   1871.5272,
                                   29,
                                   34002,
                                   303,
                                   {14, 11, 3},
                                   {14, 10, 4}},
                                  {1280115.0109,
                                   6.7197,
                                   -7.5625,
                                   3,
                                   1474.2376,
                                   29,
                                   34002,
                                   303,
                                   {14, 4, 10},
                                   {15, 3, 13}},
                                  {1280132.6988,
                                   6.7196,
                                   -7.0854,
                                   3,
                                   1474.2376,
                                   87,
                                   34002,
                                   303,
                                   {14, 5, 10},
                                   {15, 2, 13}},
                                  {1280325.2900,
                                   0.5000,
                                   -1.3561,
                                   3,
                                   71.4652,
                                   9,
                                   -34002,
                                   303,
                                   {4, 0, 4},
                                   {3, 1, 3}},
                                  {1281545.9042,
                                   14.9896,
                                   -3.4646,
                                   3,
                                   1177.9114,
                                   23,
                                   -34002,
                                   303,
                                   {11, 9, 3},
                                   {11, 8, 4}},
                                  {1281714.0800,
                                   0.5000,
                                   -0.8777,
                                   3,
                                   71.4243,
                                   27,
                                   -34002,
                                   303,
                                   {4, 1, 4},
                                   {3, 0, 3}},
                                  {1283124.9111,
                                   59.9585,
                                   -2.6160,
                                   3,
                                   738.4675,
                                   19,
                                   -34002,
                                   303,
                                   {9, 6, 4},
                                   {9, 5, 5}},
                                  {1284386.9600,
                                   0.5000,
                                   -1.8670,
                                   3,
                                   567.8815,
                                   51,
                                   -34002,
                                   303,
                                   {8, 5, 4},
                                   {8, 4, 5}},
                                  {1289633.4431,
                                   0.4448,
                                   -3.5933,
                                   3,
                                   1051.5043,
                                   21,
                                   34002,
                                   303,
                                   {10, 10, 0},
                                   {10, 9, 1}},
                                  {1293504.9252,
                                   59.9585,
                                   -2.1323,
                                   3,
                                   415.5219,
                                   15,
                                   -34002,
                                   303,
                                   {7, 4, 4},
                                   {7, 3, 5}},
                                  {1293969.6035,
                                   14.9896,
                                   -3.1982,
                                   3,
                                   1222.3933,
                                   69,
                                   -34002,
                                   303,
                                   {11, 10, 1},
                                   {11, 9, 2}},
                                  {1295453.8759,
                                   59.9585,
                                   -2.4591,
                                   3,
                                   927.1022,
                                   63,
                                   -34002,
                                   303,
                                   {10, 7, 4},
                                   {10, 6, 5}},
                                  {1297462.1856,
                                   59.9585,
                                   -1.9082,
                                   3,
                                   15.0901,
                                   5,
                                   -34002,
                                   303,
                                   {2, 2, 0},
                                   {1, 1, 1}},
                                  {1298102.5423,
                                   14.9896,
                                   -3.1114,
                                   3,
                                   1051.2106,
                                   63,
                                   -34002,
                                   303,
                                   {10, 10, 1},
                                   {10, 9, 2}},
                                  {1298798.6604,
                                   59.9585,
                                   -2.0073,
                                   3,
                                   281.5945,
                                   13,
                                   -34002,
                                   303,
                                   {6, 2, 4},
                                   {6, 1, 5}},
                                  {1299662.7857,
                                   0.0735,
                                   -4.6742,
                                   3,
                                   524.3446,
                                   17,
                                   34002,
                                   303,
                                   {8, 3, 5},
                                   {7, 6, 2}},
                                  {1305418.3777,
                                   59.9585,
                                   -1.5270,
                                   3,
                                   281.6023,
                                   39,
                                   -34002,
                                   303,
                                   {6, 3, 4},
                                   {6, 2, 5}},
                                  {1313601.5242,
                                   119.1121,
                                   -6.9088,
                                   3,
                                   3268.1362,
                                   117,
                                   34002,
                                   303,
                                   {19, 13, 6},
                                   {19, 12, 7}},
                                  {1315303.1346,
                                   59.9585,
                                   -1.5564,
                                   3,
                                   166.3435,
                                   33,
                                   -34002,
                                   303,
                                   {5, 1, 4},
                                   {5, 0, 5}},
                                  {1316705.5637,
                                   59.9585,
                                   -2.0328,
                                   3,
                                   166.3442,
                                   11,
                                   -34002,
                                   303,
                                   {5, 2, 4},
                                   {5, 1, 5}},
                                  {1319771.7461,
                                   24.7785,
                                   -5.5011,
                                   3,
                                   2313.6937,
                                   33,
                                   34002,
                                   303,
                                   {16, 10, 6},
                                   {16, 9, 7}},
                                  {1324145.7061,
                                   0.5137,
                                   -3.5935,
                                   3,
                                   1306.0224,
                                   25,
                                   34002,
                                   303,
                                   {12, 7, 5},
                                   {12, 6, 6}},
                                  {1326317.0025,
                                   0.2294,
                                   -3.2986,
                                   3,
                                   1133.6702,
                                   23,
                                   34002,
                                   303,
                                   {11, 8, 4},
                                   {11, 7, 5}},
                                  {1331809.6453,
                                   0.1662,
                                   -5.8962,
                                   3,
                                   1133.6702,
                                   25,
                                   34002,
                                   303,
                                   {12, 4, 8},
                                   {11, 7, 5}},
                                  {1341810.8303,
                                   1.4343,
                                   -3.6484,
                                   3,
                                   1220.6591,
                                   23,
                                   34002,
                                   303,
                                   {11, 10, 2},
                                   {11, 9, 3}},
                                  {1344266.1022,
                                   9.4392,
                                   -5.7040,
                                   3,
                                   2394.2289,
                                   33,
                                   34002,
                                   303,
                                   {16, 12, 4},
                                   {16, 11, 5}},
                                  {1347796.3590,
                                   0.4528,
                                   -6.1165,
                                   3,
                                   906.6886,
                                   69,
                                   34002,
                                   303,
                                   {11, 3, 8},
                                   {12, 2, 11}},
                                  {1347855.4144,
                                   0.4531,
                                   -6.5936,
                                   3,
                                   906.6886,
                                   23,
                                   34002,
                                   303,
                                   {11, 4, 8},
                                   {12, 1, 11}},
                                  {1356628.3258,
                                   59.9585,
                                   -1.5769,
                                   3,
                                   51.1402,
                                   7,
                                   -34002,
                                   303,
                                   {3, 2, 2},
                                   {2, 1, 1}},
                                  {1359329.0669,
                                   220.0918,
                                   -7.9532,
                                   3,
                                   3570.7618,
                                   41,
                                   34002,
                                   303,
                                   {20, 13, 7},
                                   {20, 12, 8}},
                                  {1378081.7738,
                                   59.9585,
                                   -3.2630,
                                   3,
                                   71.4243,
                                   21,
                                   -34002,
                                   303,
                                   {3, 3, 0},
                                   {3, 0, 3}},
                                  {1378404.0507,
                                   59.9585,
                                   -3.2228,
                                   3,
                                   1358.1293,
                                   75,
                                   -34002,
                                   303,
                                   {12, 9, 4},
                                   {12, 8, 5}},
                                  {1378842.4600,
                                   0.5000,
                                   -3.4979,
                                   3,
                                   1265.5556,
                                   69,
                                   -34002,
                                   303,
                                   {11, 11, 0},
                                   {11, 10, 1}},
                                  {1380318.9151,
                                   0.4376,
                                   -6.0656,
                                   3,
                                   469.2461,
                                   17,
                                   34002,
                                   303,
                                   {8, 2, 6},
                                   {9, 1, 9}},
                                  {1380574.4315,
                                   0.4371,
                                   -5.5884,
                                   3,
                                   469.2461,
                                   51,
                                   34002,
                                   303,
                                   {8, 3, 6},
                                   {9, 0, 9}},
                                  {1382435.9595,
                                   14.9896,
                                   -3.3832,
                                   3,
                                   1404.1081,
                                   75,
                                   -34002,
                                   303,
                                   {12, 10, 3},
                                   {12, 9, 4}},
                                  {1382845.0838,
                                   0.4952,
                                   -3.9730,
                                   3,
                                   1265.4171,
                                   23,
                                   34002,
                                   303,
                                   {11, 11, 1},
                                   {11, 10, 2}},
                                  {1392466.0830,
                                   2.2810,
                                   -3.8736,
                                   3,
                                   1652.5842,
                                   81,
                                   34002,
                                   303,
                                   {13, 11, 2},
                                   {13, 10, 3}},
                                  {1396445.9687,
                                   2.1335,
                                   -7.7562,
                                   3,
                                   1311.5489,
                                   75,
                                   34002,
                                   303,
                                   {12, 8, 5},
                                   {11, 11, 0}},
                                  {1406270.1986,
                                   1.9024,
                                   -6.6371,
                                   3,
                                   1750.1938,
                                   93,
                                   34002,
                                   303,
                                   {15, 5, 10},
                                   {14, 8, 7}},
                                  {1406327.6620,
                                   158.6964,
                                   -6.7643,
                                   3,
                                   3221.2261,
                                   117,
                                   34002,
                                   303,
                                   {19, 12, 7},
                                   {19, 11, 8}},
                                  {1414276.3561,
                                   336.2177,
                                   -8.1830,
                                   3,
                                   3933.8410,
                                   129,
                                   34002,
                                   303,
                                   {21, 14, 7},
                                   {21, 13, 8}},
                                  {1414276.3562,
                                   336.2177,
                                   -8.1830,
                                   3,
                                   3933.8410,
                                   129,
                                   34002,
                                   303,
                                   {21, 14, 7},
                                   {21, 13, 8}},
                                  {1415906.1827,
                                   56.6733,
                                   -6.7976,
                                   3,
                                   2976.4738,
                                   37,
                                   34002,
                                   303,
                                   {18, 13, 5},
                                   {18, 12, 6}},
                                  {1422107.8175,
                                   3.8839,
                                   -4.0707,
                                   3,
                                   1451.1036,
                                   25,
                                   34002,
                                   303,
                                   {12, 11, 1},
                                   {12, 10, 2}},
                                  {1422385.7029,
                                   59.9585,
                                   -2.6688,
                                   3,
                                   1082.3184,
                                   69,
                                   -34002,
                                   303,
                                   {11, 6, 5},
                                   {11, 5, 6}},
                                  {1426723.6997,
                                   59.9585,
                                   -3.1442,
                                   3,
                                   196.8021,
                                   11,
                                   -34002,
                                   303,
                                   {5, 3, 3},
                                   {4, 4, 0}},
                                  {1433272.0664,
                                   59.9585,
                                   -3.0637,
                                   3,
                                   277.3376,
                                   39,
                                   -34002,
                                   303,
                                   {6, 3, 4},
                                   {5, 4, 1}},
                                  {1440354.1449,
                                   2.5488,
                                   -7.0805,
                                   3,
                                   1749.0596,
                                   31,
                                   34002,
                                   303,
                                   {15, 6, 10},
                                   {14, 7, 7}},
                                  {1446619.9235,
                                   3.8614,
                                   -3.5810,
                                   3,
                                   1450.2212,
                                   75,
                                   34002,
                                   303,
                                   {12, 11, 2},
                                   {12, 10, 3}},
                                  {1449189.6683,
                                   0.2834,
                                   -5.2981,
                                   3,
                                   1129.7641,
                                   75,
                                   34002,
                                   303,
                                   {12, 5, 8},
                                   {11, 6, 5}},
                                  {1449232.1277,
                                   11.5026,
                                   -4.4149,
                                   3,
                                   2023.8720,
                                   93,
                                   34002,
                                   303,
                                   {15, 9, 6},
                                   {15, 8, 7}},
                                  {1450268.9887,
                                   2.9308,
                                   -4.4085,
                                   3,
                                   1498.5400,
                                   25,
                                   34002,
                                   303,
                                   {12, 12, 0},
                                   {12, 11, 1}},
                                  {1451328.5526,
                                   1.4312,
                                   -4.1394,
                                   3,
                                   1600.5004,
                                   27,
                                   34002,
                                   303,
                                   {13, 10, 4},
                                   {13, 9, 5}},
                                  {1452140.1848,
                                   2.9330,
                                   -3.9305,
                                   3,
                                   1498.4753,
                                   75,
                                   34002,
                                   303,
                                   {12, 12, 1},
                                   {12, 11, 2}},
                                  {1458835.6691,
                                   14.9896,
                                   -3.3349,
                                   3,
                                   148.1406,
                                   9,
                                   -34002,
                                   303,
                                   {4, 4, 0},
                                   {4, 1, 3}},
                                  {1480736.4758,
                                   6.0535,
                                   -4.7365,
                                   3,
                                   2145.5974,
                                   93,
                                   34002,
                                   303,
                                   {15, 12, 3},
                                   {15, 11, 4}},
                                  {1480750.7716,
                                   0.0521,
                                   -6.9099,
                                   3,
                                   227.9450,
                                   33,
                                   34002,
                                   303,
                                   {5, 4, 1},
                                   {6, 1, 6}},
                                  {1488969.6640,
                                   786.8569,
                                   -7.9844,
                                   3,
                                   3273.9627,
                                   37,
                                   34002,
                                   303,
                                   {18, 18, 0},
                                   {18, 17, 1}},
                                  {1488991.8128,
                                   786.8459,
                                   -7.5073,
                                   3,
                                   3273.9619,
                                   111,
                                   34002,
                                   303,
                                   {18, 18, 1},
                                   {18, 17, 2}},
                                  {1489565.7179,
                                   2.5469,
                                   -4.3010,
                                   3,
                                   1648.9115,
                                   27,
                                   34002,
                                   303,
                                   {13, 11, 3},
                                   {13, 10, 4}},
                                  {1493167.9014,
                                   59.9585,
                                   -2.7558,
                                   3,
                                   875.6318,
                                   21,
                                   -34002,
                                   303,
                                   {10, 5, 5},
                                   {10, 4, 6}},
                                  {1496996.8065,
                                   207.6160,
                                   -8.0212,
                                   3,
                                   3616.1042,
                                   41,
                                   34002,
                                   303,
                                   {20, 14, 6},
                                   {20, 13, 7}},
                                  {1500240.2285,
                                   22.6910,
                                   -8.0825,
                                   3,
                                   2490.1554,
                                   111,
                                   34002,
                                   303,
                                   {18, 7, 12},
                                   {17, 8, 9}},
                                  {1503706.8905,
                                   12.0849,
                                   -4.4138,
                                   3,
                                   1750.0944,
                                   81,
                                   34002,
                                   303,
                                   {13, 13, 0},
                                   {13, 12, 1}},
                                  {1504579.7078,
                                   12.0847,
                                   -4.8905,
                                   3,
                                   1750.0642,
                                   27,
                                   34002,
                                   303,
                                   {13, 13, 1},
                                   {13, 12, 2}},
                                  {1520841.9396,
                                   4.9440,
                                   -7.9030,
                                   3,
                                   1498.5400,
                                   27,
                                   34002,
                                   303,
                                   {13, 8, 6},
                                   {12, 11, 1}},
                                  {1522004.6381,
                                   59.9585,
                                   -3.0396,
                                   3,
                                   1307.3605,
                                   75,
                                   -34002,
                                   303,
                                   {12, 8, 5},
                                   {12, 7, 6}},
                                  {1524825.3854,
                                   14.9896,
                                   -3.1051,
                                   3,
                                   1082.8075,
                                   23,
                                   -34002,
                                   303,
                                   {11, 7, 5},
                                   {11, 6, 6}},
                                  {1529568.5975,
                                   100.7549,
                                   -6.5344,
                                   3,
                                   2884.7056,
                                   37,
                                   34002,
                                   303,
                                   {18, 11, 7},
                                   {18, 10, 8}},
                                  {1529829.8595,
                                   411.0961,
                                   -6.8013,
                                   3,
                                   2934.8614,
                                   105,
                                   34002,
                                   303,
                                   {17, 17, 0},
                                   {17, 16, 1}},
                                  {1529874.7286,
                                   411.0856,
                                   -7.2784,
                                   3,
                                   2934.8599,
                                   35,
                                   34002,
                                   303,
                                   {17, 17, 1},
                                   {17, 16, 2}},
                                  {1530817.3830,
                                   9.1470,
                                   -4.0453,
                                   3,
                                   1699.0318,
                                   81,
                                   34002,
                                   303,
                                   {13, 12, 1},
                                   {13, 11, 2}},
                                  {1532570.1002,
                                   0.0909,
                                   -4.5829,
                                   3,
                                   635.4082,
                                   19,
                                   34002,
                                   303,
                                   {9, 4, 6},
                                   {8, 5, 3}},
                                  {1533796.3690,
                                   5.3305,
                                   -7.5639,
                                   3,
                                   1699.0318,
                                   87,
                                   34002,
                                   303,
                                   {14, 8, 7},
                                   {13, 11, 2}},
                                  {1535854.5255,
                                   0.9250,
                                   -3.9675,
                                   3,
                                   1549.2698,
                                   27,
                                   34002,
                                   303,
                                   {13, 9, 5},
                                   {13, 8, 6}},
                                  {1538414.4300,
                                   0.5000,
                                   -2.2605,
                                   3,
                                   875.7862,
                                   63,
                                   -34002,
                                   303,
                                   {10, 6, 5},
                                   {10, 5, 6}},
                                  {1538737.2346,
                                   36.0614,
                                   -5.4205,
                                   3,
                                   2019.8587,
                                   29,
                                   34002,
                                   303,
                                   {14, 14, 0},
                                   {14, 13, 1}},
                                  {1539146.0087,
                                   36.0578,
                                   -4.9432,
                                   3,
                                   2019.8445,
                                   87,
                                   34002,
                                   303,
                                   {14, 14, 1},
                                   {14, 13, 2}},
                                  {1540101.1700,
                                   0.5000,
                                   -1.9509,
                                   3,
                                   686.4885,
                                   57,
                                   -34002,
                                   303,
                                   {9, 4, 5},
                                   {9, 3, 6}},
                                  {1541562.0207,
                                   3.9822,
                                   -4.1407,
                                   3,
                                   1860.8363,
                                   87,
                                   34002,
                                   303,
                                   {14, 11, 4},
                                   {14, 10, 5}},
                                  {1542914.3189,
                                   9.0947,
                                   -4.5167,
                                   3,
                                   1698.5981,
                                   27,
                                   34002,
                                   303,
                                   {13, 12, 2},
                                   {13, 11, 3}},
                                  {1544491.3602,
                                   6.9501,
                                   -4.8119,
                                   3,
                                   1914.2141,
                                   29,
                                   34002,
                                   303,
                                   {14, 12, 2},
                                   {14, 11, 3}},
                                  {1548688.7689,
                                   0.1167,
                                   -4.8996,
                                   3,
                                   556.9065,
                                   17,
                                   34002,
                                   303,
                                   {8, 4, 4},
                                   {7, 7, 1}},
                                  {1551369.7300,
                                   0.5000,
                                   -1.4295,
                                   3,
                                   96.3925,
                                   9,
                                   -34002,
                                   303,
                                   {4, 1, 3},
                                   {3, 2, 2}},
                                  {1551961.7112,
                                   200.9450,
                                   -6.6152,
                                   3,
                                   2612.5902,
                                   33,
                                   34002,
                                   303,
                                   {16, 16, 0},
                                   {16, 15, 1}},
                                  {1552054.1090,
                                   200.9358,
                                   -6.1381,
                                   3,
                                   2612.5870,
                                   99,
                                   34002,
                                   303,
                                   {16, 16, 1},
                                   {16, 15, 2}},
                                  {1552438.0854,
                                   17.0887,
                                   -7.3298,
                                   3,
                                   1648.2046,
                                   93,
                                   34002,
                                   303,
                                   {15, 4, 11},
                                   {16, 3, 14}},
                                  {1552442.7079,
                                   17.0883,
                                   -7.8070,
                                   3,
                                   1648.2046,
                                   31,
                                   34002,
                                   303,
                                   {15, 5, 11},
                                   {16, 2, 14}},
                                  {1554923.4146,
                                   90.2137,
                                   -5.5185,
                                   3,
                                   2307.4749,
                                   93,
                                   34002,
                                   303,
                                   {15, 15, 0},
                                   {15, 14, 1}},
                                  {1555116.5555,
                                   90.2070,
                                   -5.9955,
                                   3,
                                   2307.4682,
                                   31,
                                   34002,
                                   303,
                                   {15, 15, 1},
                                   {15, 14, 2}},
                                  {1557067.2000,
                                   0.5000,
                                   -2.4213,
                                   3,
                                   686.5293,
                                   19,
                                   -34002,
                                   303,
                                   {9, 5, 5},
                                   {9, 4, 6}},
                                  {1559303.7854,
                                   0.1330,
                                   -4.8486,
                                   3,
                                   970.3139,
                                   69,
                                   34002,
                                   303,
                                   {11, 4, 7},
                                   {10, 7, 4}},
                                  {1563818.1666,
                                   18.7637,
                                   -5.7566,
                                   3,
                                   2699.4331,
                                   105,
                                   34002,
                                   303,
                                   {17, 13, 4},
                                   {17, 12, 5}},
                                  {1563892.2399,
                                   14.9896,
                                   -0.6916,
                                   3,
                                   114.1776,
                                   33,
                                   -34002,
                                   303,
                                   {5, 0, 5},
                                   {4, 1, 4}},
                                  {1564076.3125,
                                   14.9896,
                                   -1.1686,
                                   3,
                                   114.1722,
                                   11,
                                   -34002,
                                   303,
                                   {5, 1, 5},
                                   {4, 0, 4}},
                                  {1570821.2606,
                                   3.0165,
                                   -3.9775,
                                   3,
                                   1808.4393,
                                   87,
                                   34002,
                                   303,
                                   {14, 10, 5},
                                   {14, 9, 6}},
                                  {1571153.1000,
                                   0.5000,
                                   -2.1696,
                                   3,
                                   515.2886,
                                   17,
                                   -34002,
                                   303,
                                   {8, 3, 5},
                                   {8, 2, 6}},
                                  {1576441.2524,
                                   14.9896,
                                   -1.6904,
                                   3,
                                   515.2971,
                                   51,
                                   -34002,
                                   303,
                                   {8, 4, 5},
                                   {8, 3, 6}},
                                  {1576646.5879,
                                   4.2522,
                                   -4.3188,
                                   3,
                                   1749.0596,
                                   29,
                                   34002,
                                   303,
                                   {14, 8, 6},
                                   {14, 7, 7}},
                                  {1586775.8386,
                                   491.0066,
                                   -8.0089,
                                   3,
                                   3880.9119,
                                   129,
                                   34002,
                                   303,
                                   {21, 13, 8},
                                   {21, 12, 9}},
                                  {1586775.8388,
                                   491.0066,
                                   -8.0089,
                                   3,
                                   3880.9119,
                                   129,
                                   34002,
                                   303,
                                   {21, 13, 8},
                                   {21, 12, 9}},
                                  {1592668.4186,
                                   14.9896,
                                   -1.5232,
                                   3,
                                   362.3515,
                                   45,
                                   -34002,
                                   303,
                                   {7, 2, 5},
                                   {7, 1, 6}},
                                  {1593971.6164,
                                   14.9896,
                                   -1.9998,
                                   3,
                                   362.3528,
                                   15,
                                   -34002,
                                   303,
                                   {7, 3, 5},
                                   {7, 2, 6}},
                                  {1596749.8338,
                                   0.0636,
                                   -7.1887,
                                   3,
                                   452.4945,
                                   45,
                                   34002,
                                   303,
                                   {7, 5, 2},
                                   {8, 2, 7}},
                                  {1596896.1172,
                                   7.2336,
                                   -4.3101,
                                   3,
                                   1912.2573,
                                   87,
                                   34002,
                                   303,
                                   {14, 12, 3},
                                   {14, 11, 4}},
                                  {1598917.2926,
                                   14.9896,
                                   -2.7937,
                                   3,
                                   243.3434,
                                   33,
                                   -34002,
                                   303,
                                   {5, 5, 0},
                                   {5, 2, 3}},
                                  {1599752.2146,
                                   14.9896,
                                   -0.9035,
                                   3,
                                   95.0563,
                                   27,
                                   -34002,
                                   303,
                                   {4, 2, 3},
                                   {3, 1, 2}},
                                  {1608373.9459,
                                   14.9896,
                                   -1.9912,
                                   3,
                                   227.9449,
                                   13,
                                   -34002,
                                   303,
                                   {6, 1, 5},
                                   {6, 0, 6}},
                                  {1608602.8400,
                                   0.5000,
                                   -1.5140,
                                   3,
                                   227.9450,
                                   39,
                                   -34002,
                                   303,
                                   {6, 2, 5},
                                   {6, 1, 6}},
                                  {1622652.8407,
                                   19.7031,
                                   -5.0252,
                                   3,
                                   1965.7328,
                                   29,
                                   34002,
                                   303,
                                   {14, 13, 1},
                                   {14, 12, 2}},
                                  {1625600.4463,
                                   0.0490,
                                   -7.5895,
                                   3,
                                   362.3515,
                                   39,
                                   34002,
                                   303,
                                   {6, 6, 1},
                                   {7, 1, 6}},
                                  {1628487.5147,
                                   19.6206,
                                   -4.5455,
                                   3,
                                   1965.5240,
                                   87,
                                   34002,
                                   303,
                                   {14, 13, 2},
                                   {14, 12, 3}},
                                  {1628538.3444,
                                   7.7520,
                                   -4.9768,
                                   3,
                                   2084.8554,
                                   31,
                                   34002,
                                   303,
                                   {15, 11, 5},
                                   {15, 10, 6}},
                                  {1633670.9681,
                                   1.5027,
                                   -6.7800,
                                   3,
                                   1043.5618,
                                   25,
                                   34002,
                                   303,
                                   {12, 3, 9},
                                   {13, 2, 12}},
                                  {1633684.3778,
                                   1.5027,
                                   -6.3029,
                                   3,
                                   1043.5618,
                                   75,
                                   34002,
                                   303,
                                   {12, 4, 9},
                                   {13, 1, 12}},
                                  {1643386.8066,
                                   9.8371,
                                   -5.1369,
                                   3,
                                   2139.1776,
                                   31,
                                   34002,
                                   303,
                                   {15, 12, 4},
                                   {15, 11, 5}},
                                  {1646941.4836,
                                   86.6092,
                                   -6.9172,
                                   3,
                                   3311.9532,
                                   117,
                                   34002,
                                   303,
                                   {19, 14, 5},
                                   {19, 13, 6}},
                                  {1648713.4195,
                                   14.9896,
                                   -1.9080,
                                   3,
                                   115.3406,
                                   9,
                                   -34002,
                                   303,
                                   {4, 2, 2},
                                   {3, 3, 1}},
                                  {1658301.3266,
                                   18.5046,
                                   -5.7381,
                                   3,
                                   2439.0688,
                                   33,
                                   34002,
                                   303,
                                   {16, 13, 3},
                                   {16, 12, 4}},
                                  {1670863.0153,
                                   0.9631,
                                   -6.4613,
                                   3,
                                   1549.2698,
                                   29,
                                   34002,
                                   303,
                                   {14, 5, 9},
                                   {13, 8, 6}},
                                  {1672717.6519,
                                   17.4705,
                                   -4.8537,
                                   3,
                                   2194.9895,
                                   93,
                                   34002,
                                   303,
                                   {15, 13, 2},
                                   {15, 12, 3}},
                                  {1677050.3139,
                                   1.0658,
                                   -5.7118,
                                   3,
                                   568.4724,
                                   57,
                                   34002,
                                   303,
                                   {9, 2, 7},
                                   {10, 1, 10}},
                                  {1677099.5680,
                                   1.0659,
                                   -6.1889,
                                   3,
                                   568.4724,
                                   19,
                                   34002,
                                   303,
                                   {9, 3, 7},
                                   {10, 0, 10}},
                                  {1677643.3916,
                                   29.9792,
                                   -3.3167,
                                   3,
                                   1490.0984,
                                   81,
                                   -34002,
                                   303,
                                   {13, 7, 6},
                                   {13, 6, 7}},
                                  {1679918.5446,
                                   52.2410,
                                   -5.3726,
                                   3,
                                   2561.7063,
                                   105,
                                   34002,
                                   303,
                                   {17, 10, 7},
                                   {17, 9, 8}},
                                  {1680389.3527,
                                   0.0772,
                                   -7.9621,
                                   3,
                                   624.4128,
                                   51,
                                   34002,
                                   303,
                                   {8, 7, 2},
                                   {9, 2, 7}},
                                  {1699512.4057,
                                   39.9532,
                                   -5.0985,
                                   3,
                                   2250.7853,
                                   93,
                                   34002,
                                   303,
                                   {15, 14, 1},
                                   {15, 13, 2}},
                                  {1699549.6702,
                                   17.6553,
                                   -5.3192,
                                   3,
                                   2193.9950,
                                   31,
                                   34002,
                                   303,
                                   {15, 13, 3},
                                   {15, 12, 4}},
                                  {1702290.9239,
                                   39.8441,
                                   -5.5745,
                                   3,
                                   2250.6859,
                                   31,
                                   34002,
                                   303,
                                   {15, 14, 2},
                                   {15, 13, 3}},
                                  {1706818.0826,
                                   0.1212,
                                   -4.8756,
                                   3,
                                   818.6985,
                                   21,
                                   34002,
                                   303,
                                   {10, 4, 6},
                                   {9, 7, 3}},
                                  {1707188.7665,
                                   16.8394,
                                   -5.0576,
                                   3,
                                   2378.5644,
                                   99,
                                   34002,
                                   303,
                                   {16, 12, 5},
                                   {16, 11, 6}},
                                  {1707969.4973,
                                   14.9896,
                                   -1.2967,
                                   3,
                                   58.3689,
                                   7,
                                   -34002,
                                   303,
                                   {3, 3, 1},
                                   {2, 2, 0}},
                                  {1720693.5750,
                                   0.1414,
                                   -4.1758,
                                   3,
                                   680.4646,
                                   57,
                                   34002,
                                   303,
                                   {9, 4, 5},
                                   {8, 7, 2}},
                                  {1731323.7126,
                                   329.2438,
                                   -7.7076,
                                   3,
                                   3513.0111,
                                   41,
                                   34002,
                                   303,
                                   {20, 12, 8},
                                   {20, 11, 9}},
                                  {1745664.8366,
                                   6.8450,
                                   -4.7950,
                                   3,
                                   2026.6263,
                                   31,
                                   34002,
                                   303,
                                   {15, 10, 6},
                                   {15, 9, 7}},
                                  {1746157.9918,
                                   2.1974,
                                   -3.7844,
                                   3,
                                   1750.1938,
                                   87,
                                   34002,
                                   303,
                                   {14, 9, 6},
                                   {14, 8, 7}},
                                  {1749105.8192,
                                   14.9896,
                                   -3.3224,
                                   3,
                                   1247.6784,
                                   25,
                                   -34002,
                                   303,
                                   {12, 6, 6},
                                   {12, 5, 7}},
                                  {1749136.9778,
                                   11.9154,
                                   -8.0334,
                                   3,
                                   1750.0944,
                                   87,
                                   34002,
                                   303,
                                   {14, 9, 6},
                                   {13, 12, 1}},
                                  {1750296.9806,
                                   22.5541,
                                   -5.2208,
                                   3,
                                   2435.5101,
                                   99,
                                   34002,
                                   303,
                                   {16, 13, 4},
                                   {16, 12, 5}},
                                  {1751746.1263,
                                   0.1097,
                                   -7.9059,
                                   3,
                                   742.8529,
                                   57,
                                   34002,
                                   303,
                                   {9, 6, 3},
                                   {10, 3, 8}},
                                  {1752632.6595,
                                   13.2528,
                                   -7.3958,
                                   3,
                                   2254.1506,
                                   105,
                                   34002,
                                   303,
                                   {17, 6, 11},
                                   {16, 9, 8}},
                                  {1758247.1413,
                                   319.2690,
                                   -8.2008,
                                   3,
                                   3981.0162,
                                   129,
                                   34002,
                                   303,
                                   {21, 15, 6},
                                   {21, 14, 7}},
                                  {1758247.1413,
                                   319.2690,
                                   -8.2008,
                                   3,
                                   3981.0162,
                                   129,
                                   34002,
                                   303,
                                   {21, 15, 6},
                                   {21, 14, 7}},
                                  {1761419.1609,
                                   0.7675,
                                   -3.7653,
                                   3,
                                   1490.5152,
                                   27,
                                   34002,
                                   303,
                                   {13, 8, 6},
                                   {13, 7, 7}},
                                  {1762842.8298,
                                   77.3567,
                                   -6.1712,
                                   3,
                                   2553.7881,
                                   33,
                                   34002,
                                   303,
                                   {16, 15, 1},
                                   {16, 14, 2}},
                                  {1764158.8151,
                                   77.2291,
                                   -5.6936,
                                   3,
                                   2553.7410,
                                   99,
                                   34002,
                                   303,
                                   {16, 15, 2},
                                   {16, 14, 3}},
                                  {1765589.6922,
                                   16.5957,
                                   -4.8856,
                                   3,
                                   2319.6707,
                                   99,
                                   34002,
                                   303,
                                   {16, 11, 6},
                                   {16, 10, 7}},
                                  {1765772.2086,
                                   46.4567,
                                   -6.8205,
                                   3,
                                   3023.7034,
                                   37,
                                   34002,
                                   303,
                                   {18, 14, 4},
                                   {18, 13, 5}},
                                  {1767432.7129,
                                   2.1339,
                                   -5.8980,
                                   3,
                                   1546.0586,
                                   87,
                                   34002,
                                   303,
                                   {14, 6, 9},
                                   {13, 7, 6}},
                                  {1780898.0853,
                                   38.7453,
                                   -5.9017,
                                   3,
                                   2494.3838,
                                   33,
                                   34002,
                                   303,
                                   {16, 14, 2},
                                   {16, 13, 3}},
                                  {1783128.1450,
                                   14.5142,
                                   -7.8465,
                                   3,
                                   2253.1366,
                                   35,
                                   34002,
                                   303,
                                   {17, 7, 11},
                                   {16, 8, 8}},
                                  {1784118.4607,
                                   0.2027,
                                   -4.7950,
                                   3,
                                   716.8650,
                                   57,
                                   34002,
                                   303,
                                   {9, 5, 4},
                                   {8, 8, 1}},
                                  {1785165.4557,
                                   14.9896,
                                   -2.8330,
                                   3,
                                   1247.8137,
                                   75,
                                   -34002,
                                   303,
                                   {12, 7, 6},
                                   {12, 6, 7}},
                                  {1794176.3944,
                                   38.7517,
                                   -5.4192,
                                   3,
                                   2493.8937,
                                   99,
                                   34002,
                                   303,
                                   {16, 14, 3},
                                   {16, 13, 4}},
                                  {1798505.6205,
                                   14.9896,
                                   -2.4284,
                                   3,
                                   1022.3267,
                                   69,
                                   -34002,
                                   303,
                                   {11, 5, 6},
                                   {11, 4, 7}},
                                  {1801937.1164,
                                   33.8169,
                                   -6.1296,
                                   3,
                                   2689.6302,
                                   35,
                                   34002,
                                   303,
                                   {17, 13, 5},
                                   {17, 12, 6}},
                                  {1807737.6574,
                                   45.1108,
                                   -5.8443,
                                   3,
                                   2751.5965,
                                   105,
                                   34002,
                                   303,
                                   {17, 14, 3},
                                   {17, 13, 4}},
                                  {1808683.5744,
                                   14.9896,
                                   -3.4268,
                                   3,
                                   356.5151,
                                   13,
                                   -34002,
                                   303,
                                   {6, 6, 0},
                                   {6, 3, 3}},
                                  {1809239.8473,
                                   33.6578,
                                   -5.9634,
                                   3,
                                   2629.2805,
                                   35,
                                   34002,
                                   303,
                                   {17, 12, 6},
                                   {17, 11, 7}},
                                  {1812030.1576,
                                   14.9896,
                                   -2.9010,
                                   3,
                                   1022.3647,
                                   23,
                                   -34002,
                                   303,
                                   {11, 6, 6},
                                   {11, 5, 7}},
                                  {1814009.7046,
                                   143.7390,
                                   -6.3327,
                                   3,
                                   2874.3526,
                                   105,
                                   34002,
                                   303,
                                   {17, 16, 1},
                                   {17, 15, 2}},
                                  {1814633.0375,
                                   143.6032,
                                   -6.8096,
                                   3,
                                   2874.3302,
                                   35,
                                   34002,
                                   303,
                                   {17, 16, 2},
                                   {17, 15, 3}},
                                  {1815454.4363,
                                   22.1062,
                                   -5.2034,
                                   3,
                                   2253.1366,
                                   33,
                                   34002,
                                   303,
                                   {16, 9, 7},
                                   {16, 8, 8}},
                                  {1821230.1506,
                                   38.6267,
                                   -8.0888,
                                   3,
                                   1831.3448,
                                   33,
                                   34002,
                                   303,
                                   {16, 4, 12},
                                   {17, 3, 15}},
                                  {1821231.3925,
                                   38.6264,
                                   -7.6117,
                                   3,
                                   1831.3448,
                                   99,
                                   34002,
                                   303,
                                   {16, 5, 12},
                                   {17, 2, 15}},
                                  {1833978.8629,
                                   14.9896,
                                   -2.5460,
                                   3,
                                   814.4569,
                                   21,
                                   -34002,
                                   303,
                                   {10, 4, 6},
                                   {10, 3, 7}},
                                  {1838340.2435,
                                   14.9896,
                                   -2.0674,
                                   3,
                                   814.4658,
                                   63,
                                   -34002,
                                   303,
                                   {10, 5, 6},
                                   {10, 4, 7}},
                                  {1842660.1784,
                                   0.1297,
                                   -5.0390,
                                   3,
                                   960.9002,
                                   23,
                                   34002,
                                   303,
                                   {11, 5, 7},
                                   {10, 6, 4}},
                                  {1846759.0153,
                                   14.9896,
                                   -0.5764,
                                   3,
                                   166.3435,
                                   39,
                                   -34002,
                                   303,
                                   {6, 1, 6},
                                   {5, 0, 5}},
                                  {1846759.0153,
                                   14.9896,
                                   -1.0536,
                                   3,
                                   166.3442,
                                   13,
                                   -34002,
                                   303,
                                   {6, 0, 6},
                                   {5, 1, 5}},
                                  {1852685.6124,
                                   14.9896,
                                   -0.7498,
                                   3,
                                   148.4183,
                                   33,
                                   -34002,
                                   303,
                                   {5, 1, 4},
                                   {4, 2, 3}},
                                  {1854603.3817,
                                   256.4256,
                                   -7.4898,
                                   3,
                                   3212.0998,
                                   37,
                                   34002,
                                   303,
                                   {18, 17, 1},
                                   {18, 16, 2}},
                                  {1854899.8916,
                                   256.2919,
                                   -7.0126,
                                   3,
                                   3212.0891,
                                   111,
                                   34002,
                                   303,
                                   {18, 17, 2},
                                   {18, 16, 3}},
                                  {1856398.5385,
                                   48.5320,
                                   -6.3018,
                                   3,
                                   2749.7364,
                                   35,
                                   34002,
                                   303,
                                   {17, 14, 4},
                                   {17, 13, 5}},
                                  {1860980.8698,
                                   14.9896,
                                   -1.7749,
                                   3,
                                   624.4128,
                                   57,
                                   -34002,
                                   303,
                                   {9, 3, 6},
                                   {9, 2, 7}},
                                  {1862156.3560,
                                   14.9896,
                                   -2.2517,
                                   3,
                                   624.4145,
                                   19,
                                   -34002,
                                   303,
                                   {9, 4, 6},
                                   {9, 3, 7}},
                                  {1862436.3621,
                                   14.9896,
                                   -1.2202,
                                   3,
                                   148.1406,
                                   11,
                                   -34002,
                                   303,
                                   {5, 2, 4},
                                   {4, 1, 3}},
                                  {1862465.3861,
                                   0.0906,
                                   -3.3206,
                                   3,
                                   505.7564,
                                   51,
                                   34002,
                                   303,
                                   {8, 4, 5},
                                   {7, 5, 2}},
                                  {1865620.4578,
                                   14.9896,
                                   -0.8558,
                                   3,
                                   55.1616,
                                   21,
                                   -34002,
                                   303,
                                   {3, 3, 0},
                                   {2, 2, 1}},
                                  {1870119.6055,
                                   134.5405,
                                   -8.0428,
                                   3,
                                   3666.0386,
                                   41,
                                   34002,
                                   303,
                                   {20, 15, 5},
                                   {20, 14, 6}},
                                  {1872398.7036,
                                   78.9363,
                                   -6.0429,
                                   3,
                                   2811.8961,
                                   105,
                                   34002,
                                   303,
                                   {17, 15, 2},
                                   {17, 14, 3}},
                                  {1876423.3823,
                                   60.9247,
                                   -6.1200,
                                   3,
                                   2955.4923,
                                   111,
                                   34002,
                                   303,
                                   {18, 13, 6},
                                   {18, 12, 7}},
                                  {1878831.2012,
                                   78.7478,
                                   -6.5177,
                                   3,
                                   2811.6592,
                                   35,
                                   34002,
                                   303,
                                   {17, 15, 3},
                                   {17, 14, 4}},
                                  {1879356.6485,
                                   14.9896,
                                   -2.4901,
                                   3,
                                   296.6776,
                                   39,
                                   -34002,
                                   303,
                                   {6, 4, 3},
                                   {5, 5, 0}},
                                  {1882523.6560,
                                   14.9896,
                                   -2.0455,
                                   3,
                                   452.4943,
                                   17,
                                   -34002,
                                   303,
                                   {8, 2, 6},
                                   {8, 1, 7}},
                                  {1882773.0100,
                                   0.5000,
                                   -1.5683,
                                   3,
                                   452.4945,
                                   51,
                                   -34002,
                                   303,
                                   {8, 3, 6},
                                   {8, 2, 7}},
                                  {1886664.5382,
                                   438.7408,
                                   -7.7324,
                                   3,
                                   3566.6632,
                                   117,
                                   34002,
                                   303,
                                   {19, 18, 1},
                                   {19, 17, 2}},
                                  {1897045.8521,
                                   181.5588,
                                   -6.4754,
                                   3,
                                   3157.9475,
                                   117,
                                   34002,
                                   303,
                                   {19, 11, 8},
                                   {19, 10, 9}},
                                  {1900140.8700,
                                   0.5000,
                                   -1.5220,
                                   3,
                                   298.9696,
                                   45,
                                   -34002,
                                   303,
                                   {7, 1, 6},
                                   {7, 0, 7}},
                                  {1900177.2800,
                                   0.5000,
                                   -1.9991,
                                   3,
                                   298.9696,
                                   15,
                                   -34002,
                                   303,
                                   {7, 2, 6},
                                   {7, 1, 7}},
                                  {1906244.7110,
                                   66.0619,
                                   -6.2862,
                                   3,
                                   3018.0831,
                                   111,
                                   34002,
                                   303,
                                   {18, 14, 5},
                                   {18, 13, 6}},
                                  {1910891.2683,
                                   0.5708,
                                   -5.3810,
                                   3,
                                   1358.1293,
                                   81,
                                   34002,
                                   303,
                                   {13, 5, 8},
                                   {12, 8, 5}},
                                  {1916934.6216,
                                   4.1628,
                                   -6.5266,
                                   3,
                                   1189.7267,
                                   81,
                                   34002,
                                   303,
                                   {13, 3, 10},
                                   {14, 2, 13}},
                                  {1916937.7673,
                                   4.1627,
                                   -7.0037,
                                   3,
                                   1189.7267,
                                   27,
                                   34002,
                                   303,
                                   {13, 4, 10},
                                   {14, 1, 13}},
                                  {1918675.4911,
                                   7.7709,
                                   -4.1275,
                                   3,
                                   1959.8719,
                                   93,
                                   34002,
                                   303,
                                   {15, 8, 7},
                                   {15, 7, 8}},
                                  {1932655.2701,
                                   95.9691,
                                   -6.9575,
                                   3,
                                   3082.6032,
                                   37,
                                   34002,
                                   303,
                                   {18, 15, 3},
                                   {18, 14, 4}},
                                  {1936989.7747,
                                   108.1366,
                                   -6.9886,
                                   3,
                                   3366.8893,
                                   117,
                                   34002,
                                   303,
                                   {19, 15, 4},
                                   {19, 14, 5}},
                                  {1940522.5594,
                                   0.4470,
                                   -5.8309,
                                   3,
                                   896.1713,
                                   21,
                                   34002,
                                   303,
                                   {10, 6, 4},
                                   {9, 9, 1}},
                                  {1949555.2123,
                                   151.6825,
                                   -7.1829,
                                   3,
                                   3147.0696,
                                   37,
                                   34002,
                                   303,
                                   {18, 16, 2},
                                   {18, 15, 3}},
                                  {1951659.5280,
                                   33.0323,
                                   -5.7708,
                                   3,
                                   2564.1801,
                                   35,
                                   34002,
                                   303,
                                   {17, 11, 7},
                                   {17, 10, 8}},
                                  {1952632.8892,
                                   151.3344,
                                   -6.7047,
                                   3,
                                   3146.9563,
                                   111,
                                   34002,
                                   303,
                                   {18, 16, 3},
                                   {18, 15, 4}},
                                  {1957276.4708,
                                   98.2375,
                                   -6.4713,
                                   3,
                                   3081.6686,
                                   111,
                                   34002,
                                   303,
                                   {18, 15, 4},
                                   {18, 14, 5}},
                                  {1958646.9043,
                                   66.1852,
                                   -5.9377,
                                   3,
                                   2890.1589,
                                   111,
                                   34002,
                                   303,
                                   {18, 12, 7},
                                   {18, 11, 8}},
                                  {1963384.3668,
                                   104.2887,
                                   -7.2650,
                                   3,
                                   3298.3873,
                                   39,
                                   34002,
                                   303,
                                   {19, 14, 6},
                                   {19, 13, 7}},
                                  {1964244.0709,
                                   14.0589,
                                   -4.6824,
                                   3,
                                   2254.1506,
                                   99,
                                   34002,
                                   303,
                                   {16, 10, 7},
                                   {16, 9, 8}},
                                  {1972074.0856,
                                   2.6405,
                                   -6.3496,
                                   3,
                                   677.0713,
                                   21,
                                   34002,
                                   303,
                                   {10, 2, 8},
                                   {11, 1, 11}},
                                  {1972083.8661,
                                   2.6407,
                                   -5.8725,
                                   3,
                                   677.0713,
                                   63,
                                   34002,
                                   303,
                                   {10, 3, 8},
                                   {11, 0, 11}},
                                  {1989806.3947,
                                   117.5305,
                                   -7.0899,
                                   3,
                                   3232.0145,
                                   39,
                                   34002,
                                   303,
                                   {19, 13, 7},
                                   {19, 12, 8}},
                                  {1989986.4302,
                                   4.8645,
                                   -4.5836,
                                   3,
                                   1960.2475,
                                   31,
                                   34002,
                                   303,
                                   {15, 9, 7},
                                   {15, 8, 8}},
                                  {1992253.4006,
                                   2.5094,
                                   -4.0552,
                                   3,
                                   1682.6052,
                                   29,
                                   34002,
                                   303,
                                   {14, 7, 7},
                                   {14, 6, 8}},
                                  {1996588.3906,
                                   14.9896,
                                   -0.8336,
                                   3,
                                   107.3682,
                                   27,
                                   -34002,
                                   303,
                                   {4, 3, 2},
                                   {3, 2, 1}},
                                  {2002046.1123,
                                   14.9896,
                                   -3.1425,
                                   3,
                                   391.8876,
                                   15,
                                   -34002,
                                   303,
                                   {7, 4, 4},
                                   {6, 5, 1}},
                                  {2007704.6950,
                                   14.9896,
                                   -3.2824,
                                   3,
                                   324.9179,
                                   13,
                                   -34002,
                                   303,
                                   {6, 5, 1},
                                   {6, 2, 4}},
                                  {2012214.1731,
                                   14.9896,
                                   -3.0377,
                                   3,
                                   210.2173,
                                   33,
                                   -34002,
                                   303,
                                   {5, 4, 1},
                                   {5, 1, 4}},
                                  {2013515.8316,
                                   126.7875,
                                   -7.4369,
                                   3,
                                   3363.8788,
                                   39,
                                   34002,
                                   303,
                                   {19, 15, 5},
                                   {19, 14, 6}},
                                  {2013967.0329,
                                   280.1486,
                                   -7.4108,
                                   3,
                                   3499.4845,
                                   117,
                                   34002,
                                   303,
                                   {19, 17, 2},
                                   {19, 16, 3}},
                                  {2014209.0399,
                                   7.5094,
                                   -7.2034,
                                   3,
                                   2026.6263,
                                   33,
                                   34002,
                                   303,
                                   {16, 6, 10},
                                   {15, 9, 7}},
                                  {2015430.8405,
                                   279.7027,
                                   -7.8875,
                                   3,
                                   3499.4305,
                                   39,
                                   34002,
                                   303,
                                   {19, 17, 3},
                                   {19, 16, 4}},
                                  {2022512.3462,
                                   1.5502,
                                   -3.5693,
                                   3,
                                   1682.7300,
                                   87,
                                   34002,
                                   303,
                                   {14, 8, 7},
                                   {14, 7, 8}},
                                  {2027444.2135,
                                   1.0715,
                                   -6.0554,
                                   3,
                                   1094.5108,
                                   69,
                                   34002,
                                   303,
                                   {11, 7, 4},
                                   {10, 10, 1}},
                                  {2038114.3523,
                                   187.0410,
                                   -7.1642,
                                   3,
                                   3431.5003,
                                   117,
                                   34002,
                                   303,
                                   {19, 16, 3},
                                   {19, 15, 4}},
                                  {2038790.4867,
                                   84.7045,
                                   -6.2368,
                                   3,
                                   2816.6988,
                                   37,
                                   34002,
                                   303,
                                   {18, 10, 8},
                                   {18, 9, 9}},
                                  {2040912.7055,
                                   14.9896,
                                   -1.6004,
                                   3,
                                   195.6614,
                                   33,
                                   -34002,
                                   303,
                                   {5, 3, 2},
                                   {4, 4, 1}},
                                  {2045443.7687,
                                   14.9896,
                                   -3.0777,
                                   3,
                                   1421.8697,
                                   81,
                                   -34002,
                                   303,
                                   {13, 6, 7},
                                   {13, 5, 8}},
                                  {2046360.6759,
                                   191.0694,
                                   -7.3187,
                                   3,
                                   3589.7768,
                                   123,
                                   34002,
                                   303,
                                   {20, 14, 7},
                                   {20, 13, 8}},
                                  {2050222.6127,
                                   188.0751,
                                   -7.6372,
                                   3,
                                   3431.0424,
                                   39,
                                   34002,
                                   303,
                                   {19, 16, 4},
                                   {19, 15, 5}},
                                  {2052867.8552,
                                   0.0545,
                                   -4.0449,
                                   3,
                                   114.1722,
                                   9,
                                   34002,
                                   303,
                                   {4, 3, 1},
                                   {4, 0, 4}},
                                  {2056839.5064,
                                   0.8405,
                                   -3.5516,
                                   3,
                                   1421.9064,
                                   27,
                                   34002,
                                   303,
                                   {13, 7, 7},
                                   {13, 6, 8}},
                                  {2061786.6548,
                                   14.9896,
                                   -2.7889,
                                   3,
                                   457.8796,
                                   45,
                                   -34002,
                                   303,
                                   {7, 6, 1},
                                   {7, 3, 4}},
                                  {2063893.4567,
                                   177.3776,
                                   -7.4915,
                                   3,
                                   3658.0360,
                                   123,
                                   34002,
                                   303,
                                   {20, 15, 6},
                                   {20, 14, 7}},
                                  {2067602.3452,
                                   2.2710,
                                   -7.3433,
                                   3,
                                   1311.5438,
                                   25,
                                   34002,
                                   303,
                                   {12, 8, 4},
                                   {11, 11, 1}},
                                  {2070680.8974,
                                   14.9896,
                                   -1.5132,
                                   3,
                                   38.2978,
                                   21,
                                   -34002,
                                   303,
                                   {3, 2, 1},
                                   {2, 1, 2}},
                                  {2077350.3433,
                                   0.2922,
                                   -4.8485,
                                   3,
                                   856.1456,
                                   21,
                                   34002,
                                   303,
                                   {10, 5, 5},
                                   {9, 8, 2}},
                                  {2079846.4523,
                                   14.9896,
                                   -0.9241,
                                   3,
                                   173.9673,
                                   33,
                                   -34002,
                                   303,
                                   {5, 2, 3},
                                   {4, 3, 2}},
                                  {2086073.4414,
                                   14.9896,
                                   -3.1046,
                                   3,
                                   1178.0946,
                                   25,
                                   -34002,
                                   303,
                                   {12, 5, 7},
                                   {12, 4, 8}},
                                  {2086438.0784,
                                   80.2276,
                                   -7.9226,
                                   3,
                                   2023.6312,
                                   105,
                                   34002,
                                   303,
                                   {17, 4, 13},
                                   {18, 3, 16}},
                                  {2089849.0276,
                                   14.9896,
                                   -2.6264,
                                   3,
                                   1178.1039,
                                   75,
                                   -34002,
                                   303,
                                   {12, 6, 7},
                                   {12, 5, 8}},
                                  {2091564.9803,
                                   0.3894,
                                   -5.3336,
                                   3,
                                   1177.9114,
                                   25,
                                   34002,
                                   303,
                                   {12, 5, 7},
                                   {11, 8, 4}},
                                  {2092393.9658,
                                   14.9896,
                                   -3.2644,
                                   3,
                                   487.2351,
                                   45,
                                   -34002,
                                   303,
                                   {7, 7, 0},
                                   {7, 4, 3}},
                                  {2092396.2460,
                                   5.3888,
                                   -7.7869,
                                   3,
                                   1546.9134,
                                   81,
                                   34002,
                                   303,
                                   {13, 9, 4},
                                   {12, 12, 1}},
                                  {2096987.8915,
                                   530.1343,
                                   -7.7149,
                                   3,
                                   3810.9639,
                                   129,
                                   34002,
                                   303,
                                   {21, 12, 9},
                                   {21, 11, 10}},
                                  {2096987.8916,
                                   530.1343,
                                   -7.7149,
                                   3,
                                   3810.9639,
                                   129,
                                   34002,
                                   303,
                                   {21, 12, 9},
                                   {21, 11, 10}},
                                  {2097109.0280,
                                   10.8963,
                                   -6.6600,
                                   3,
                                   2023.8720,
                                   99,
                                   34002,
                                   303,
                                   {16, 7, 10},
                                   {15, 8, 7}},
                                  {2118416.4068,
                                   236.7274,
                                   -7.6737,
                                   3,
                                   3726.8801,
                                   123,
                                   34002,
                                   303,
                                   {20, 16, 5},
                                   {20, 15, 6}},
                                  {2118945.9842,
                                   14.9896,
                                   -2.2317,
                                   3,
                                   951.6463,
                                   69,
                                   -34002,
                                   303,
                                   {11, 4, 7},
                                   {11, 3, 8}},
                                  {2120027.6354,
                                   14.9896,
                                   -2.7086,
                                   3,
                                   951.6482,
                                   23,
                                   -34002,
                                   303,
                                   {11, 5, 7},
                                   {11, 4, 8}},
                                  {2129265.1404,
                                   14.9896,
                                   -0.5168,
                                   3,
                                   227.9450,
                                   45,
                                   -34002,
                                   303,
                                   {7, 0, 7},
                                   {6, 1, 6}},
                                  {2129265.1404,
                                   14.9896,
                                   -0.9940,
                                   3,
                                   227.9449,
                                   15,
                                   -34002,
                                   303,
                                   {7, 1, 7},
                                   {6, 0, 6}},
                                  {2134047.8617,
                                   343.3102,
                                   -7.8899,
                                   3,
                                   3797.5429,
                                   123,
                                   34002,
                                   303,
                                   {20, 17, 4},
                                   {20, 16, 5}},
                                  {2138413.3073,
                                   14.9896,
                                   -1.1258,
                                   3,
                                   210.2648,
                                   13,
                                   -34002,
                                   303,
                                   {6, 1, 5},
                                   {5, 2, 4}},
                                  {2140067.2623,
                                   14.9896,
                                   -0.6477,
                                   3,
                                   210.2173,
                                   39,
                                   -34002,
                                   303,
                                   {6, 2, 5},
                                   {5, 1, 4}},
                                  {2145043.2342,
                                   34.4344,
                                   -5.0894,
                                   3,
                                   2490.1554,
                                   105,
                                   34002,
                                   303,
                                   {17, 9, 8},
                                   {17, 8, 9}},
                                  {2146643.5096,
                                   14.9896,
                                   -2.3772,
                                   3,
                                   742.8526,
                                   21,
                                   -34002,
                                   303,
                                   {10, 3, 7},
                                   {10, 2, 8}},
                                  {2146900.4318,
                                   14.9896,
                                   -1.9000,
                                   3,
                                   742.8529,
                                   63,
                                   -34002,
                                   303,
                                   {10, 4, 7},
                                   {10, 3, 8}},
                                  {2146923.1800,
                                   210.8263,
                                   -7.1259,
                                   3,
                                   3518.1632,
                                   123,
                                   34002,
                                   303,
                                   {20, 13, 8},
                                   {20, 12, 9}},
                                  {2149970.7720,
                                   1.1720,
                                   -5.6523,
                                   3,
                                   1350.1911,
                                   27,
                                   34002,
                                   303,
                                   {13, 6, 8},
                                   {12, 7, 5}},
                                  {2150900.9210,
                                   118.0793,
                                   -6.8867,
                                   3,
                                   3160.2682,
                                   39,
                                   34002,
                                   303,
                                   {19, 12, 8},
                                   {19, 11, 9}},
                                  {2164708.5565,
                                   0.3208,
                                   -4.4766,
                                   3,
                                   1010.1115,
                                   69,
                                   34002,
                                   303,
                                   {11, 5, 6},
                                   {10, 8, 3}},
                                  {2170467.1169,
                                   14.9896,
                                   -1.6554,
                                   3,
                                   552.0138,
                                   57,
                                   -34002,
                                   303,
                                   {9, 2, 7},
                                   {9, 1, 8}},
                                  {2170516.8824,
                                   14.9896,
                                   -2.1325,
                                   3,
                                   552.0139,
                                   19,
                                   -34002,
                                   303,
                                   {9, 3, 7},
                                   {9, 2, 8}},
                                  {2173730.9784,
                                   59.3716,
                                   -5.7245,
                                   3,
                                   2817.6511,
                                   111,
                                   34002,
                                   303,
                                   {18, 11, 8},
                                   {18, 10, 9}},
                                  {2191097.6347,
                                   14.9896,
                                   -1.5710,
                                   3,
                                   379.4073,
                                   51,
                                   -34002,
                                   303,
                                   {8, 2, 7},
                                   {8, 1, 8}},
                                  {2191097.6347,
                                   14.9896,
                                   -2.0481,
                                   3,
                                   379.4073,
                                   17,
                                   -34002,
                                   303,
                                   {8, 1, 7},
                                   {8, 0, 8}},
                                  {2192124.1240,
                                   14.9896,
                                   -3.4359,
                                   3,
                                   608.5652,
                                   17,
                                   -34002,
                                   303,
                                   {8, 7, 1},
                                   {8, 4, 4}},
                                  {2197627.9539,
                                   10.6154,
                                   -7.2567,
                                   3,
                                   1345.1614,
                                   29,
                                   34002,
                                   303,
                                   {14, 3, 11},
                                   {15, 2, 14}},
                                  {2197628.7181,
                                   10.6153,
                                   -6.7796,
                                   3,
                                   1345.1614,
                                   87,
                                   34002,
                                   303,
                                   {14, 4, 11},
                                   {15, 1, 14}},
                                  {2208520.8367,
                                   26.1694,
                                   -5.5502,
                                   3,
                                   2490.5118,
                                   35,
                                   34002,
                                   303,
                                   {17, 10, 8},
                                   {17, 9, 9}},
                                  {2220165.5114,
                                   14.9896,
                                   -1.2708,
                                   3,
                                   170.3358,
                                   11,
                                   -34002,
                                   303,
                                   {5, 3, 3},
                                   {4, 2, 2}},
                                  {2222153.4285,
                                   12.1646,
                                   -4.9420,
                                   3,
                                   2179.0136,
                                   33,
                                   34002,
                                   303,
                                   {16, 8, 8},
                                   {16, 7, 9}},
                                  {2233493.3783,
                                   0.1627,
                                   -3.7879,
                                   3,
                                   801.2849,
                                   63,
                                   34002,
                                   303,
                                   {10, 5, 6},
                                   {9, 6, 3}},
                                  {2246507.6912,
                                   266.1791,
                                   -7.4097,
                                   3,
                                   3438.0757,
                                   41,
                                   34002,
                                   303,
                                   {20, 11, 9},
                                   {20, 10, 10}},
                                  {2248919.2907,
                                   9.7895,
                                   -4.4581,
                                   3,
                                   2179.1347,
                                   99,
                                   34002,
                                   303,
                                   {16, 9, 8},
                                   {16, 8, 9}},
                                  {2259705.5397,
                                   4.0922,
                                   -6.0909,
                                   3,
                                   1808.4393,
                                   93,
                                   34002,
                                   303,
                                   {15, 6, 9},
                                   {14, 9, 6}},
                                  {2265533.5503,
                                   6.3736,
                                   -6.0633,
                                   3,
                                   795.0260,
                                   69,
                                   34002,
                                   303,
                                   {11, 2, 9},
                                   {12, 1, 12}},
                                  {2265535.5695,
                                   6.3737,
                                   -6.5404,
                                   3,
                                   795.0260,
                                   23,
                                   34002,
                                   303,
                                   {11, 3, 9},
                                   {12, 0, 12}},
                                  {2280127.2805,
                                   3.7551,
                                   -3.8853,
                                   3,
                                   1883.8150,
                                   93,
                                   34002,
                                   303,
                                   {15, 7, 8},
                                   {15, 6, 9}},
                                  {2290278.8603,
                                   3.1699,
                                   -4.3599,
                                   3,
                                   1883.8520,
                                   31,
                                   34002,
                                   303,
                                   {15, 8, 8},
                                   {15, 7, 9}},
                                  {2294077.5432,
                                   14.9896,
                                   -2.9923,
                                   3,
                                   416.8462,
                                   15,
                                   -34002,
                                   303,
                                   {7, 5, 3},
                                   {6, 6, 0}},
                                  {2318132.5902,
                                   14.9896,
                                   -2.2869,
                                   3,
                                   38.0161,
                                   7,
                                   -34002,
                                   303,
                                   {3, 3, 1},
                                   {2, 0, 2}},
                                  {2326430.0422,
                                   1.3710,
                                   -3.8281,
                                   3,
                                   1605.0038,
                                   29,
                                   34002,
                                   303,
                                   {14, 6, 8},
                                   {14, 5, 9}},
                                  {2329876.0604,
                                   14.9896,
                                   -3.3501,
                                   3,
                                   1605.0138,
                                   87,
                                   -34002,
                                   303,
                                   {14, 7, 8},
                                   {14, 6, 9}},
                                  {2346458.1808,
                                   14.9896,
                                   -0.5480,
                                   3,
                                   117.3920,
                                   27,
                                   -34002,
                                   303,
                                   {4, 4, 1},
                                   {3, 3, 0}},
                                  {2354975.9752,
                                   0.8359,
                                   -4.7618,
                                   3,
                                   1051.2106,
                                   69,
                                   34002,
                                   303,
                                   {11, 6, 5},
                                   {10, 9, 2}},
                                  {2356764.3440,
                                   122.2989,
                                   -6.1933,
                                   3,
                                   3079.3343,
                                   117,
                                   34002,
                                   303,
                                   {19, 10, 9},
                                   {19, 9, 10}},
                                  {2365451.8318,
                                   14.9896,
                                   -2.8646,
                                   3,
                                   1342.9668,
                                   81,
                                   -34002,
                                   303,
                                   {13, 5, 8},
                                   {13, 4, 9}},
                                  {2366481.6189,
                                   14.9896,
                                   -3.3414,
                                   3,
                                   1342.9691,
                                   27,
                                   -34002,
                                   303,
                                   {13, 6, 8},
                                   {13, 5, 9}},
                                  {2373025.1357,
                                   196.0022,
                                   -6.9031,
                                   3,
                                   3439.0076,
                                   123,
                                   34002,
                                   303,
                                   {20, 12, 9},
                                   {20, 11, 10}},
                                  {2378146.8432,
                                   14.9896,
                                   -2.3344,
                                   3,
                                   296.1044,
                                   13,
                                   -34002,
                                   303,
                                   {6, 4, 2},
                                   {5, 5, 1}},
                                  {2399520.8463,
                                   14.9896,
                                   -2.9084,
                                   3,
                                   1098.0552,
                                   25,
                                   -34002,
                                   303,
                                   {12, 4, 8},
                                   {12, 3, 9}},
                                  {2399785.5630,
                                   14.9896,
                                   -2.4312,
                                   3,
                                   1098.0557,
                                   75,
                                   -34002,
                                   303,
                                   {12, 5, 8},
                                   {12, 4, 9}},
                                  {2409698.8002,
                                   14.9896,
                                   -3.2889,
                                   3,
                                   776.3768,
                                   57,
                                   -34002,
                                   303,
                                   {9, 8, 1},
                                   {9, 5, 4}},
                                  {2411461.8797,
                                   14.9896,
                                   -0.5030,
                                   3,
                                   298.9696,
                                   51,
                                   -34002,
                                   303,
                                   {8, 1, 8},
                                   {7, 0, 7}},
                                  {2411461.8797,
                                   14.9896,
                                   -0.9802,
                                   3,
                                   298.9696,
                                   17,
                                   -34002,
                                   303,
                                   {8, 0, 8},
                                   {7, 1, 7}},
                                  {2414091.3593,
                                   14.9896,
                                   -1.2312,
                                   3,
                                   244.3925,
                                   13,
                                   -34002,
                                   303,
                                   {6, 2, 4},
                                   {5, 3, 3}},
                                  {2415738.4100,
                                   100.1713,
                                   -6.6569,
                                   3,
                                   3079.6878,
                                   39,
                                   34002,
                                   303,
                                   {19, 11, 9},
                                   {19, 10, 10}},
                                  {2420803.1129,
                                   14.9896,
                                   -0.6088,
                                   3,
                                   281.6023,
                                   45,
                                   -34002,
                                   303,
                                   {7, 1, 6},
                                   {6, 2, 5}},
                                  {2421070.5278,
                                   14.9896,
                                   -1.0858,
                                   3,
                                   281.5945,
                                   15,
                                   -34002,
                                   303,
                                   {7, 2, 6},
                                   {6, 1, 5}},
                                  {2429822.9686,
                                   14.9896,
                                   -2.0615,
                                   3,
                                   870.5961,
                                   69,
                                   -34002,
                                   303,
                                   {11, 3, 8},
                                   {11, 2, 9}},
                                  {2429881.1283,
                                   14.9896,
                                   -2.5386,
                                   3,
                                   870.5962,
                                   23,
                                   -34002,
                                   303,
                                   {11, 4, 8},
                                   {11, 3, 9}},
                                  {2437435.1880,
                                   44.5588,
                                   -7.5704,
                                   3,
                                   2561.7063,
                                   111,
                                   34002,
                                   303,
                                   {18, 8, 11},
                                   {17, 9, 8}},
                                  {2438486.1063,
                                   51.8197,
                                   -5.9737,
                                   3,
                                   2735.3597,
                                   37,
                                   34002,
                                   303,
                                   {18, 9, 9},
                                   {18, 8, 10}},
                                  {2442156.1305,
                                   14.9896,
                                   -1.0263,
                                   3,
                                   115.3406,
                                   9,
                                   -34002,
                                   303,
                                   {4, 4, 0},
                                   {3, 3, 1}},
                                  {2442823.3546,
                                   0.0539,
                                   -4.0509,
                                   3,
                                   114.1776,
                                   27,
                                   34002,
                                   303,
                                   {4, 4, 1},
                                   {4, 1, 4}},
                                  {2443670.6364,
                                   0.1016,
                                   -4.1644,
                                   3,
                                   635.4082,
                                   17,
                                   34002,
                                   303,
                                   {8, 8, 0},
                                   {8, 5, 3}},
                                  {2452389.8458,
                                   14.9896,
                                   -0.7310,
                                   3,
                                   243.3434,
                                   39,
                                   -34002,
                                   303,
                                   {6, 3, 4},
                                   {5, 2, 3}},
                                  {2456983.8655,
                                   14.9896,
                                   -1.7783,
                                   3,
                                   660.8967,
                                   63,
                                   -34002,
                                   303,
                                   {10, 3, 8},
                                   {10, 2, 9}},
                                  {2456983.8655,
                                   14.9896,
                                   -2.2554,
                                   3,
                                   660.8967,
                                   21,
                                   -34002,
                                   303,
                                   {10, 2, 8},
                                   {10, 1, 9}},
                                  {2461438.5320,
                                   2.1679,
                                   -5.9875,
                                   3,
                                   1600.5004,
                                   29,
                                   34002,
                                   303,
                                   {14, 6, 8},
                                   {13, 9, 5}},
                                  {2463352.0126,
                                   46.0010,
                                   -5.4910,
                                   3,
                                   2735.4825,
                                   111,
                                   34002,
                                   303,
                                   {18, 10, 9},
                                   {18, 9, 10}},
                                  {2464326.9348,
                                   7.3306,
                                   -6.4113,
                                   3,
                                   1801.6509,
                                   31,
                                   34002,
                                   303,
                                   {15, 7, 9},
                                   {14, 8, 6}},
                                  {2475740.3320,
                                   25.3796,
                                   -7.0567,
                                   3,
                                   1509.8429,
                                   93,
                                   34002,
                                   303,
                                   {15, 3, 12},
                                   {16, 2, 15}},
                                  {2475740.5242,
                                   25.3795,
                                   -7.5339,
                                   3,
                                   1509.8429,
                                   31,
                                   34002,
                                   303,
                                   {15, 4, 12},
                                   {16, 1, 15}},
                                  {2481316.8201,
                                   14.9896,
                                   -1.6555,
                                   3,
                                   469.2461,
                                   57,
                                   -34002,
                                   303,
                                   {9, 1, 8},
                                   {9, 0, 9}},
                                  {2481316.8201,
                                   14.9896,
                                   -2.1326,
                                   3,
                                   469.2461,
                                   19,
                                   -34002,
                                   303,
                                   {9, 2, 8},
                                   {9, 1, 9}},
                                  {2496468.0100,
                                   0.1656,
                                   -3.5141,
                                   3,
                                   655.1942,
                                   19,
                                   34002,
                                   303,
                                   {9, 5, 5},
                                   {8, 6, 2}},
                                  {2501923.7848,
                                   19.7597,
                                   -4.8413,
                                   3,
                                   2406.7002,
                                   105,
                                   34002,
                                   303,
                                   {17, 8, 9},
                                   {17, 7, 10}},
                                  {2511447.4210,
                                   18.5945,
                                   -5.3164,
                                   3,
                                   2406.7389,
                                   35,
                                   34002,
                                   303,
                                   {17, 9, 9},
                                   {17, 8, 10}},
                                  {2520377.0793,
                                   14.9896,
                                   -2.5242,
                                   3,
                                   526.6533,
                                   51,
                                   -34002,
                                   303,
                                   {8, 5, 4},
                                   {7, 6, 1}},
                                  {2541462.8422,
                                   2.3171,
                                   -5.8129,
                                   3,
                                   1265.4171,
                                   25,
                                   34002,
                                   303,
                                   {12, 7, 5},
                                   {11, 10, 2}},
                                  {2554247.8848,
                                   6.6711,
                                   -4.7049,
                                   3,
                                   2093.8130,
                                   33,
                                   34002,
                                   303,
                                   {16, 7, 9},
                                   {16, 6, 10}},
                                  {2556452.2885,
                                   365.4353,
                                   -7.4310,
                                   3,
                                   3725.6898,
                                   129,
                                   34002,
                                   303,
                                   {21, 11, 10},
                                   {21, 10, 11}},
                                  {2556452.2886,
                                   365.4353,
                                   -7.4310,
                                   3,
                                   3725.6898,
                                   129,
                                   34002,
                                   303,
                                   {21, 11, 10},
                                   {21, 10, 11}},
                                  {2557480.5623,
                                   14.5988,
                                   -6.7562,
                                   3,
                                   922.3182,
                                   25,
                                   34002,
                                   303,
                                   {12, 2, 10},
                                   {13, 1, 13}},
                                  {2557480.9974,
                                   14.5989,
                                   -6.2791,
                                   3,
                                   922.3182,
                                   75,
                                   34002,
                                   303,
                                   {12, 3, 10},
                                   {13, 0, 13}},
                                  {2557549.5912,
                                   6.6318,
                                   -4.2271,
                                   3,
                                   2093.8240,
                                   99,
                                   34002,
                                   303,
                                   {16, 8, 9},
                                   {16, 7, 10}},
                                  {2559127.9664,
                                   0.9837,
                                   -5.2079,
                                   3,
                                   1220.6591,
                                   25,
                                   34002,
                                   303,
                                   {12, 6, 6},
                                   {11, 9, 3}},
                                  {2560498.7300,
                                   0.5000,
                                   -1.5237,
                                   3,
                                   271.1060,
                                   13,
                                   -34002,
                                   303,
                                   {6, 3, 3},
                                   {5, 4, 2}},
                                  {2568462.9942,
                                   0.5397,
                                   -4.4109,
                                   3,
                                   1162.1390,
                                   75,
                                   34002,
                                   303,
                                   {12, 6, 7},
                                   {11, 7, 4}},
                                  {2573406.7932,
                                   0.0623,
                                   -4.2003,
                                   3,
                                   210.2648,
                                   11,
                                   34002,
                                   303,
                                   {5, 5, 1},
                                   {5, 2, 4}},
                                  {2577925.1683,
                                   1.2828,
                                   -5.0292,
                                   3,
                                   1404.1081,
                                   81,
                                   34002,
                                   303,
                                   {13, 6, 7},
                                   {12, 9, 4}},
                                  {2585891.0249,
                                   14.9896,
                                   -1.5019,
                                   3,
                                   96.3925,
                                   9,
                                   -34002,
                                   303,
                                   {4, 3, 1},
                                   {3, 2, 2}},
                                  {2590527.6151,
                                   14.9896,
                                   -3.1468,
                                   3,
                                   737.8607,
                                   57,
                                   -34002,
                                   303,
                                   {9, 7, 2},
                                   {9, 4, 5}},
                                  {2599593.3330,
                                   2.3624,
                                   -3.6576,
                                   3,
                                   1797.1019,
                                   93,
                                   34002,
                                   303,
                                   {15, 6, 9},
                                   {15, 5, 10}},
                                  {2600619.3777,
                                   2.4663,
                                   -4.1345,
                                   3,
                                   1797.1046,
                                   31,
                                   34002,
                                   303,
                                   {15, 7, 9},
                                   {15, 6, 10}},
                                  {2609078.1153,
                                   21.2925,
                                   -6.9574,
                                   3,
                                   2319.6707,
                                   105,
                                   34002,
                                   303,
                                   {17, 7, 10},
                                   {16, 10, 7}},
                                  {2610000.2374,
                                   310.5591,
                                   -7.8967,
                                   3,
                                   3726.0843,
                                   43,
                                   34002,
                                   303,
                                   {21, 12, 10},
                                   {21, 11, 11}},
                                  {2623111.7575,
                                   14.9896,
                                   -3.5773,
                                   3,
                                   567.6967,
                                   17,
                                   -34002,
                                   303,
                                   {8, 6, 2},
                                   {8, 3, 5}},
                                  {2631008.3547,
                                   32.6333,
                                   -7.9375,
                                   3,
                                   2019.8445,
                                   93,
                                   34002,
                                   303,
                                   {15, 10, 5},
                                   {14, 13, 2}},
                                  {2635282.2817,
                                   0.1296,
                                   -3.8215,
                                   3,
                                   925.4386,
                                   21,
                                   34002,
                                   303,
                                   {10, 8, 2},
                                   {10, 5, 5}},
                                  {2636167.8414,
                                   5.7842,
                                   -6.0724,
                                   3,
                                   1498.4753,
                                   81,
                                   34002,
                                   303,
                                   {13, 8, 5},
                                   {12, 11, 2}},
                                  {2640159.1529,
                                   1.1174,
                                   -3.6113,
                                   3,
                                   1516.9376,
                                   29,
                                   34002,
                                   303,
                                   {14, 5, 9},
                                   {14, 4, 10}},
                                  {2640440.3854,
                                   1.1507,
                                   -3.1341,
                                   3,
                                   1516.9382,
                                   87,
                                   34002,
                                   303,
                                   {14, 6, 9},
                                   {14, 5, 10}},
                                  {2641117.6518,
                                   176.6875,
                                   -7.1424,
                                   3,
                                   3349.9775,
                                   41,
                                   34002,
                                   303,
                                   {20, 10, 10},
                                   {20, 9, 11}},
                                  {2651890.9341,
                                   14.9896,
                                   -1.1565,
                                   3,
                                   182.6485,
                                   11,
                                   -34002,
                                   303,
                                   {5, 4, 2},
                                   {4, 3, 1}},
                                  {2655675.9540,
                                   13.8870,
                                   -7.4205,
                                   3,
                                   1750.0642,
                                   29,
                                   34002,
                                   303,
                                   {14, 9, 5},
                                   {13, 12, 2}},
                                  {2665191.9291,
                                   161.8746,
                                   -6.6605,
                                   3,
                                   3350.1063,
                                   123,
                                   34002,
                                   303,
                                   {20, 11, 10},
                                   {20, 10, 11}},
                                  {2673571.6249,
                                   14.9896,
                                   -2.1570,
                                   3,
                                   416.5757,
                                   45,
                                   -34002,
                                   303,
                                   {7, 5, 2},
                                   {6, 6, 1}},
                                  {2677087.8906,
                                   14.9896,
                                   -2.6631,
                                   3,
                                   1253.6687,
                                   81,
                                   -34002,
                                   303,
                                   {13, 4, 9},
                                   {13, 3, 10}},
                                  {2677152.9456,
                                   14.9896,
                                   -3.1402,
                                   3,
                                   1253.6688,
                                   27,
                                   -34002,
                                   303,
                                   {13, 5, 9},
                                   {13, 4, 10}},
                                  {2681025.3648,
                                   14.9896,
                                   -2.6673,
                                   3,
                                   557.0299,
                                   51,
                                   -34002,
                                   303,
                                   {8, 6, 3},
                                   {7, 7, 0}},
                                  {2693299.1678,
                                   14.9896,
                                   -0.5288,
                                   3,
                                   379.4073,
                                   57,
                                   -34002,
                                   303,
                                   {9, 0, 9},
                                   {8, 1, 8}},
                                  {2693299.1678,
                                   14.9896,
                                   -1.0059,
                                   3,
                                   379.4073,
                                   19,
                                   -34002,
                                   303,
                                   {9, 1, 9},
                                   {8, 0, 8}},
                                  {2702381.0805,
                                   14.9896,
                                   -1.0934,
                                   3,
                                   362.3528,
                                   17,
                                   -34002,
                                   303,
                                   {8, 1, 7},
                                   {7, 2, 6}},
                                  {2702424.5504,
                                   14.9896,
                                   -0.6162,
                                   3,
                                   362.3515,
                                   51,
                                   -34002,
                                   303,
                                   {8, 2, 7},
                                   {7, 1, 6}},
                                  {2706499.6293,
                                   14.9896,
                                   -3.1969,
                                   3,
                                   415.4772,
                                   45,
                                   -34002,
                                   303,
                                   {7, 5, 2},
                                   {7, 2, 5}},
                                  {2708052.8540,
                                   14.9896,
                                   -0.7104,
                                   3,
                                   325.1463,
                                   45,
                                   -34002,
                                   303,
                                   {7, 2, 5},
                                   {6, 3, 4}},
                                  {2710397.5537,
                                   80.6748,
                                   -5.9378,
                                   3,
                                   2988.9252,
                                   117,
                                   34002,
                                   303,
                                   {19, 9, 10},
                                   {19, 8, 11}},
                                  {2710992.0193,
                                   14.9896,
                                   -2.2550,
                                   3,
                                   1007.6266,
                                   75,
                                   -34002,
                                   303,
                                   {12, 4, 9},
                                   {12, 3, 10}},
                                  {2710992.0193,
                                   14.9896,
                                   -2.7321,
                                   3,
                                   1007.6266,
                                   25,
                                   -34002,
                                   303,
                                   {12, 3, 9},
                                   {12, 2, 10}},
                                  {2716243.4838,
                                   14.9896,
                                   -1.1836,
                                   3,
                                   324.9179,
                                   15,
                                   -34002,
                                   303,
                                   {7, 3, 5},
                                   {6, 2, 4}},
                                  {2716244.1309,
                                   0.3963,
                                   -4.2255,
                                   3,
                                   960.9002,
                                   21,
                                   34002,
                                   303,
                                   {10, 9, 1},
                                   {10, 6, 4}},
                                  {2719737.6764,
                                   77.6434,
                                   -6.4131,
                                   3,
                                   2988.9672,
                                   39,
                                   34002,
                                   303,
                                   {19, 10, 10},
                                   {19, 9, 11}},
                                  {2740982.8483,
                                   0.0635,
                                   -3.6789,
                                   3,
                                   325.1463,
                                   39,
                                   34002,
                                   303,
                                   {6, 6, 1},
                                   {6, 3, 4}},
                                  {2742165.9380,
                                   14.9896,
                                   -1.9333,
                                   3,
                                   779.1273,
                                   69,
                                   -34002,
                                   303,
                                   {11, 2, 9},
                                   {11, 1, 10}},
                                  {2742165.9380,
                                   14.9896,
                                   -2.4104,
                                   3,
                                   779.1273,
                                   23,
                                   -34002,
                                   303,
                                   {11, 3, 9},
                                   {11, 2, 10}},
                                  {2751229.1893,
                                   56.8998,
                                   -7.8315,
                                   3,
                                   1683.7465,
                                   33,
                                   34002,
                                   303,
                                   {16, 3, 13},
                                   {17, 2, 16}},
                                  {2751229.2393,
                                   56.8998,
                                   -7.3544,
                                   3,
                                   1683.7465,
                                   99,
                                   34002,
                                   303,
                                   {16, 4, 13},
                                   {17, 1, 16}},
                                  {2768934.4683,
                                   34.2557,
                                   -5.7262,
                                   3,
                                   2642.9980,
                                   37,
                                   34002,
                                   303,
                                   {18, 8, 10},
                                   {18, 7, 11}},
                                  {2770810.2080,
                                   29.9792,
                                   -1.7718,
                                   3,
                                   568.4724,
                                   63,
                                   -34002,
                                   303,
                                   {10, 2, 9},
                                   {10, 1, 10}},
                                  {2770810.2080,
                                   59.9585,
                                   -2.2489,
                                   3,
                                   568.4724,
                                   21,
                                   -34002,
                                   303,
                                   {10, 1, 9},
                                   {10, 0, 10}},
                                  {2772243.2890,
                                   34.0206,
                                   -5.2485,
                                   3,
                                   2643.0104,
                                   111,
                                   34002,
                                   303,
                                   {18, 9, 10},
                                   {18, 8, 11}},
                                  {2776954.2017,
                                   0.1513,
                                   -3.6930,
                                   3,
                                   1129.7641,
                                   69,
                                   34002,
                                   303,
                                   {11, 9, 2},
                                   {11, 6, 5}},
                                  {2789426.2183,
                                   31.8162,
                                   -7.3081,
                                   3,
                                   2313.6937,
                                   35,
                                   34002,
                                   303,
                                   {17, 8, 10},
                                   {16, 9, 7}},
                                  {2813140.3493,
                                   0.0811,
                                   -3.9222,
                                   3,
                                   281.5945,
                                   13,
                                   34002,
                                   303,
                                   {6, 4, 2},
                                   {6, 1, 5}},
                                  {2820689.5266,
                                   13.7511,
                                   -4.6003,
                                   3,
                                   2312.6121,
                                   105,
                                   34002,
                                   303,
                                   {17, 7, 10},
                                   {17, 6, 11}},
                                  {2821752.5096,
                                   13.9472,
                                   -5.0772,
                                   3,
                                   2312.6154,
                                   35,
                                   34002,
                                   303,
                                   {17, 8, 10},
                                   {17, 7, 11}},
                                  {2822792.0880,
                                   12.0254,
                                   -6.8072,
                                   3,
                                   2084.8554,
                                   33,
                                   34002,
                                   303,
                                   {16, 7, 9},
                                   {15, 10, 6}},
                                  {2845363.3168,
                                   0.2614,
                                   -4.1789,
                                   3,
                                   801.2849,
                                   57,
                                   34002,
                                   303,
                                   {9, 9, 0},
                                   {9, 6, 3}},
                                  {2847949.9887,
                                   31.7176,
                                   -6.5164,
                                   3,
                                   1058.9274,
                                   81,
                                   34002,
                                   303,
                                   {13, 2, 11},
                                   {14, 1, 14}},
                                  {2847950.0866,
                                   31.7176,
                                   -6.9935,
                                   3,
                                   1058.9274,
                                   27,
                                   34002,
                                   303,
                                   {13, 3, 11},
                                   {14, 0, 14}},
                                  {2866832.8329,
                                   14.9896,
                                   -0.7684,
                                   3,
                                   263.7389,
                                   39,
                                   -34002,
                                   303,
                                   {6, 4, 3},
                                   {5, 3, 2}},
                                  {2867795.4725,
                                   5.7920,
                                   -4.4709,
                                   3,
                                   1998.1537,
                                   33,
                                   34002,
                                   303,
                                   {16, 6, 10},
                                   {16, 5, 11}},
                                  {2868101.5643,
                                   5.9104,
                                   -3.9937,
                                   3,
                                   1998.1544,
                                   99,
                                   34002,
                                   303,
                                   {16, 7, 10},
                                   {16, 6, 11}},
                                  {2873132.2965,
                                   2.4184,
                                   -5.1544,
                                   3,
                                   1450.2212,
                                   81,
                                   34002,
                                   303,
                                   {13, 7, 6},
                                   {12, 10, 3}},
                                  {2887651.1042,
                                   3.8681,
                                   -5.1701,
                                   3,
                                   1586.4084,
                                   87,
                                   34002,
                                   303,
                                   {14, 7, 8},
                                   {13, 8, 5}},
                                  {2905055.0958,
                                   261.4329,
                                   -7.1679,
                                   3,
                                   3628.7876,
                                   129,
                                   34002,
                                   303,
                                   {21, 10, 11},
                                   {21, 9, 12}},
                                  {2905055.0959,
                                   261.4329,
                                   -7.1679,
                                   3,
                                   3628.7876,
                                   129,
                                   34002,
                                   303,
                                   {21, 10, 11},
                                   {21, 9, 12}},
                                  {2911389.6286,
                                   2.6561,
                                   -3.4331,
                                   3,
                                   1699.9884,
                                   93,
                                   34002,
                                   303,
                                   {15, 5, 10},
                                   {15, 4, 11}},
                                  {2911467.2759,
                                   2.6860,
                                   -3.9102,
                                   3,
                                   1699.9886,
                                   31,
                                   34002,
                                   303,
                                   {15, 6, 10},
                                   {15, 5, 11}},
                                  {2915821.4031,
                                   252.5529,
                                   -7.6432,
                                   3,
                                   3628.8230,
                                   43,
                                   34002,
                                   303,
                                   {21, 11, 11},
                                   {21, 10, 12}},
                                  {2919842.5415,
                                   0.0787,
                                   -3.9286,
                                   3,
                                   166.3435,
                                   33,
                                   34002,
                                   303,
                                   {5, 3, 2},
                                   {5, 0, 5}},
                                  {2921657.4664,
                                   0.3552,
                                   -4.1014,
                                   3,
                                   985.3515,
                                   23,
                                   34002,
                                   303,
                                   {11, 6, 6},
                                   {10, 7, 3}},
                                  {2945098.7541,
                                   0.0759,
                                   -4.2839,
                                   3,
                                   458.6686,
                                   15,
                                   34002,
                                   303,
                                   {7, 7, 1},
                                   {7, 4, 4}},
                                  {2946592.0168,
                                   14.9896,
                                   -2.9533,
                                   3,
                                   556.9065,
                                   17,
                                   -34002,
                                   303,
                                   {8, 6, 2},
                                   {7, 7, 1}},
                                  {2952101.6026,
                                   14.9896,
                                   -2.9243,
                                   3,
                                   1418.4664,
                                   87,
                                   -34002,
                                   303,
                                   {14, 5, 10},
                                   {14, 4, 11}},
                                  {2952101.6026,
                                   14.9896,
                                   -3.4014,
                                   3,
                                   1418.4664,
                                   29,
                                   -34002,
                                   303,
                                   {14, 4, 10},
                                   {14, 3, 11}},
                                  {2953358.0328,
                                   14.9896,
                                   -0.8646,
                                   3,
                                   359.3662,
                                   45,
                                   -34002,
                                   303,
                                   {7, 3, 4},
                                   {6, 4, 3}},
                                  {2964066.9192,
                                   14.9896,
                                   -2.0278,
                                   3,
                                   71.4652,
                                   9,
                                   -34002,
                                   303,
                                   {4, 2, 2},
                                   {3, 1, 3}},
                                  {2969011.5596,
                                   6.3551,
                                   -5.7749,
                                   3,
                                   1860.8363,
                                   93,
                                   34002,
                                   303,
                                   {15, 7, 8},
                                   {14, 10, 5}},
                                  {2970133.8995,
                                   128.3567,
                                   -6.8847,
                                   3,
                                   3250.9045,
                                   41,
                                   34002,
                                   303,
                                   {20, 9, 11},
                                   {20, 8, 12}},
                                  {2973567.4499,
                                   127.2470,
                                   -6.4070,
                                   3,
                                   3250.9188,
                                   123,
                                   34002,
                                   303,
                                   {20, 10, 11},
                                   {20, 9, 12}},
                                  {2974733.2350,
                                   14.9896,
                                   -0.5900,
                                   3,
                                   469.2461,
                                   63,
                                   -34002,
                                   303,
                                   {10, 1, 10},
                                   {9, 0, 9}},
                                  {2974733.2350,
                                   14.9896,
                                   -1.0671,
                                   3,
                                   469.2461,
                                   21,
                                   -34002,
                                   303,
                                   {10, 0, 10},
                                   {9, 1, 9}},
                                  {2977007.7604,
                                   14.9896,
                                   -0.8837,
                                   3,
                                   196.8021,
                                   11,
                                   -34002,
                                   303,
                                   {5, 5, 1},
                                   {4, 4, 0}},
                                  {2978450.9613,
                                   14.9896,
                                   -1.2954,
                                   3,
                                   387.8848,
                                   45,
                                   -34002,
                                   303,
                                   {7, 4, 3},
                                   {6, 5, 2}},
                                  {2983517.1541,
                                   14.9896,
                                   -0.6637,
                                   3,
                                   452.4945,
                                   57,
                                   -34002,
                                   303,
                                   {9, 1, 8},
                                   {8, 2, 7}},
                                  {2983517.1541,
                                   14.9896,
                                   -1.1408,
                                   3,
                                   452.4943,
                                   19,
                                   -34002,
                                   303,
                                   {9, 2, 8},
                                   {8, 1, 7}},
                                  {2985373.4690,
                                   14.9896,
                                   -3.0563,
                                   3,
                                   681.6867,
                                   19,
                                   -34002,
                                   303,
                                   {9, 6, 4},
                                   {8, 7, 1}},
                                  {2990251.6918,
                                   14.9896,
                                   -2.4780,
                                   3,
                                   1153.9248,
                                   81,
                                   -34002,
                                   303,
                                   {13, 3, 10},
                                   {13, 2, 11}},
                                  {2990251.6918,
                                   14.9896,
                                   -2.9551,
                                   3,
                                   1153.9248,
                                   27,
                                   -34002,
                                   303,
                                   {13, 4, 10},
                                   {13, 3, 11}},
                                  {2990930.1222,
                                   14.9896,
                                   -1.2068,
                                   3,
                                   415.5219,
                                   17,
                                   -34002,
                                   303,
                                   {8, 2, 6},
                                   {7, 3, 5}},
                                  {2992526.5170,
                                   14.9896,
                                   -0.7290,
                                   3,
                                   415.4772,
                                   51,
                                   -34002,
                                   303,
                                   {8, 3, 6},
                                   {7, 2, 5}},
                                  {3002363.3799,
                                   3.3809,
                                   -5.8271,
                                   3,
                                   1648.9115,
                                   29,
                                   34002,
                                   303,
                                   {14, 7, 7},
                                   {13, 10, 4}},
                                  {3016065.9210,
                                   14.9896,
                                   -1.5882,
                                   3,
                                   95.0563,
                                   27,
                                   -34002,
                                   303,
                                   {4, 4, 1},
                                   {3, 1, 2}},
                                  {3024025.7858,
                                   119.9037,
                                   -7.6699,
                                   3,
                                   1866.8464,
                                   105,
                                   34002,
                                   303,
                                   {17, 3, 14},
                                   {18, 2, 17}},
                                  {3025279.6821,
                                   0.7232,
                                   -4.6571,
                                   3,
                                   1350.1911,
                                   25,
                                   34002,
                                   303,
                                   {12, 10, 2},
                                   {12, 7, 5}},
                                  {3026046.9113,
                                   14.9896,
                                   -2.1177,
                                   3,
                                   906.6886,
                                   75,
                                   -34002,
                                   303,
                                   {12, 3, 10},
                                   {12, 2, 11}},
                                  {3026046.9113,
                                   14.9896,
                                   -2.5948,
                                   3,
                                   906.6886,
                                   25,
                                   -34002,
                                   303,
                                   {12, 2, 10},
                                   {12, 1, 11}},
                                  {3028282.6590,
                                   60.4224,
                                   -5.6844,
                                   3,
                                   2887.9125,
                                   117,
                                   34002,
                                   303,
                                   {19, 8, 11},
                                   {19, 7, 12}},
                                  {3028388.5902,
                                   14.9896,
                                   -0.4022,
                                   3,
                                   195.6614,
                                   33,
                                   -34002,
                                   303,
                                   {5, 5, 0},
                                   {4, 4, 1}},
                                  {3029421.2001,
                                   60.6055,
                                   -6.1613,
                                   3,
                                   2887.9166,
                                   39,
                                   34002,
                                   303,
                                   {19, 9, 11},
                                   {19, 8, 12}},
                                  {3051232.7755,
                                   14.9896,
                                   -3.3821,
                                   3,
                                   716.9203,
                                   19,
                                   -34002,
                                   303,
                                   {9, 7, 3},
                                   {8, 8, 0}},
                                  {3059562.7398,
                                   2.5939,
                                   -1.9175,
                                   3,
                                   677.0713,
                                   69,
                                   34002,
                                   303,
                                   {11, 1, 10},
                                   {11, 0, 11}},
                                  {3059562.7812,
                                   2.5939,
                                   -2.3946,
                                   3,
                                   677.0713,
                                   23,
                                   34002,
                                   303,
                                   {11, 2, 10},
                                   {11, 1, 11}},
                                  {3062486.9844,
                                   14.9896,
                                   -1.2798,
                                   3,
                                   356.5151,
                                   15,
                                   -34002,
                                   303,
                                   {7, 4, 4},
                                   {6, 3, 3}},
                                  {3074163.3010,
                                   14.9896,
                                   -1.5866,
                                   3,
                                   71.4243,
                                   27,
                                   -34002,
                                   303,
                                   {4, 3, 2},
                                   {3, 0, 3}},
                                  {3081895.8727,
                                   28.1447,
                                   -5.4772,
                                   3,
                                   2540.1970,
                                   37,
                                   34002,
                                   303,
                                   {18, 7, 11},
                                   {18, 6, 12}},
                                  {3082238.1937,
                                   28.3458,
                                   -5.0000,
                                   3,
                                   2540.1980,
                                   111,
                                   34002,
                                   303,
                                   {18, 8, 11},
                                   {18, 7, 12}},
                                  {3082788.9296,
                                   14.9896,
                                   -2.9428,
                                   3,
                                   824.2715,
                                   63,
                                   -34002,
                                   303,
                                   {10, 6, 5},
                                   {9, 7, 2}},
                                  {3089444.2499,
                                   5.5865,
                                   -6.2275,
                                   3,
                                   1698.5981,
                                   29,
                                   34002,
                                   303,
                                   {14, 8, 6},
                                   {13, 11, 3}},
                                  {3098963.9319,
                                   14.9896,
                                   -0.7574,
                                   3,
                                   173.9673,
                                   33,
                                   -34002,
                                   303,
                                   {5, 4, 1},
                                   {4, 3, 2}},
                                  {3100351.6418,
                                   1.4454,
                                   -4.3015,
                                   3,
                                   1162.1390,
                                   69,
                                   34002,
                                   303,
                                   {11, 10, 1},
                                   {11, 7, 4}},
                                  {3132109.5279,
                                   13.6308,
                                   -4.3571,
                                   3,
                                   2208.1362,
                                   105,
                                   34002,
                                   303,
                                   {17, 6, 11},
                                   {17, 5, 12}},
                                  {3132201.6896,
                                   13.7059,
                                   -4.8342,
                                   3,
                                   2208.1364,
                                   35,
                                   34002,
                                   303,
                                   {17, 7, 11},
                                   {17, 6, 12}},
                                  {3133481.1087,
                                   0.7429,
                                   -4.4534,
                                   3,
                                   1306.0224,
                                   25,
                                   34002,
                                   303,
                                   {12, 9, 3},
                                   {12, 6, 6}},
                                  {3136990.0877,
                                   65.7313,
                                   -7.2494,
                                   3,
                                   1204.8307,
                                   29,
                                   34002,
                                   303,
                                   {14, 2, 12},
                                   {15, 1, 15}},
                                  {3136990.1108,
                                   65.7313,
                                   -6.7723,
                                   3,
                                   1204.8307,
                                   87,
                                   34002,
                                   303,
                                   {14, 3, 12},
                                   {15, 0, 15}},
                                  {3140680.8975,
                                   0.0804,
                                   -4.5314,
                                   3,
                                   166.3442,
                                   11,
                                   34002,
                                   303,
                                   {5, 4, 2},
                                   {5, 1, 5}},
                                  {3175317.9345,
                                   0.3173,
                                   -3.7344,
                                   3,
                                   1082.3184,
                                   69,
                                   34002,
                                   303,
                                   {11, 8, 3},
                                   {11, 5, 6}},
                                  {3179574.3717,
                                   6.6179,
                                   -4.2366,
                                   3,
                                   1892.0945,
                                   33,
                                   34002,
                                   303,
                                   {16, 5, 11},
                                   {16, 4, 12}},
                                  {3179596.1944,
                                   6.6331,
                                   -3.7594,
                                   3,
                                   1892.0945,
                                   99,
                                   34002,
                                   303,
                                   {16, 6, 11},
                                   {16, 5, 12}},
                                  {3180174.8015,
                                   50.0143,
                                   -7.7747,
                                   3,
                                   2629.2805,
                                   37,
                                   34002,
                                   303,
                                   {18, 8, 10},
                                   {17, 11, 7}},
                                  {3182024.7302,
                                   0.1119,
                                   -4.0625,
                                   3,
                                   610.7241,
                                   51,
                                   34002,
                                   303,
                                   {8, 8, 1},
                                   {8, 5, 4}},
                                  {3186269.3690,
                                   0.0816,
                                   -3.6901,
                                   3,
                                   281.6023,
                                   39,
                                   34002,
                                   303,
                                   {6, 5, 2},
                                   {6, 2, 5}},
                                  {3193556.4668,
                                   2.5635,
                                   -4.3524,
                                   3,
                                   1546.0586,
                                   81,
                                   34002,
                                   303,
                                   {13, 10, 3},
                                   {13, 7, 6}},
                                  {3198461.0406,
                                   14.7171,
                                   -6.5158,
                                   3,
                                   1965.5240,
                                   93,
                                   34002,
                                   303,
                                   {15, 9, 6},
                                   {14, 12, 3}},
                                  {3205426.4915,
                                   19.6034,
                                   -6.0578,
                                   3,
                                   2072.2132,
                                   99,
                                   34002,
                                   303,
                                   {16, 8, 9},
                                   {15, 9, 6}},
                                  {3219967.6628,
                                   14.9896,
                                   -2.8079,
                                   3,
                                   716.8650,
                                   57,
                                   -34002,
                                   303,
                                   {9, 7, 2},
                                   {8, 8, 1}},
                                  {3222503.2080,
                                   204.0356,
                                   -6.9029,
                                   3,
                                   3521.2965,
                                   129,
                                   34002,
                                   303,
                                   {21, 9, 12},
                                   {21, 8, 13}},
                                  {3222503.2081,
                                   204.0356,
                                   -6.9029,
                                   3,
                                   3521.2965,
                                   129,
                                   34002,
                                   303,
                                   {21, 9, 12},
                                   {21, 8, 13}},
                                  {3223319.1159,
                                   204.1576,
                                   -7.3799,
                                   3,
                                   3521.3046,
                                   43,
                                   34002,
                                   303,
                                   {21, 10, 12},
                                   {21, 9, 13}},
                                  {3224676.5029,
                                   17.9875,
                                   -3.2130,
                                   3,
                                   1592.4247,
                                   93,
                                   -34002,
                                   303,
                                   {15, 4, 11},
                                   {15, 3, 12}},
                                  {3224676.5029,
                                   17.9875,
                                   -3.6901,
                                   3,
                                   1592.4247,
                                   31,
                                   -34002,
                                   303,
                                   {15, 5, 11},
                                   {15, 4, 12}},
                                  {3255719.7121,
                                   29.9792,
                                   -0.6834,
                                   3,
                                   568.4724,
                                   69,
                                   -34002,
                                   303,
                                   {11, 0, 11},
                                   {10, 1, 10}},
                                  {3255719.7121,
                                   29.9792,
                                   -1.1606,
                                   3,
                                   568.4724,
                                   23,
                                   -34002,
                                   303,
                                   {11, 1, 11},
                                   {10, 0, 10}},
                                  {3262420.5368,
                                   0.0932,
                                   -4.0772,
                                   3,
                                   415.5219,
                                   15,
                                   34002,
                                   303,
                                   {7, 6, 2},
                                   {7, 3, 5}},
                                  {3264227.5223,
                                   14.9896,
                                   -0.7465,
                                   3,
                                   552.0138,
                                   63,
                                   -34002,
                                   303,
                                   {10, 2, 9},
                                   {9, 1, 8}},
                                  {3264227.5223,
                                   14.9896,
                                   -1.2237,
                                   3,
                                   552.0139,
                                   21,
                                   -34002,
                                   303,
                                   {10, 1, 9},
                                   {9, 2, 8}},
                                  {3267647.2015,
                                   1.9655,
                                   -3.2056,
                                   3,
                                   1309.4694,
                                   29,
                                   34002,
                                   303,
                                   {14, 3, 11},
                                   {14, 2, 12}},
                                  {3267647.9426,
                                   1.9652,
                                   -2.7285,
                                   3,
                                   1309.4694,
                                   87,
                                   34002,
                                   303,
                                   {14, 4, 11},
                                   {14, 3, 12}},
                                  {3268580.2090,
                                   14.9896,
                                   -1.3324,
                                   3,
                                   458.6686,
                                   17,
                                   -34002,
                                   303,
                                   {8, 3, 5},
                                   {7, 4, 4}},
                                  {3271208.4895,
                                   14.9896,
                                   -0.7936,
                                   3,
                                   515.2971,
                                   57,
                                   -34002,
                                   303,
                                   {9, 2, 7},
                                   {8, 3, 6}},
                                  {3271513.9780,
                                   14.9896,
                                   -1.2706,
                                   3,
                                   515.2886,
                                   19,
                                   -34002,
                                   303,
                                   {9, 3, 7},
                                   {8, 2, 6}},
                                  {3272844.8335,
                                   0.6887,
                                   -5.1964,
                                   3,
                                   985.3515,
                                   21,
                                   34002,
                                   303,
                                   {10, 10, 0},
                                   {10, 7, 3}},
                                  {3282166.3922,
                                   105.0098,
                                   -6.6222,
                                   3,
                                   3141.4232,
                                   41,
                                   34002,
                                   303,
                                   {20, 8, 12},
                                   {20, 7, 13}},
                                  {3282556.7854,
                                   105.2813,
                                   -6.1450,
                                   3,
                                   3141.4245,
                                   123,
                                   34002,
                                   303,
                                   {20, 9, 12},
                                   {20, 8, 13}},
                                  {3289313.5622,
                                   0.2274,
                                   -4.0854,
                                   3,
                                   875.6318,
                                   21,
                                   34002,
                                   303,
                                   {10, 7, 3},
                                   {10, 4, 6}},
                                  {3297778.1957,
                                   14.9896,
                                   -0.8427,
                                   3,
                                   457.8796,
                                   51,
                                   -34002,
                                   303,
                                   {8, 4, 5},
                                   {7, 3, 4}},
                                  {3297825.6915,
                                   1.8190,
                                   -4.8448,
                                   3,
                                   1380.5116,
                                   27,
                                   34002,
                                   303,
                                   {13, 7, 7},
                                   {12, 8, 4}},
                                  {3308600.4036,
                                   14.9896,
                                   -2.3297,
                                   3,
                                   1043.5618,
                                   81,
                                   -34002,
                                   303,
                                   {13, 2, 11},
                                   {13, 1, 12}},
                                  {3308600.4036,
                                   14.9896,
                                   -2.8068,
                                   3,
                                   1043.5618,
                                   27,
                                   -34002,
                                   303,
                                   {13, 3, 11},
                                   {13, 2, 12}},
                                  {3314123.4801,
                                   14.9896,
                                   -0.6052,
                                   3,
                                   277.3376,
                                   39,
                                   -34002,
                                   303,
                                   {6, 5, 2},
                                   {5, 4, 1}},
                                  {3329606.2615,
                                   14.9896,
                                   -2.1143,
                                   3,
                                   524.3446,
                                   17,
                                   -34002,
                                   303,
                                   {8, 5, 3},
                                   {7, 6, 2}},
                                  {3338881.9681,
                                   54.5197,
                                   -5.4253,
                                   3,
                                   2776.5394,
                                   117,
                                   34002,
                                   303,
                                   {19, 7, 12},
                                   {19, 6, 13}},
                                  {3338992.7367,
                                   54.6606,
                                   -5.9024,
                                   3,
                                   2776.5397,
                                   39,
                                   34002,
                                   303,
                                   {19, 8, 12},
                                   {19, 7, 13}},
                                  {3345412.2079,
                                   29.4204,
                                   -6.6833,
                                   3,
                                   2378.5644,
                                   105,
                                   34002,
                                   303,
                                   {17, 8, 9},
                                   {16, 11, 6}},
                                  {3346125.0300,
                                   8.3914,
                                   -5.8198,
                                   3,
                                   1912.2573,
                                   93,
                                   34002,
                                   303,
                                   {15, 8, 7},
                                   {14, 11, 4}},
                                  {3347560.0398,
                                   6.0688,
                                   -2.5678,
                                   3,
                                   795.0260,
                                   25,
                                   34002,
                                   303,
                                   {12, 1, 11},
                                   {12, 0, 12}},
                                  {3347560.0484,
                                   6.0688,
                                   -2.0906,
                                   3,
                                   795.0260,
                                   75,
                                   34002,
                                   303,
                                   {12, 2, 11},
                                   {12, 1, 12}},
                                  {3374560.7807,
                                   6.4817,
                                   -5.3437,
                                   3,
                                   1801.6509,
                                   29,
                                   34002,
                                   303,
                                   {14, 11, 3},
                                   {14, 8, 6}},
                                  {3375154.1157,
                                   0.1124,
                                   -3.6805,
                                   3,
                                   567.8815,
                                   51,
                                   34002,
                                   303,
                                   {8, 7, 2},
                                   {8, 4, 5}},
                                  {3376367.0814,
                                   2.6321,
                                   -4.7801,
                                   3,
                                   1586.4084,
                                   81,
                                   34002,
                                   303,
                                   {13, 11, 2},
                                   {13, 8, 5}},
                                  {3392977.5198,
                                   28.9459,
                                   -5.2232,
                                   3,
                                   2427.0194,
                                   37,
                                   34002,
                                   303,
                                   {18, 6, 12},
                                   {18, 5, 13}},
                                  {3393005.6178,
                                   28.9860,
                                   -4.7461,
                                   3,
                                   2427.0195,
                                   111,
                                   34002,
                                   303,
                                   {18, 7, 12},
                                   {18, 6, 13}},
                                  {3404390.9892,
                                   14.9896,
                                   -2.7770,
                                   3,
                                   856.7557,
                                   63,
                                   -34002,
                                   303,
                                   {10, 7, 4},
                                   {9, 8, 1}},
                                  {3415107.3704,
                                   14.9896,
                                   -3.2049,
                                   3,
                                   896.1960,
                                   63,
                                   -34002,
                                   303,
                                   {10, 8, 3},
                                   {9, 9, 0}},
                                  {3416407.1722,
                                   16.0912,
                                   -6.6440,
                                   3,
                                   2139.1776,
                                   33,
                                   34002,
                                   303,
                                   {16, 8, 8},
                                   {15, 11, 5}},
                                  {3424687.6869,
                                   130.5402,
                                   -7.0445,
                                   3,
                                   1360.0023,
                                   93,
                                   34002,
                                   303,
                                   {15, 2, 13},
                                   {16, 1, 16}},
                                  {3424687.6925,
                                   130.5403,
                                   -7.5216,
                                   3,
                                   1360.0023,
                                   31,
                                   34002,
                                   303,
                                   {15, 3, 13},
                                   {16, 0, 16}},
                                  {3441510.3869,
                                   0.1766,
                                   -3.5771,
                                   3,
                                   686.4885,
                                   57,
                                   34002,
                                   303,
                                   {9, 6, 3},
                                   {9, 3, 6}},
                                  {3444717.4852,
                                   0.2723,
                                   -4.8995,
                                   3,
                                   781.2679,
                                   19,
                                   34002,
                                   303,
                                   {9, 9, 1},
                                   {9, 6, 4}},
                                  {3444883.1550,
                                   15.0307,
                                   -4.1115,
                                   3,
                                   2093.2272,
                                   105,
                                   34002,
                                   303,
                                   {17, 5, 12},
                                   {17, 4, 13}},
                                  {3444889.4117,
                                   15.0371,
                                   -4.5886,
                                   3,
                                   2093.2272,
                                   35,
                                   34002,
                                   303,
                                   {17, 6, 12},
                                   {17, 5, 13}},
                                  {3453514.9813,
                                   14.9896,
                                   -1.5540,
                                   3,
                                   493.3685,
                                   17,
                                   -34002,
                                   303,
                                   {8, 4, 4},
                                   {7, 5, 3}},
                                  {3457226.4120,
                                   14.9896,
                                   -1.1751,
                                   3,
                                   148.4183,
                                   33,
                                   -34002,
                                   303,
                                   {5, 3, 2},
                                   {4, 2, 3}},
                                  {3494885.3432,
                                   7.6345,
                                   -4.0047,
                                   3,
                                   1775.5177,
                                   33,
                                   34002,
                                   303,
                                   {16, 4, 12},
                                   {16, 3, 13}},
                                  {3494886.5347,
                                   7.6345,
                                   -3.5276,
                                   3,
                                   1775.5177,
                                   99,
                                   34002,
                                   303,
                                   {16, 5, 12},
                                   {16, 4, 13}},
                                  {3512693.2525,
                                   0.4875,
                                   -3.6341,
                                   3,
                                   896.1713,
                                   21,
                                   34002,
                                   303,
                                   {10, 8, 2},
                                   {9, 9, 1}},
                                  {3527902.8291,
                                   0.1435,
                                   -4.3705,
                                   3,
                                   738.4675,
                                   19,
                                   34002,
                                   303,
                                   {9, 8, 2},
                                   {9, 5, 5}},
                                  {3529759.9323,
                                   75.4530,
                                   -7.0696,
                                   3,
                                   2617.7424,
                                   111,
                                   34002,
                                   303,
                                   {18, 9, 10},
                                   {17, 10, 7}},
                                  {3531586.2349,
                                   177.8882,
                                   -6.6298,
                                   3,
                                   3403.4954,
                                   129,
                                   34002,
                                   303,
                                   {21, 8, 13},
                                   {21, 7, 14}},
                                  {3531586.2350,
                                   177.8882,
                                   -6.6298,
                                   3,
                                   3403.4954,
                                   129,
                                   34002,
                                   303,
                                   {21, 8, 13},
                                   {21, 7, 14}},
                                  {3531842.2072,
                                   178.0462,
                                   -7.1069,
                                   3,
                                   3403.4950,
                                   43,
                                   34002,
                                   303,
                                   {21, 9, 13},
                                   {21, 8, 14}},
                                  {3535685.6957,
                                   14.9896,
                                   -1.2768,
                                   3,
                                   375.4308,
                                   15,
                                   -34002,
                                   303,
                                   {7, 5, 3},
                                   {6, 4, 2}},
                                  {3536194.0170,
                                   3.9660,
                                   -1.2841,
                                   3,
                                   677.0713,
                                   25,
                                   34002,
                                   303,
                                   {12, 0, 12},
                                   {11, 1, 11}},
                                  {3536194.0175,
                                   3.9660,
                                   -0.8070,
                                   3,
                                   677.0713,
                                   75,
                                   34002,
                                   303,
                                   {12, 1, 12},
                                   {11, 0, 11}},
                                  {3538402.9128,
                                   4.2902,
                                   -5.3966,
                                   3,
                                   1380.5116,
                                   25,
                                   34002,
                                   303,
                                   {12, 11, 1},
                                   {12, 8, 4}},
                                  {3543162.1155,
                                   4.6243,
                                   -3.0051,
                                   3,
                                   1474.2376,
                                   93,
                                   34002,
                                   303,
                                   {15, 3, 12},
                                   {15, 2, 13}},
                                  {3543162.3022,
                                   4.6241,
                                   -3.4822,
                                   3,
                                   1474.2376,
                                   31,
                                   34002,
                                   303,
                                   {15, 4, 12},
                                   {15, 3, 13}},
                                  {3544467.5162,
                                   14.9896,
                                   -0.8616,
                                   3,
                                   660.8967,
                                   69,
                                   -34002,
                                   303,
                                   {11, 1, 10},
                                   {10, 2, 9}},
                                  {3544467.5162,
                                   14.9896,
                                   -1.3388,
                                   3,
                                   660.8967,
                                   23,
                                   -34002,
                                   303,
                                   {11, 2, 10},
                                   {10, 1, 9}},
                                  {3550690.9078,
                                   14.9896,
                                   -1.3718,
                                   3,
                                   624.4145,
                                   21,
                                   -34002,
                                   303,
                                   {10, 2, 8},
                                   {9, 3, 7}},
                                  {3550749.0676,
                                   14.9896,
                                   -0.8947,
                                   3,
                                   624.4128,
                                   63,
                                   -34002,
                                   303,
                                   {10, 3, 8},
                                   {9, 2, 7}},
                                  {3555749.0062,
                                   14.9896,
                                   -0.9225,
                                   3,
                                   567.8815,
                                   57,
                                   -34002,
                                   303,
                                   {9, 3, 6},
                                   {8, 4, 5}},
                                  {3562514.4226,
                                   14.9896,
                                   -1.3971,
                                   3,
                                   567.6967,
                                   19,
                                   -34002,
                                   303,
                                   {9, 4, 6},
                                   {8, 3, 5}},
                                  {3571275.6754,
                                   0.9627,
                                   -3.5977,
                                   3,
                                   1188.2356,
                                   75,
                                   34002,
                                   303,
                                   {12, 7, 6},
                                   {11, 8, 3}},
                                  {3588474.8018,
                                   23.0762,
                                   -6.9260,
                                   3,
                                   2193.9950,
                                   33,
                                   34002,
                                   303,
                                   {16, 9, 7},
                                   {15, 12, 4}},
                                  {3589797.8197,
                                   4.3059,
                                   -3.0449,
                                   3,
                                   1189.7267,
                                   29,
                                   34002,
                                   303,
                                   {14, 2, 12},
                                   {14, 1, 13}},
                                  {3589797.8423,
                                   4.3059,
                                   -2.5678,
                                   3,
                                   1189.7267,
                                   87,
                                   34002,
                                   303,
                                   {14, 3, 12},
                                   {14, 2, 13}},
                                  {3592132.0978,
                                   99.9453,
                                   -6.3518,
                                   3,
                                   3021.6026,
                                   41,
                                   34002,
                                   303,
                                   {20, 7, 13},
                                   {20, 6, 14}},
                                  {3592168.1227,
                                   100.0295,
                                   -5.8746,
                                   3,
                                   3021.6026,
                                   123,
                                   34002,
                                   303,
                                   {20, 8, 13},
                                   {20, 7, 14}},
                                  {3594455.5102,
                                   14.9896,
                                   -0.3715,
                                   3,
                                   296.6776,
                                   39,
                                   -34002,
                                   303,
                                   {6, 6, 1},
                                   {5, 5, 0}},
                                  {3601096.3167,
                                   0.1517,
                                   -4.1115,
                                   3,
                                   515.2886,
                                   17,
                                   34002,
                                   303,
                                   {8, 5, 3},
                                   {8, 2, 6}},
                                  {3607347.4853,
                                   14.9896,
                                   -3.5000,
                                   3,
                                   1013.3421,
                                   23,
                                   -34002,
                                   303,
                                   {11, 7, 5},
                                   {10, 8, 2}},
                                  {3619752.5974,
                                   14.9896,
                                   -0.8456,
                                   3,
                                   296.1044,
                                   13,
                                   -34002,
                                   303,
                                   {6, 6, 0},
                                   {5, 5, 1}},
                                  {3620940.6749,
                                   14.9896,
                                   -1.1009,
                                   3,
                                   271.1060,
                                   13,
                                   -34002,
                                   303,
                                   {6, 5, 1},
                                   {5, 4, 2}},
                                  {3622104.4693,
                                   14.9896,
                                   -2.0357,
                                   3,
                                   680.4646,
                                   57,
                                   -34002,
                                   303,
                                   {9, 6, 3},
                                   {8, 7, 2}},
                                  {3634791.6965,
                                   13.6372,
                                   -2.2899,
                                   3,
                                   922.3182,
                                   81,
                                   34002,
                                   303,
                                   {13, 1, 12},
                                   {13, 0, 13}},
                                  {3634791.6984,
                                   13.6372,
                                   -2.7670,
                                   3,
                                   922.3182,
                                   27,
                                   34002,
                                   303,
                                   {13, 2, 12},
                                   {13, 1, 13}},
                                  {3645459.7721,
                                   9.8087,
                                   -5.7200,
                                   3,
                                   1838.6480,
                                   31,
                                   34002,
                                   303,
                                   {15, 8, 8},
                                   {14, 9, 5}},
                                  {3649236.1944,
                                   21.1291,
                                   -5.2106,
                                   3,
                                   2023.8720,
                                   93,
                                   34002,
                                   303,
                                   {15, 11, 4},
                                   {15, 8, 7}},
                                  {3650608.4328,
                                   56.6257,
                                   -5.1595,
                                   3,
                                   2654.7682,
                                   117,
                                   34002,
                                   303,
                                   {19, 6, 13},
                                   {19, 5, 14}},
                                  {3650617.1050,
                                   56.6453,
                                   -5.6366,
                                   3,
                                   2654.7683,
                                   39,
                                   34002,
                                   303,
                                   {19, 7, 13},
                                   {19, 6, 14}},
                                  {3671485.7663,
                                   9.2765,
                                   -5.2733,
                                   3,
                                   1749.0596,
                                   29,
                                   34002,
                                   303,
                                   {14, 10, 4},
                                   {14, 7, 7}},
                                  {3680740.5425,
                                   13.5291,
                                   -5.5055,
                                   3,
                                   2072.2132,
                                   93,
                                   34002,
                                   303,
                                   {15, 12, 3},
                                   {15, 9, 6}},
                                  {3686412.3498,
                                   14.9896,
                                   -1.7643,
                                   3,
                                   148.1406,
                                   11,
                                   -34002,
                                   303,
                                   {5, 4, 2},
                                   {4, 1, 3}},
                                  {3696838.0922,
                                   1.5173,
                                   -5.3092,
                                   3,
                                   1188.2356,
                                   69,
                                   34002,
                                   303,
                                   {11, 11, 0},
                                   {11, 8, 3}},
                                  {3702107.0846,
                                   14.9896,
                                   -0.9348,
                                   3,
                                   487.2351,
                                   51,
                                   -34002,
                                   303,
                                   {8, 5, 4},
                                   {7, 4, 3}},
                                  {3707379.0474,
                                   31.2090,
                                   -4.9654,
                                   3,
                                   2303.3546,
                                   37,
                                   34002,
                                   303,
                                   {18, 5, 13},
                                   {18, 4, 14}},
                                  {3707380.8760,
                                   31.2115,
                                   -4.4883,
                                   3,
                                   2303.3546,
                                   111,
                                   34002,
                                   303,
                                   {18, 6, 13},
                                   {18, 5, 14}},
                                  {3711199.2343,
                                   249.2464,
                                   -7.3308,
                                   3,
                                   1524.4124,
                                   99,
                                   34002,
                                   303,
                                   {16, 3, 14},
                                   {17, 0, 17}},
                                  {3712888.2395,
                                   63.6566,
                                   -7.2457,
                                   3,
                                   2493.8937,
                                   105,
                                   34002,
                                   303,
                                   {17, 10, 7},
                                   {16, 13, 4}},
                                  {3720675.5704,
                                   0.4095,
                                   -4.2184,
                                   3,
                                   927.1022,
                                   63,
                                   34002,
                                   303,
                                   {10, 9, 2},
                                   {10, 6, 5}},
                                  {3723327.2219,
                                   0.7094,
                                   -4.8735,
                                   3,
                                   970.3139,
                                   63,
                                   34002,
                                   303,
                                   {10, 10, 1},
                                   {10, 7, 4}},
                                  {3743916.4319,
                                   0.1106,
                                   -3.8062,
                                   3,
                                   362.3515,
                                   45,
                                   34002,
                                   303,
                                   {7, 4, 3},
                                   {7, 1, 6}},
                                  {3757507.0092,
                                   0.0785,
                                   -3.4967,
                                   3,
                                   71.4652,
                                   9,
                                   34002,
                                   303,
                                   {4, 4, 0},
                                   {3, 1, 3}},
                                  {3758651.4342,
                                   40.7165,
                                   -6.2334,
                                   3,
                                   2313.6937,
                                   33,
                                   34002,
                                   303,
                                   {16, 12, 4},
                                   {16, 9, 7}},
                                  {3762701.2763,
                                   16.9871,
                                   -3.8670,
                                   3,
                                   1967.7170,
                                   105,
                                   34002,
                                   303,
                                   {17, 4, 13},
                                   {17, 3, 14}},
                                  {3762701.6053,
                                   16.9870,
                                   -4.3442,
                                   3,
                                   1967.7170,
                                   35,
                                   34002,
                                   303,
                                   {17, 5, 13},
                                   {17, 4, 14}},
                                  {3770448.7529,
                                   0.0864,
                                   -2.1359,
                                   3,
                                   170.3358,
                                   11,
                                   34002,
                                   303,
                                   {5, 5, 1},
                                   {4, 2, 2}},
                                  {3781498.5733,
                                   1.1269,
                                   -4.0314,
                                   3,
                                   1094.5219,
                                   23,
                                   34002,
                                   303,
                                   {11, 9, 3},
                                   {10, 10, 0}},
                                  {3783266.6755,
                                   39.8987,
                                   -6.6919,
                                   3,
                                   2435.5101,
                                   105,
                                   34002,
                                   303,
                                   {17, 9, 8},
                                   {16, 12, 5}},
                                  {3789587.6028,
                                   0.9700,
                                   -3.5530,
                                   3,
                                   1051.5043,
                                   23,
                                   34002,
                                   303,
                                   {11, 8, 4},
                                   {10, 9, 1}},
                                  {3795667.8456,
                                   3.2323,
                                   -4.4958,
                                   3,
                                   1490.0984,
                                   81,
                                   34002,
                                   303,
                                   {13, 9, 4},
                                   {13, 6, 7}},
                                  {3809421.0605,
                                   66.5361,
                                   -7.6258,
                                   3,
                                   2689.6302,
                                   37,
                                   34002,
                                   303,
                                   {18, 9, 9},
                                   {17, 12, 6}},
                                  {3809906.1179,
                                   7.8334,
                                   -5.9404,
                                   3,
                                   1838.6480,
                                   29,
                                   34002,
                                   303,
                                   {14, 12, 2},
                                   {14, 9, 5}},
                                  {3811460.9027,
                                   0.1668,
                                   -1.0763,
                                   3,
                                   610.7241,
                                   57,
                                   34002,
                                   303,
                                   {9, 4, 5},
                                   {8, 5, 4}},
                                  {3816123.5908,
                                   8.5866,
                                   -0.9589,
                                   3,
                                   795.0260,
                                   81,
                                   34002,
                                   303,
                                   {13, 0, 13},
                                   {12, 1, 12}},
                                  {3816123.5909,
                                   8.5866,
                                   -1.4360,
                                   3,
                                   795.0260,
                                   27,
                                   34002,
                                   303,
                                   {13, 1, 13},
                                   {12, 0, 12}},
                                  {3816748.5047,
                                   10.1932,
                                   -3.7838,
                                   3,
                                   1648.2046,
                                   33,
                                   34002,
                                   303,
                                   {16, 3, 13},
                                   {16, 2, 14}},
                                  {3816748.5532,
                                   10.1931,
                                   -3.3067,
                                   3,
                                   1648.2046,
                                   99,
                                   34002,
                                   303,
                                   {16, 4, 13},
                                   {16, 3, 14}},
                                  {3824191.2757,
                                   0.4530,
                                   -1.4838,
                                   3,
                                   779.1273,
                                   25,
                                   34002,
                                   303,
                                   {12, 1, 11},
                                   {11, 2, 10}},
                                  {3824191.3261,
                                   0.4530,
                                   -1.0067,
                                   3,
                                   779.1273,
                                   75,
                                   34002,
                                   303,
                                   {12, 2, 11},
                                   {11, 1, 10}},
                                  {3829643.7017,
                                   0.2129,
                                   -1.0289,
                                   3,
                                   742.8529,
                                   69,
                                   34002,
                                   303,
                                   {11, 2, 9},
                                   {10, 3, 8}},
                                  {3829655.5008,
                                   0.2126,
                                   -1.5060,
                                   3,
                                   742.8526,
                                   23,
                                   34002,
                                   303,
                                   {11, 3, 9},
                                   {10, 2, 8}},
                                  {3833823.1379,
                                   1.0812,
                                   -3.5308,
                                   3,
                                   1094.5108,
                                   69,
                                   34002,
                                   303,
                                   {11, 9, 2},
                                   {10, 10, 1}},
                                  {3835173.4710,
                                   0.1736,
                                   -1.5133,
                                   3,
                                   686.5293,
                                   21,
                                   34002,
                                   303,
                                   {10, 3, 7},
                                   {9, 4, 6}},
                                  {3836663.4420,
                                   0.1721,
                                   -1.0357,
                                   3,
                                   686.4885,
                                   63,
                                   34002,
                                   303,
                                   {10, 4, 7},
                                   {9, 3, 6}},
                                  {3841935.4042,
                                   174.8280,
                                   -6.8244,
                                   3,
                                   3275.3419,
                                   43,
                                   34002,
                                   303,
                                   {21, 8, 14},
                                   {21, 7, 15}},
                                  {3841953.3589,
                                   174.7639,
                                   -6.3473,
                                   3,
                                   3275.3417,
                                   129,
                                   34002,
                                   303,
                                   {21, 7, 14},
                                   {21, 6, 15}},
                                  {3841953.3589,
                                   174.7639,
                                   -6.3473,
                                   3,
                                   3275.3417,
                                   129,
                                   34002,
                                   303,
                                   {21, 7, 14},
                                   {21, 6, 15}},
                                  {3854435.4540,
                                   0.0976,
                                   -4.6765,
                                   3,
                                   227.9449,
                                   13,
                                   34002,
                                   303,
                                   {6, 3, 3},
                                   {6, 0, 6}},
                                  {3869605.0348,
                                   10.8127,
                                   -2.8310,
                                   3,
                                   1345.1614,
                                   93,
                                   34002,
                                   303,
                                   {15, 2, 13},
                                   {15, 1, 14}},
                                  {3869605.0403,
                                   10.8127,
                                   -3.3081,
                                   3,
                                   1345.1614,
                                   31,
                                   34002,
                                   303,
                                   {15, 3, 13},
                                   {15, 2, 14}},
                                  {3872293.7968,
                                   0.0918,
                                   -1.5536,
                                   3,
                                   114.1776,
                                   33,
                                   34002,
                                   303,
                                   {5, 2, 3},
                                   {4, 1, 4}},
                                  {3873494.6724,
                                   0.3491,
                                   -2.9405,
                                   3,
                                   856.1456,
                                   21,
                                   34002,
                                   303,
                                   {10, 7, 3},
                                   {9, 8, 2}},
                                  {3894371.2261,
                                   0.1830,
                                   -1.5194,
                                   3,
                                   608.5652,
                                   19,
                                   34002,
                                   303,
                                   {9, 5, 5},
                                   {8, 4, 4}},
                                  {3894829.5073,
                                   0.1969,
                                   -1.3914,
                                   3,
                                   646.4593,
                                   57,
                                   34002,
                                   303,
                                   {9, 5, 4},
                                   {8, 6, 3}},
                                  {3903905.4485,
                                   0.0926,
                                   -2.0328,
                                   3,
                                   114.1722,
                                   11,
                                   34002,
                                   303,
                                   {5, 3, 3},
                                   {4, 0, 4}},
                                  {3905109.6392,
                                   104.0486,
                                   -6.0735,
                                   3,
                                   2891.3421,
                                   41,
                                   34002,
                                   303,
                                   {20, 6, 14},
                                   {20, 5, 15}},
                                  {3905112.3399,
                                   104.0576,
                                   -5.5964,
                                   3,
                                   2891.3421,
                                   123,
                                   34002,
                                   303,
                                   {20, 7, 14},
                                   {20, 6, 15}},
                                  {3907318.3013,
                                   0.0900,
                                   -5.5022,
                                   3,
                                   166.3435,
                                   33,
                                   34002,
                                   303,
                                   {5, 5, 0},
                                   {5, 0, 5}},
                                  {3921263.5502,
                                   29.1715,
                                   -2.9913,
                                   3,
                                   1058.9274,
                                   29,
                                   34002,
                                   303,
                                   {14, 1, 13},
                                   {14, 0, 14}},
                                  {3921263.5507,
                                   29.1715,
                                   -2.5141,
                                   3,
                                   1058.9274,
                                   87,
                                   34002,
                                   303,
                                   {14, 2, 13},
                                   {14, 1, 14}},
                                  {3927751.9378,
                                   0.1094,
                                   -4.3497,
                                   3,
                                   362.3528,
                                   15,
                                   34002,
                                   303,
                                   {7, 5, 3},
                                   {7, 2, 6}},
                                  {3928430.1091,
                                   0.1031,
                                   -1.4737,
                                   3,
                                   244.3925,
                                   13,
                                   34002,
                                   303,
                                   {6, 4, 2},
                                   {5, 3, 3}},
                                  {3932142.2595,
                                   0.1543,
                                   -3.7783,
                                   3,
                                   515.2971,
                                   51,
                                   34002,
                                   303,
                                   {8, 6, 3},
                                   {8, 3, 6}},
                                  {3939906.2162,
                                   0.0974,
                                   -4.2250,
                                   3,
                                   227.9450,
                                   39,
                                   34002,
                                   303,
                                   {6, 4, 3},
                                   {6, 1, 6}},
                                  {3949672.2466,
                                   1.3833,
                                   -5.1192,
                                   3,
                                   1133.6702,
                                   23,
                                   34002,
                                   303,
                                   {11, 10, 2},
                                   {11, 7, 5}},
                                  {3962333.7687,
                                   0.2022,
                                   -4.3181,
                                   3,
                                   686.5293,
                                   19,
                                   34002,
                                   303,
                                   {9, 7, 3},
                                   {9, 4, 6}},
                                  {3967063.3654,
                                   60.3007,
                                   -4.8887,
                                   3,
                                   2522.4413,
                                   117,
                                   34002,
                                   303,
                                   {19, 5, 14},
                                   {19, 4, 15}},
                                  {3967063.9083,
                                   60.3016,
                                   -5.3659,
                                   3,
                                   2522.4413,
                                   39,
                                   34002,
                                   303,
                                   {19, 6, 14},
                                   {19, 5, 15}},
                                  {3970960.2621,
                                   0.1056,
                                   -1.0812,
                                   3,
                                   391.8876,
                                   15,
                                   34002,
                                   303,
                                   {7, 6, 2},
                                   {6, 5, 1}},
                                  {3981101.8933,
                                   42.7450,
                                   -6.7168,
                                   3,
                                   2357.7165,
                                   35,
                                   34002,
                                   303,
                                   {17, 9, 9},
                                   {16, 10, 6}},
                                  {3982237.6928,
                                   1.0871,
                                   -4.7588,
                                   3,
                                   1247.6784,
                                   25,
                                   34002,
                                   303,
                                   {12, 8, 4},
                                   {12, 5, 7}},
                                  {3998815.8751,
                                   11.0650,
                                   -5.5906,
                                   3,
                                   1616.7083,
                                   81,
                                   34002,
                                   303,
                                   {13, 12, 1},
                                   {13, 9, 4}},
                                  {4001794.8611,
                                   4.4287,
                                   -4.4235,
                                   3,
                                   1616.7083,
                                   87,
                                   34002,
                                   303,
                                   {14, 8, 7},
                                   {13, 9, 4}},
                                  {4006200.3279,
                                   1.5263,
                                   -5.8898,
                                   3,
                                   1177.9114,
                                   23,
                                   34002,
                                   303,
                                   {11, 11, 1},
                                   {11, 8, 4}},
                                  {4012844.8278,
                                   71.6296,
                                   -6.4317,
                                   3,
                                   2617.7424,
                                   105,
                                   34002,
                                   303,
                                   {17, 13, 4},
                                   {17, 10, 7}},
                                  {4026970.5891,
                                   0.3198,
                                   -4.0274,
                                   3,
                                   875.7862,
                                   63,
                                   34002,
                                   303,
                                   {10, 8, 3},
                                   {10, 5, 6}},
                                  {4028069.6822,
                                   34.6041,
                                   -4.7076,
                                   3,
                                   2168.9926,
                                   37,
                                   34002,
                                   303,
                                   {18, 4, 14},
                                   {18, 3, 15}},
                                  {4028069.7751,
                                   34.6040,
                                   -4.2304,
                                   3,
                                   2168.9926,
                                   111,
                                   34002,
                                   303,
                                   {18, 5, 14},
                                   {18, 4, 15}},
                                  {4054744.1125,
                                   0.0910,
                                   -5.5991,
                                   3,
                                   281.5945,
                                   13,
                                   34002,
                                   303,
                                   {6, 6, 0},
                                   {6, 1, 5}},
                                  {4069260.9629,
                                   1.2752,
                                   -3.2528,
                                   3,
                                   1222.3933,
                                   75,
                                   34002,
                                   303,
                                   {12, 8, 5},
                                   {11, 9, 2}},
                                  {4088321.0858,
                                   3.8601,
                                   -6.4468,
                                   3,
                                   1410.5441,
                                   25,
                                   34002,
                                   303,
                                   {12, 12, 0},
                                   {12, 9, 3}},
                                  {4088336.7334,
                                   21.2866,
                                   -3.6324,
                                   3,
                                   1831.3448,
                                   105,
                                   34002,
                                   303,
                                   {17, 3, 14},
                                   {17, 2, 15}},
                                  {4088336.7465,
                                   21.2866,
                                   -4.1095,
                                   3,
                                   1831.3448,
                                   35,
                                   34002,
                                   303,
                                   {17, 4, 14},
                                   {17, 3, 15}},
                                  {4095440.1631,
                                   17.5172,
                                   -1.1377,
                                   3,
                                   922.3182,
                                   87,
                                   34002,
                                   303,
                                   {14, 1, 14},
                                   {13, 0, 13}},
                                  {4095440.1631,
                                   17.5172,
                                   -1.6148,
                                   3,
                                   922.3182,
                                   29,
                                   34002,
                                   303,
                                   {14, 0, 14},
                                   {13, 1, 13}},
                                  {4097181.0147,
                                   27.6451,
                                   -6.7207,
                                   3,
                                   2357.7165,
                                   33,
                                   34002,
                                   303,
                                   {16, 13, 3},
                                   {16, 10, 6}},
                                  {4103355.2389,
                                   1.1420,
                                   -1.1799,
                                   3,
                                   906.6886,
                                   81,
                                   34002,
                                   303,
                                   {13, 1, 12},
                                   {12, 2, 11}},
                                  {4103355.2495,
                                   1.1420,
                                   -1.6570,
                                   3,
                                   906.6886,
                                   27,
                                   34002,
                                   303,
                                   {13, 2, 12},
                                   {12, 1, 11}},
                                  {4107907.0789,
                                   0.8502,
                                   -2.9051,
                                   3,
                                   1051.2106,
                                   69,
                                   34002,
                                   303,
                                   {11, 8, 3},
                                   {10, 9, 2}},
                                  {4108068.5837,
                                   0.3060,
                                   -1.6706,
                                   3,
                                   870.5962,
                                   25,
                                   34002,
                                   303,
                                   {12, 2, 10},
                                   {11, 3, 9}},
                                  {4108071.0379,
                                   0.3058,
                                   -1.1935,
                                   3,
                                   870.5961,
                                   75,
                                   34002,
                                   303,
                                   {12, 3, 10},
                                   {11, 2, 9}},
                                  {4112083.9393,
                                   0.2138,
                                   -1.6567,
                                   3,
                                   738.4675,
                                   21,
                                   34002,
                                   303,
                                   {10, 4, 6},
                                   {9, 5, 5}},
                                  {4112567.3773,
                                   0.2501,
                                   -1.1862,
                                   3,
                                   814.4658,
                                   69,
                                   34002,
                                   303,
                                   {11, 3, 8},
                                   {10, 4, 7}},
                                  {4112893.3548,
                                   0.2459,
                                   -1.6632,
                                   3,
                                   814.4569,
                                   23,
                                   34002,
                                   303,
                                   {11, 4, 8},
                                   {10, 3, 7}},
                                  {4128945.2059,
                                   101.7456,
                                   -6.2333,
                                   3,
                                   2561.7063,
                                   105,
                                   34002,
                                   303,
                                   {17, 12, 5},
                                   {17, 9, 8}},
                                  {4132685.9405,
                                   0.4730,
                                   -4.7995,
                                   3,
                                   1082.8075,
                                   23,
                                   34002,
                                   303,
                                   {11, 9, 3},
                                   {11, 6, 6}},
                                  {4134902.1585,
                                   0.2069,
                                   -1.1717,
                                   3,
                                   737.8607,
                                   63,
                                   34002,
                                   303,
                                   {10, 5, 6},
                                   {9, 4, 5}},
                                  {4147979.9897,
                                   24.9241,
                                   -3.5954,
                                   3,
                                   1509.8429,
                                   33,
                                   34002,
                                   303,
                                   {16, 2, 14},
                                   {16, 1, 15}},
                                  {4147979.9911,
                                   24.9241,
                                   -3.1183,
                                   3,
                                   1509.8429,
                                   99,
                                   34002,
                                   303,
                                   {16, 3, 14},
                                   {16, 2, 15}},
                                  {4153698.5725,
                                   2.2878,
                                   -3.4483,
                                   3,
                                   1265.5556,
                                   75,
                                   34002,
                                   303,
                                   {12, 9, 4},
                                   {11, 10, 1}},
                                  {4156522.0721,
                                   181.9765,
                                   -6.0560,
                                   3,
                                   3136.6950,
                                   129,
                                   34002,
                                   303,
                                   {21, 6, 15},
                                   {21, 5, 16}},
                                  {4156522.0721,
                                   181.9765,
                                   -6.0560,
                                   3,
                                   3136.6950,
                                   129,
                                   34002,
                                   303,
                                   {21, 6, 15},
                                   {21, 5, 16}},
                                  {4156529.0030,
                                   181.9748,
                                   -6.5331,
                                   3,
                                   3136.6950,
                                   43,
                                   34002,
                                   303,
                                   {21, 7, 15},
                                   {21, 6, 16}},
                                  {4157292.9553,
                                   1.9919,
                                   -3.9456,
                                   3,
                                   1311.5489,
                                   75,
                                   34002,
                                   303,
                                   {12, 10, 3},
                                   {11, 11, 0}},
                                  {4158894.0366,
                                   2.1529,
                                   -4.1916,
                                   3,
                                   1410.5441,
                                   27,
                                   34002,
                                   303,
                                   {13, 8, 6},
                                   {12, 9, 3}},
                                  {4160177.0439,
                                   0.1050,
                                   -0.5999,
                                   3,
                                   387.8848,
                                   45,
                                   34002,
                                   303,
                                   {7, 6, 1},
                                   {6, 5, 2}},
                                  {4167049.1968,
                                   168.9643,
                                   -7.2935,
                                   3,
                                   2884.7056,
                                   37,
                                   34002,
                                   303,
                                   {18, 13, 5},
                                   {18, 10, 8}},
                                  {4183897.4405,
                                   1.9811,
                                   -4.4113,
                                   3,
                                   1311.5438,
                                   25,
                                   34002,
                                   303,
                                   {12, 10, 2},
                                   {11, 11, 1}},
                                  {4191467.6501,
                                   0.5987,
                                   -4.1312,
                                   3,
                                   1022.3267,
                                   69,
                                   34002,
                                   303,
                                   {11, 7, 4},
                                   {11, 4, 7}},
                                  {4198901.2719,
                                   0.1105,
                                   -0.9015,
                                   3,
                                   416.8462,
                                   15,
                                   34002,
                                   303,
                                   {7, 7, 1},
                                   {6, 6, 0}},
                                  {4207009.3353,
                                   59.3670,
                                   -2.7626,
                                   3,
                                   1204.8307,
                                   93,
                                   34002,
                                   303,
                                   {15, 1, 14},
                                   {15, 0, 15}},
                                  {4207009.3354,
                                   59.3670,
                                   -3.2397,
                                   3,
                                   1204.8307,
                                   31,
                                   34002,
                                   303,
                                   {15, 2, 14},
                                   {15, 1, 15}},
                                  {4207466.9102,
                                   4.1427,
                                   -5.1561,
                                   3,
                                   1358.1293,
                                   75,
                                   34002,
                                   303,
                                   {12, 11, 2},
                                   {12, 8, 5}},
                                  {4210710.2204,
                                   0.1099,
                                   -0.4229,
                                   3,
                                   416.5757,
                                   45,
                                   34002,
                                   303,
                                   {7, 7, 0},
                                   {6, 6, 1}},
                                  {4218165.8578,
                                   0.1637,
                                   -0.8763,
                                   3,
                                   505.7564,
                                   51,
                                   34002,
                                   303,
                                   {8, 6, 3},
                                   {7, 5, 2}},
                                  {4223880.1892,
                                   109.9903,
                                   -5.7893,
                                   3,
                                   2750.4486,
                                   41,
                                   34002,
                                   303,
                                   {20, 5, 15},
                                   {20, 4, 16}},
                                  {4223880.3520,
                                   109.9906,
                                   -5.3122,
                                   3,
                                   2750.4486,
                                   123,
                                   34002,
                                   303,
                                   {20, 6, 15},
                                   {20, 5, 16}},
                                  {4229839.7683,
                                   55.0287,
                                   -6.2484,
                                   3,
                                   2253.1366,
                                   33,
                                   34002,
                                   303,
                                   {16, 11, 5},
                                   {16, 8, 8}},
                                  {4243641.2553,
                                   0.1077,
                                   -5.0835,
                                   3,
                                   415.4772,
                                   45,
                                   34002,
                                   303,
                                   {7, 7, 0},
                                   {7, 2, 5}},
                                  {4263098.9042,
                                   0.3046,
                                   -2.2764,
                                   3,
                                   818.6985,
                                   21,
                                   34002,
                                   303,
                                   {10, 6, 4},
                                   {9, 7, 3}},
                                  {4281196.7679,
                                   3.8113,
                                   -6.0482,
                                   3,
                                   1404.1081,
                                   75,
                                   34002,
                                   303,
                                   {12, 12, 1},
                                   {12, 9, 4}},
                                  {4282855.3721,
                                   1.0001,
                                   -4.7170,
                                   3,
                                   1307.3605,
                                   75,
                                   34002,
                                   303,
                                   {12, 10, 3},
                                   {12, 7, 6}},
                                  {4290887.0751,
                                   65.8590,
                                   -4.6171,
                                   3,
                                   2379.3127,
                                   117,
                                   34002,
                                   303,
                                   {19, 4, 15},
                                   {19, 3, 16}},
                                  {4290887.1018,
                                   65.8590,
                                   -5.0942,
                                   3,
                                   2379.3127,
                                   39,
                                   34002,
                                   303,
                                   {19, 5, 15},
                                   {19, 4, 16}},
                                  {4292423.3656,
                                   21.6771,
                                   -6.2089,
                                   3,
                                   2107.6055,
                                   93,
                                   34002,
                                   303,
                                   {15, 13, 2},
                                   {15, 10, 5}},
                                  {4322128.4560,
                                   0.2819,
                                   -1.8518,
                                   3,
                                   781.2679,
                                   21,
                                   34002,
                                   303,
                                   {10, 5, 5},
                                   {9, 6, 4}},
                                  {4333239.7583,
                                   0.1046,
                                   -1.2521,
                                   3,
                                   243.3434,
                                   39,
                                   34002,
                                   303,
                                   {6, 5, 2},
                                   {5, 2, 3}},
                                  {4350798.2448,
                                   2.2272,
                                   -3.8362,
                                   3,
                                   1265.4171,
                                   25,
                                   34002,
                                   303,
                                   {12, 9, 3},
                                   {11, 10, 2}},
                                  {4357828.0287,
                                   42.4895,
                                   -4.4586,
                                   3,
                                   2023.6312,
                                   37,
                                   34002,
                                   303,
                                   {18, 3, 15},
                                   {18, 2, 16}},
                                  {4357828.0323,
                                   42.4895,
                                   -3.9815,
                                   3,
                                   2023.6312,
                                   111,
                                   34002,
                                   303,
                                   {18, 4, 15},
                                   {18, 3, 16}},
                                  {4366870.6698,
                                   269.4715,
                                   -7.5291,
                                   3,
                                   3221.2261,
                                   117,
                                   34002,
                                   303,
                                   {19, 14, 5},
                                   {19, 11, 8}},
                                  {4372762.7286,
                                   0.2395,
                                   -1.5868,
                                   3,
                                   635.4082,
                                   19,
                                   34002,
                                   303,
                                   {9, 6, 4},
                                   {8, 5, 3}},
                                  {4374071.2822,
                                   34.1167,
                                   -1.3425,
                                   3,
                                   1058.9274,
                                   93,
                                   34002,
                                   303,
                                   {15, 0, 15},
                                   {14, 1, 14}},
                                  {4374071.2822,
                                   34.1167,
                                   -1.8196,
                                   3,
                                   1058.9274,
                                   31,
                                   34002,
                                   303,
                                   {15, 1, 15},
                                   {14, 0, 14}},
                                  {4381912.0150,
                                   2.7664,
                                   -1.8570,
                                   3,
                                   1043.5618,
                                   29,
                                   34002,
                                   303,
                                   {14, 1, 13},
                                   {13, 2, 12}},
                                  {4381912.0173,
                                   2.7664,
                                   -1.3799,
                                   3,
                                   1043.5618,
                                   87,
                                   34002,
                                   303,
                                   {14, 2, 13},
                                   {13, 1, 12}},
                                  {4384473.0578,
                                   0.1115,
                                   -1.7356,
                                   3,
                                   210.2648,
                                   13,
                                   34002,
                                   303,
                                   {6, 3, 3},
                                   {5, 2, 4}},
                                  {4385909.1544,
                                   0.6744,
                                   -1.3865,
                                   3,
                                   1007.6266,
                                   81,
                                   34002,
                                   303,
                                   {13, 2, 11},
                                   {12, 3, 10}},
                                  {4385909.6874,
                                   0.6743,
                                   -1.8636,
                                   3,
                                   1007.6266,
                                   27,
                                   34002,
                                   303,
                                   {13, 3, 11},
                                   {12, 2, 10}},
                                  {4388668.4521,
                                   0.1180,
                                   -0.9211,
                                   3,
                                   359.3662,
                                   45,
                                   34002,
                                   303,
                                   {7, 5, 2},
                                   {6, 4, 3}},
                                  {4389170.8032,
                                   0.4486,
                                   -1.8452,
                                   3,
                                   951.6482,
                                   25,
                                   34002,
                                   303,
                                   {12, 3, 9},
                                   {11, 4, 8}},
                                  {4389243.2578,
                                   0.4450,
                                   -1.3681,
                                   3,
                                   951.6463,
                                   75,
                                   34002,
                                   303,
                                   {12, 4, 9},
                                   {11, 3, 8}},
                                  {4390259.2020,
                                   0.3293,
                                   -4.5150,
                                   3,
                                   814.4569,
                                   21,
                                   34002,
                                   303,
                                   {10, 6, 4},
                                   {10, 3, 7}},
                                  {4393173.6037,
                                   0.3138,
                                   -1.3371,
                                   3,
                                   875.7862,
                                   69,
                                   34002,
                                   303,
                                   {11, 4, 7},
                                   {10, 5, 6}},
                                  {4393310.9534,
                                   20.1151,
                                   -5.3878,
                                   3,
                                   2107.6055,
                                   99,
                                   34002,
                                   303,
                                   {16, 9, 8},
                                   {15, 10, 5}},
                                  {4398941.0001,
                                   0.2866,
                                   -1.8124,
                                   3,
                                   875.6318,
                                   23,
                                   34002,
                                   303,
                                   {11, 5, 7},
                                   {10, 4, 6}},
                                  {4424884.3815,
                                   53.4105,
                                   -3.4292,
                                   3,
                                   1683.7465,
                                   105,
                                   34002,
                                   303,
                                   {17, 2, 15},
                                   {17, 1, 16}},
                                  {4424884.3819,
                                   53.4105,
                                   -3.9063,
                                   3,
                                   1683.7465,
                                   35,
                                   34002,
                                   303,
                                   {17, 3, 15},
                                   {17, 2, 16}},
                                  {4426990.3565,
                                   12.4594,
                                   -6.7428,
                                   3,
                                   1652.5842,
                                   81,
                                   34002,
                                   303,
                                   {13, 13, 0},
                                   {13, 10, 3}},
                                  {4428942.4477,
                                   25.3949,
                                   -5.4039,
                                   3,
                                   1959.8719,
                                   93,
                                   34002,
                                   303,
                                   {15, 10, 5},
                                   {15, 7, 8}},
                                  {4444700.0116,
                                   0.1018,
                                   -2.5753,
                                   3,
                                   148.4183,
                                   33,
                                   34002,
                                   303,
                                   {5, 5, 0},
                                   {4, 2, 3}},
                                  {4446865.8032,
                                   26.3124,
                                   -6.8013,
                                   3,
                                   1871.5272,
                                   29,
                                   34002,
                                   303,
                                   {14, 13, 1},
                                   {14, 10, 4}},
                                  {4471370.7531,
                                   0.1112,
                                   -1.2709,
                                   3,
                                   210.2173,
                                   39,
                                   34002,
                                   303,
                                   {6, 4, 3},
                                   {5, 1, 4}},
                                  {4473612.4133,
                                   0.1439,
                                   -5.7445,
                                   3,
                                   567.6967,
                                   17,
                                   34002,
                                   303,
                                   {8, 8, 0},
                                   {8, 3, 5}},
                                  {4476748.7960,
                                   3.8781,
                                   -5.6820,
                                   3,
                                   1549.2698,
                                   27,
                                   34002,
                                   303,
                                   {13, 11, 3},
                                   {13, 8, 6}},
                                  {4477699.4211,
                                   191.4206,
                                   -6.2352,
                                   3,
                                   2987.3350,
                                   43,
                                   34002,
                                   303,
                                   {21, 6, 16},
                                   {21, 5, 17}},
                                  {4477700.4140,
                                   191.4190,
                                   -5.7581,
                                   3,
                                   2987.3350,
                                   129,
                                   34002,
                                   303,
                                   {21, 5, 16},
                                   {21, 4, 17}},
                                  {4477700.4141,
                                   191.4190,
                                   -5.7581,
                                   3,
                                   2987.3350,
                                   129,
                                   34002,
                                   303,
                                   {21, 5, 16},
                                   {21, 4, 17}},
                                  {4478804.2826,
                                   2.3877,
                                   -4.0762,
                                   3,
                                   1451.1036,
                                   27,
                                   34002,
                                   303,
                                   {13, 9, 5},
                                   {12, 10, 2}},
                                  {4483808.5894,
                                   10.9755,
                                   -6.2345,
                                   3,
                                   1600.5004,
                                   27,
                                   34002,
                                   303,
                                   {13, 12, 2},
                                   {13, 9, 5}},
                                  {4492109.4704,
                                   115.2867,
                                   -3.0345,
                                   3,
                                   1360.0023,
                                   99,
                                   34002,
                                   303,
                                   {16, 2, 15},
                                   {16, 1, 16}},
                                  {4492109.4704,
                                   115.2867,
                                   -3.5117,
                                   3,
                                   1360.0023,
                                   33,
                                   34002,
                                   303,
                                   {16, 1, 15},
                                   {16, 0, 16}},
                                  {4508025.0177,
                                   5.4196,
                                   -4.3544,
                                   3,
                                   1498.5400,
                                   27,
                                   34002,
                                   303,
                                   {13, 10, 4},
                                   {12, 11, 1}},
                                  {4518634.3011,
                                   0.3065,
                                   -1.2943,
                                   3,
                                   776.3768,
                                   63,
                                   34002,
                                   303,
                                   {10, 6, 5},
                                   {9, 5, 4}},
                                  {4537059.7446,
                                   12.3181,
                                   -7.2879,
                                   3,
                                   1648.9115,
                                   27,
                                   34002,
                                   303,
                                   {13, 13, 1},
                                   {13, 10, 4}},
                                  {4547321.7468,
                                   4.6074,
                                   -4.8520,
                                   3,
                                   1546.9158,
                                   27,
                                   34002,
                                   303,
                                   {13, 11, 3},
                                   {12, 12, 0}},
                                  {4550986.1681,
                                   118.8817,
                                   -5.5035,
                                   3,
                                   2598.6441,
                                   41,
                                   34002,
                                   303,
                                   {20, 4, 16},
                                   {20, 3, 17}},
                                  {4550986.1758,
                                   118.8817,
                                   -5.0264,
                                   3,
                                   2598.6441,
                                   123,
                                   34002,
                                   303,
                                   {20, 5, 16},
                                   {20, 4, 17}},
                                  {4555765.2159,
                                   0.2247,
                                   -4.0171,
                                   3,
                                   624.4128,
                                   57,
                                   34002,
                                   303,
                                   {9, 5, 4},
                                   {9, 2, 7}},
                                  {4557670.6647,
                                   0.5910,
                                   -2.2772,
                                   3,
                                   1010.1115,
                                   69,
                                   34002,
                                   303,
                                   {11, 7, 4},
                                   {10, 8, 3}},
                                  {4560394.7380,
                                   4.7092,
                                   -4.3693,
                                   3,
                                   1546.9134,
                                   81,
                                   34002,
                                   303,
                                   {13, 11, 2},
                                   {12, 12, 1}},
                                  {4581930.6383,
                                   0.1007,
                                   -1.8827,
                                   3,
                                   263.7389,
                                   39,
                                   34002,
                                   303,
                                   {6, 6, 1},
                                   {5, 3, 2}},
                                  {4590204.7450,
                                   61.3339,
                                   -7.0551,
                                   3,
                                   2658.7833,
                                   105,
                                   34002,
                                   303,
                                   {17, 14, 3},
                                   {17, 11, 6}},
                                  {4611144.9239,
                                   0.1612,
                                   -0.6784,
                                   3,
                                   526.6533,
                                   51,
                                   34002,
                                   303,
                                   {8, 7, 2},
                                   {7, 6, 1}},
                                  {4616975.0384,
                                   371.2169,
                                   -7.3981,
                                   3,
                                   3157.9475,
                                   117,
                                   34002,
                                   303,
                                   {19, 13, 6},
                                   {19, 10, 9}},
                                  {4620068.8398,
                                   5.6633,
                                   -3.8316,
                                   3,
                                   1498.4753,
                                   81,
                                   34002,
                                   303,
                                   {13, 10, 3},
                                   {12, 11, 2}},
                                  {4625086.8174,
                                   81.5308,
                                   -4.3534,
                                   3,
                                   2225.0364,
                                   117,
                                   34002,
                                   303,
                                   {19, 3, 16},
                                   {19, 2, 17}},
                                  {4625086.8183,
                                   81.5308,
                                   -4.8305,
                                   3,
                                   2225.0364,
                                   39,
                                   34002,
                                   303,
                                   {19, 4, 16},
                                   {19, 3, 17}},
                                  {4649753.0469,
                                   8.7336,
                                   -5.0693,
                                   3,
                                   1871.5272,
                                   31,
                                   34002,
                                   303,
                                   {15, 9, 7},
                                   {14, 10, 4}},
                                  {4651926.6832,
                                   63.9515,
                                   -1.5723,
                                   3,
                                   1204.8307,
                                   99,
                                   34002,
                                   303,
                                   {16, 1, 16},
                                   {15, 0, 15}},
                                  {4651926.6832,
                                   63.9515,
                                   -2.0495,
                                   3,
                                   1204.8307,
                                   33,
                                   34002,
                                   303,
                                   {16, 0, 16},
                                   {15, 1, 15}},
                                  {4653264.7148,
                                   0.4800,
                                   -1.4983,
                                   3,
                                   927.1022,
                                   69,
                                   34002,
                                   303,
                                   {11, 5, 6},
                                   {10, 6, 5}},
                                  {4659817.0668,
                                   6.5306,
                                   -1.6057,
                                   3,
                                   1189.7267,
                                   93,
                                   34002,
                                   303,
                                   {15, 1, 14},
                                   {14, 2, 13}},
                                  {4659817.0673,
                                   6.5306,
                                   -2.0828,
                                   3,
                                   1189.7267,
                                   31,
                                   34002,
                                   303,
                                   {15, 2, 14},
                                   {14, 1, 13}},
                                  {4663111.2833,
                                   1.8391,
                                   -2.0835,
                                   3,
                                   1153.9248,
                                   29,
                                   34002,
                                   303,
                                   {14, 2, 12},
                                   {13, 3, 11}},
                                  {4663111.4043,
                                   1.8391,
                                   -1.6064,
                                   3,
                                   1153.9248,
                                   87,
                                   34002,
                                   303,
                                   {14, 3, 12},
                                   {13, 2, 11}},
                                  {4663171.5552,
                                   0.7726,
                                   -4.7980,
                                   3,
                                   1022.3647,
                                   23,
                                   34002,
                                   303,
                                   {11, 8, 4},
                                   {11, 5, 7}},
                                  {4665162.2611,
                                   0.8704,
                                   -1.5791,
                                   3,
                                   1098.0557,
                                   81,
                                   34002,
                                   303,
                                   {13, 3, 10},
                                   {12, 4, 9}},
                                  {4665178.8142,
                                   0.8680,
                                   -2.0562,
                                   3,
                                   1098.0552,
                                   27,
                                   34002,
                                   303,
                                   {13, 4, 10},
                                   {12, 3, 9}},
                                  {4668664.1980,
                                   0.5408,
                                   -2.0094,
                                   3,
                                   1022.3647,
                                   25,
                                   34002,
                                   303,
                                   {12, 4, 8},
                                   {11, 5, 7}},
                                  {4670082.0410,
                                   0.5174,
                                   -1.5318,
                                   3,
                                   1022.3267,
                                   75,
                                   34002,
                                   303,
                                   {12, 5, 8},
                                   {11, 4, 7}},
                                  {4672210.1415,
                                   0.4067,
                                   -4.1357,
                                   3,
                                   814.4658,
                                   63,
                                   34002,
                                   303,
                                   {10, 7, 4},
                                   {10, 4, 7}},
                                  {4672420.4439,
                                   4.0945,
                                   -3.9749,
                                   3,
                                   1652.5842,
                                   87,
                                   34002,
                                   303,
                                   {14, 9, 6},
                                   {13, 10, 3}},
                                  {4678046.0115,
                                   9.6444,
                                   -5.5779,
                                   3,
                                   1682.6052,
                                   29,
                                   34002,
                                   303,
                                   {14, 9, 5},
                                   {14, 6, 8}},
                                  {4678887.4188,
                                   0.1584,
                                   -4.5907,
                                   3,
                                   452.4943,
                                   17,
                                   34002,
                                   303,
                                   {8, 4, 4},
                                   {8, 1, 7}},
                                  {4685587.2200,
                                   1.1910,
                                   -4.6037,
                                   3,
                                   1247.8137,
                                   75,
                                   34002,
                                   303,
                                   {12, 9, 4},
                                   {12, 6, 7}},
                                  {4700288.9835,
                                   107.5601,
                                   -4.2401,
                                   3,
                                   1866.8464,
                                   37,
                                   34002,
                                   303,
                                   {18, 2, 16},
                                   {18, 1, 17}},
                                  {4700288.9836,
                                   107.5601,
                                   -3.7629,
                                   3,
                                   1866.8464,
                                   111,
                                   34002,
                                   303,
                                   {18, 3, 16},
                                   {18, 2, 17}},
                                  {4702347.7377,
                                   0.2466,
                                   -4.5384,
                                   3,
                                   624.4145,
                                   19,
                                   34002,
                                   303,
                                   {9, 6, 4},
                                   {9, 3, 7}},
                                  {4709279.3985,
                                   11.7683,
                                   -5.7810,
                                   3,
                                   1808.4393,
                                   87,
                                   34002,
                                   303,
                                   {14, 12, 3},
                                   {14, 9, 6}},
                                  {4716997.0216,
                                   0.1542,
                                   -1.1482,
                                   3,
                                   524.3446,
                                   17,
                                   34002,
                                   303,
                                   {8, 7, 1},
                                   {7, 6, 2}},
                                  {4717801.7954,
                                   0.4126,
                                   -1.9543,
                                   3,
                                   925.4386,
                                   23,
                                   34002,
                                   303,
                                   {11, 6, 6},
                                   {10, 5, 5}},
                                  {4743602.0079,
                                   0.1637,
                                   -4.1311,
                                   3,
                                   452.4945,
                                   51,
                                   34002,
                                   303,
                                   {8, 5, 4},
                                   {8, 2, 7}},
                                  {4746772.0970,
                                   0.2761,
                                   -5.6440,
                                   3,
                                   737.8607,
                                   57,
                                   34002,
                                   303,
                                   {9, 9, 0},
                                   {9, 4, 5}},
                                  {4748602.2390,
                                   2.6499,
                                   -5.4558,
                                   3,
                                   1490.5152,
                                   27,
                                   34002,
                                   303,
                                   {13, 10, 4},
                                   {13, 7, 7}},
                                  {4753918.9871,
                                   0.1170,
                                   -2.0655,
                                   3,
                                   166.3442,
                                   13,
                                   34002,
                                   303,
                                   {6, 2, 4},
                                   {5, 1, 5}},
                                  {4760790.3314,
                                   0.1174,
                                   -1.5885,
                                   3,
                                   166.3435,
                                   39,
                                   34002,
                                   303,
                                   {6, 3, 4},
                                   {5, 0, 5}},
                                  {4762734.2046,
                                   78.6034,
                                   -6.4747,
                                   3,
                                   2658.7833,
                                   111,
                                   34002,
                                   303,
                                   {18, 10, 9},
                                   {17, 11, 6}},
                                  {4763999.6861,
                                   0.1165,
                                   -4.4422,
                                   3,
                                   298.9696,
                                   45,
                                   34002,
                                   303,
                                   {7, 3, 4},
                                   {7, 0, 7}},
                                  {4766945.6525,
                                   26.4559,
                                   -6.4511,
                                   3,
                                   1860.8363,
                                   87,
                                   34002,
                                   303,
                                   {14, 13, 2},
                                   {14, 10, 5}},
                                  {4776718.5482,
                                   214.5114,
                                   -3.3294,
                                   3,
                                   1524.4124,
                                   105,
                                   34002,
                                   303,
                                   {17, 1, 16},
                                   {17, 0, 17}},
                                  {4776718.5482,
                                   214.5114,
                                   -3.8065,
                                   3,
                                   1524.4124,
                                   35,
                                   34002,
                                   303,
                                   {17, 2, 16},
                                   {17, 1, 17}},
                                  {4780196.1581,
                                   0.6163,
                                   -1.7567,
                                   3,
                                   970.3139,
                                   69,
                                   34002,
                                   303,
                                   {11, 6, 5},
                                   {10, 7, 4}},
                                  {4787654.2504,
                                   0.1165,
                                   -4.9254,
                                   3,
                                   298.9696,
                                   15,
                                   34002,
                                   303,
                                   {7, 4, 4},
                                   {7, 1, 7}},
                                  {4789933.5007,
                                   228.1609,
                                   -7.3592,
                                   3,
                                   2816.6988,
                                   37,
                                   34002,
                                   303,
                                   {18, 12, 6},
                                   {18, 9, 9}},
                                  {4791736.6714,
                                   0.1706,
                                   -0.5515,
                                   3,
                                   557.0299,
                                   51,
                                   34002,
                                   303,
                                   {8, 8, 1},
                                   {7, 7, 0}},
                                  {4792258.2593,
                                   1.1779,
                                   -3.2741,
                                   3,
                                   1220.6591,
                                   25,
                                   34002,
                                   303,
                                   {12, 8, 4},
                                   {11, 9, 3}},
                                  {4797092.7418,
                                   14.9896,
                                   -1.0280,
                                   3,
                                   556.9065,
                                   17,
                                   -34002,
                                   303,
                                   {8, 8, 0},
                                   {7, 7, 1}},
                                  {4808110.3078,
                                   205.7699,
                                   -5.4578,
                                   3,
                                   2826.9537,
                                   129,
                                   34002,
                                   303,
                                   {21, 4, 17},
                                   {21, 3, 18}},
                                  {4808110.3078,
                                   205.7699,
                                   -5.4578,
                                   3,
                                   2826.9537,
                                   129,
                                   34002,
                                   303,
                                   {21, 4, 17},
                                   {21, 3, 18}},
                                  {4808110.4559,
                                   205.7696,
                                   -5.9349,
                                   3,
                                   2826.9537,
                                   43,
                                   34002,
                                   303,
                                   {21, 5, 17},
                                   {21, 4, 18}},
                                  {4850775.6215,
                                   5.2360,
                                   -4.0319,
                                   3,
                                   1699.0318,
                                   87,
                                   34002,
                                   303,
                                   {14, 10, 5},
                                   {13, 11, 2}},
                                  {4851414.2358,
                                   14.9896,
                                   -1.4015,
                                   3,
                                   493.3685,
                                   17,
                                   -34002,
                                   303,
                                   {8, 6, 2},
                                   {7, 5, 3}},
                                  {4852966.5334,
                                   59.0170,
                                   -7.1508,
                                   3,
                                   2145.5974,
                                   93,
                                   34002,
                                   303,
                                   {15, 14, 1},
                                   {15, 11, 4}},
                                  {4858541.2732,
                                   8.4392,
                                   -5.4432,
                                   3,
                                   1750.1938,
                                   87,
                                   34002,
                                   303,
                                   {14, 11, 4},
                                   {14, 8, 7}},
                                  {4859297.2785,
                                   14.9896,
                                   -1.1727,
                                   3,
                                   325.1463,
                                   45,
                                   -34002,
                                   303,
                                   {7, 4, 3},
                                   {6, 3, 4}},
                                  {4861520.2591,
                                   12.9706,
                                   -4.3527,
                                   3,
                                   1750.0944,
                                   87,
                                   34002,
                                   303,
                                   {14, 11, 4},
                                   {13, 12, 1}},
                                  {4889930.6448,
                                   150.9607,
                                   -5.2248,
                                   3,
                                   2435.5336,
                                   41,
                                   34002,
                                   303,
                                   {20, 3, 17},
                                   {20, 2, 18}},
                                  {4889930.6450,
                                   150.9607,
                                   -4.7477,
                                   3,
                                   2435.5336,
                                   123,
                                   34002,
                                   303,
                                   {20, 4, 17},
                                   {20, 3, 18}},
                                  {4891412.1245,
                                   0.1163,
                                   -3.3612,
                                   3,
                                   114.1776,
                                   33,
                                   34002,
                                   303,
                                   {5, 4, 1},
                                   {4, 1, 4}},
                                  {4901732.0015,
                                   14.9896,
                                   -1.4748,
                                   3,
                                   655.1942,
                                   19,
                                   -34002,
                                   303,
                                   {9, 7, 3},
                                   {8, 6, 2}},
                                  {4914877.2388,
                                   0.1155,
                                   -6.4102,
                                   3,
                                   227.9449,
                                   13,
                                   34002,
                                   303,
                                   {6, 5, 1},
                                   {6, 0, 6}},
                                  {4921090.7117,
                                   13.7587,
                                   -4.8068,
                                   3,
                                   1750.0642,
                                   29,
                                   34002,
                                   303,
                                   {14, 11, 3},
                                   {13, 12, 2}},
                                  {4925645.0115,
                                   0.1248,
                                   -5.3678,
                                   3,
                                   362.3515,
                                   45,
                                   34002,
                                   303,
                                   {7, 6, 1},
                                   {7, 1, 6}},
                                  {4928890.2273,
                                   115.9063,
                                   -1.8264,
                                   3,
                                   1360.0023,
                                   105,
                                   34002,
                                   303,
                                   {17, 0, 17},
                                   {16, 1, 16}},
                                  {4928890.2273,
                                   115.9063,
                                   -2.3036,
                                   3,
                                   1360.0023,
                                   35,
                                   34002,
                                   303,
                                   {17, 1, 17},
                                   {16, 0, 16}},
                                  {4932743.4832,
                                   3.2726,
                                   -4.8421,
                                   3,
                                   1421.8697,
                                   81,
                                   34002,
                                   303,
                                   {13, 8, 5},
                                   {13, 5, 8}},
                                  {4937026.8182,
                                   14.8093,
                                   -2.3334,
                                   3,
                                   1345.1614,
                                   33,
                                   34002,
                                   303,
                                   {16, 1, 15},
                                   {15, 2, 14}},
                                  {4937030.7647,
                                   14.9896,
                                   -1.8563,
                                   3,
                                   1345.1614,
                                   99,
                                   -34002,
                                   303,
                                   {16, 2, 15},
                                   {15, 1, 14}},
                                  {4939624.2593,
                                   4.7682,
                                   -1.8520,
                                   3,
                                   1309.4694,
                                   93,
                                   34002,
                                   303,
                                   {15, 2, 13},
                                   {14, 3, 12}},
                                  {4939624.2879,
                                   4.7682,
                                   -2.3291,
                                   3,
                                   1309.4694,
                                   31,
                                   34002,
                                   303,
                                   {15, 3, 13},
                                   {14, 2, 12}},
                                  {4940509.8562,
                                   14.9896,
                                   -1.8175,
                                   3,
                                   1253.6687,
                                   87,
                                   -34002,
                                   303,
                                   {14, 4, 11},
                                   {13, 3, 10}},
                                  {4940509.8562,
                                   14.9896,
                                   -2.2946,
                                   3,
                                   1253.6688,
                                   29,
                                   -34002,
                                   303,
                                   {14, 3, 11},
                                   {13, 4, 10}},
                                  {4942461.8049,
                                   14.9896,
                                   -1.7590,
                                   3,
                                   1178.1039,
                                   81,
                                   -34002,
                                   303,
                                   {13, 4, 9},
                                   {12, 5, 8}},
                                  {4942701.9387,
                                   14.9896,
                                   -2.1696,
                                   3,
                                   1082.8075,
                                   25,
                                   -34002,
                                   303,
                                   {12, 5, 7},
                                   {11, 6, 6}},
                                  {4942811.0631,
                                   14.9896,
                                   -2.2360,
                                   3,
                                   1178.0946,
                                   27,
                                   -34002,
                                   303,
                                   {13, 5, 9},
                                   {12, 4, 8}},
                                  {4954709.4858,
                                   15.0605,
                                   -4.8394,
                                   3,
                                   1800.2527,
                                   87,
                                   34002,
                                   303,
                                   {14, 12, 3},
                                   {13, 13, 0}},
                                  {4961002.3641,
                                   15.2381,
                                   -5.3138,
                                   3,
                                   1800.2516,
                                   29,
                                   34002,
                                   303,
                                   {14, 12, 2},
                                   {13, 13, 1}},
                                  {4961421.8791,
                                   14.9896,
                                   -1.6870,
                                   3,
                                   1082.3184,
                                   75,
                                   -34002,
                                   303,
                                   {12, 6, 7},
                                   {11, 5, 6}},
                                  {4971474.8212,
                                   29.8349,
                                   -6.9209,
                                   3,
                                   2084.8554,
                                   31,
                                   34002,
                                   303,
                                   {15, 13, 3},
                                   {15, 10, 6}},
                                  {4974185.6669,
                                   205.4303,
                                   -4.1192,
                                   3,
                                   2059.1154,
                                   117,
                                   34002,
                                   303,
                                   {19, 2, 17},
                                   {19, 1, 18}},
                                  {4974185.6670,
                                   205.4303,
                                   -4.5963,
                                   3,
                                   2059.1154,
                                   39,
                                   34002,
                                   303,
                                   {19, 3, 17},
                                   {19, 2, 18}},
                                  {4988488.7758,
                                   0.1656,
                                   -5.6578,
                                   3,
                                   515.2886,
                                   17,
                                   34002,
                                   303,
                                   {8, 7, 1},
                                   {8, 2, 6}},
                                  {4991150.7982,
                                   29.9792,
                                   -3.3373,
                                   3,
                                   1450.2212,
                                   81,
                                   -34002,
                                   303,
                                   {13, 9, 4},
                                   {12, 10, 3}},
                                  {5017589.9876,
                                   22.2530,
                                   -6.4722,
                                   3,
                                   2026.6263,
                                   31,
                                   34002,
                                   303,
                                   {15, 12, 4},
                                   {15, 9, 7}},
                                  {5031399.1349,
                                   14.9896,
                                   -1.8734,
                                   3,
                                   356.5151,
                                   15,
                                   -34002,
                                   303,
                                   {7, 6, 2},
                                   {6, 3, 3}},
                                  {5050021.0432,
                                   14.9896,
                                   -1.6951,
                                   3,
                                   324.9179,
                                   15,
                                   -34002,
                                   303,
                                   {7, 5, 3},
                                   {6, 2, 4}},
                                  {5055339.5191,
                                   123.1793,
                                   -6.4446,
                                   3,
                                   2490.1554,
                                   105,
                                   34002,
                                   303,
                                   {17, 11, 6},
                                   {17, 8, 9}},
                                  {5061104.7271,
                                   384.0569,
                                   -3.6467,
                                   3,
                                   1698.0261,
                                   111,
                                   34002,
                                   303,
                                   {18, 2, 17},
                                   {18, 1, 18}},
                                  {5061104.7271,
                                   384.0569,
                                   -4.1238,
                                   3,
                                   1698.0261,
                                   37,
                                   34002,
                                   303,
                                   {18, 1, 17},
                                   {18, 0, 18}},
                                  {5067360.7392,
                                   14.9896,
                                   -1.3164,
                                   3,
                                   801.2849,
                                   63,
                                   -34002,
                                   303,
                                   {10, 7, 4},
                                   {9, 6, 3}},
                                  {5068989.1626,
                                   0.7010,
                                   -6.7126,
                                   3,
                                   925.4386,
                                   21,
                                   34002,
                                   303,
                                   {10, 10, 0},
                                   {10, 5, 5}},
                                  {5095009.1441,
                                   36.0630,
                                   -6.0972,
                                   3,
                                   2394.2289,
                                   35,
                                   34002,
                                   303,
                                   {17, 10, 8},
                                   {16, 11, 5}},
                                  {5104484.0159,
                                   0.1986,
                                   -5.2438,
                                   3,
                                   686.4885,
                                   57,
                                   34002,
                                   303,
                                   {9, 8, 1},
                                   {9, 3, 6}},
                                  {5115696.2812,
                                   9.1370,
                                   -4.8382,
                                   3,
                                   1914.2141,
                                   31,
                                   34002,
                                   303,
                                   {15, 10, 6},
                                   {14, 11, 3}},
                                  {5152119.1587,
                                   270.4977,
                                   -5.6410,
                                   3,
                                   2655.0975,
                                   43,
                                   34002,
                                   303,
                                   {21, 4, 18},
                                   {21, 3, 19}},
                                  {5152119.1749,
                                   270.4977,
                                   -5.1639,
                                   3,
                                   2655.0975,
                                   129,
                                   34002,
                                   303,
                                   {21, 3, 18},
                                   {21, 2, 19}},
                                  {5152119.1749,
                                   270.4977,
                                   -5.1639,
                                   3,
                                   2655.0975,
                                   129,
                                   34002,
                                   303,
                                   {21, 3, 18},
                                   {21, 2, 19}},
                                  {5159325.4434,
                                   1.5043,
                                   -5.1017,
                                   3,
                                   1178.0946,
                                   25,
                                   34002,
                                   303,
                                   {12, 7, 5},
                                   {12, 4, 8}},
                                  {5164816.6718,
                                   29.9792,
                                   -2.7026,
                                   3,
                                   1177.9114,
                                   25,
                                   -34002,
                                   303,
                                   {12, 7, 5},
                                   {11, 8, 4}},
                                  {5166986.2698,
                                   14.9896,
                                   -2.3580,
                                   3,
                                   1133.6702,
                                   25,
                                   -34002,
                                   303,
                                   {12, 6, 6},
                                   {11, 7, 5}},
                                  {5170033.0606,
                                   14.9896,
                                   -2.9340,
                                   3,
                                   244.3925,
                                   13,
                                   -34002,
                                   303,
                                   {6, 6, 0},
                                   {5, 3, 3}},
                                  {5179510.9991,
                                   14.9896,
                                   -2.0641,
                                   3,
                                   960.9002,
                                   23,
                                   -34002,
                                   303,
                                   {11, 7, 5},
                                   {10, 6, 4}},
                                  {5184283.4284,
                                   7.6355,
                                   -4.3663,
                                   3,
                                   1698.5981,
                                   29,
                                   34002,
                                   303,
                                   {14, 10, 4},
                                   {13, 11, 3}},
                                  {5199743.2654,
                                   14.4790,
                                   -5.0070,
                                   3,
                                   1965.7328,
                                   31,
                                   34002,
                                   303,
                                   {15, 11, 5},
                                   {14, 12, 2}},
                                  {5204809.1502,
                                   203.6800,
                                   -2.1042,
                                   3,
                                   1524.4124,
                                   111,
                                   34002,
                                   303,
                                   {18, 1, 18},
                                   {17, 0, 17}},
                                  {5204809.1502,
                                   203.6800,
                                   -2.5813,
                                   3,
                                   1524.4124,
                                   37,
                                   34002,
                                   303,
                                   {18, 0, 18},
                                   {17, 1, 17}},
                                  {5213499.3050,
                                   31.7686,
                                   -2.1311,
                                   3,
                                   1509.8429,
                                   105,
                                   34002,
                                   303,
                                   {17, 1, 16},
                                   {16, 2, 15}},
                                  {5213499.3050,
                                   31.7686,
                                   -2.6082,
                                   3,
                                   1509.8429,
                                   35,
                                   34002,
                                   303,
                                   {17, 2, 16},
                                   {16, 1, 15}},
                                  {5215144.9303,
                                   14.9896,
                                   -2.0819,
                                   3,
                                   1418.4664,
                                   93,
                                   -34002,
                                   303,
                                   {15, 3, 12},
                                   {14, 4, 11}},
                                  {5215144.9303,
                                   14.9896,
                                   -2.5590,
                                   3,
                                   1418.4664,
                                   31,
                                   -34002,
                                   303,
                                   {15, 4, 12},
                                   {14, 3, 11}},
                                  {5215401.7676,
                                   11.2674,
                                   -2.5996,
                                   3,
                                   1474.2376,
                                   33,
                                   34002,
                                   303,
                                   {16, 2, 14},
                                   {15, 3, 13}},
                                  {5215401.7747,
                                   11.2674,
                                   -2.1225,
                                   3,
                                   1474.2376,
                                   99,
                                   34002,
                                   303,
                                   {16, 3, 14},
                                   {15, 2, 13}},
                                  {5215444.8050,
                                   1.6569,
                                   -2.4913,
                                   3,
                                   1342.9691,
                                   29,
                                   34002,
                                   303,
                                   {14, 4, 10},
                                   {13, 5, 9}},
                                  {5215531.3628,
                                   14.9896,
                                   -2.0142,
                                   3,
                                   1342.9668,
                                   87,
                                   -34002,
                                   303,
                                   {14, 5, 10},
                                   {13, 4, 9}},
                                  {5218064.6090,
                                   14.9896,
                                   -1.9291,
                                   3,
                                   1247.8137,
                                   81,
                                   -34002,
                                   303,
                                   {13, 5, 8},
                                   {12, 6, 7}},
                                  {5218585.7865,
                                   17.2413,
                                   -4.8893,
                                   3,
                                   2145.5974,
                                   99,
                                   34002,
                                   303,
                                   {16, 10, 7},
                                   {15, 11, 4}},
                                  {5220477.2313,
                                   30.1531,
                                   -5.3461,
                                   3,
                                   2019.8587,
                                   31,
                                   34002,
                                   303,
                                   {15, 12, 4},
                                   {14, 13, 1}},
                                  {5223075.4394,
                                   50.0392,
                                   -6.6323,
                                   3,
                                   2319.6707,
                                   99,
                                   34002,
                                   303,
                                   {16, 13, 4},
                                   {16, 10, 7}},
                                  {5223221.9387,
                                   14.9896,
                                   -2.4048,
                                   3,
                                   1247.6784,
                                   27,
                                   -34002,
                                   303,
                                   {13, 6, 8},
                                   {12, 5, 7}},
                                  {5230148.0439,
                                   14.9896,
                                   -1.3049,
                                   3,
                                   681.6867,
                                   19,
                                   -34002,
                                   303,
                                   {9, 8, 2},
                                   {8, 7, 1}},
                                  {5246607.1221,
                                   374.9504,
                                   -4.9745,
                                   3,
                                   2260.5256,
                                   41,
                                   34002,
                                   303,
                                   {20, 2, 18},
                                   {20, 1, 19}},
                                  {5246607.1222,
                                   374.9504,
                                   -4.4974,
                                   3,
                                   2260.5256,
                                   123,
                                   34002,
                                   303,
                                   {20, 3, 18},
                                   {20, 2, 19}},
                                  {5250714.0684,
                                   31.5313,
                                   -4.8575,
                                   3,
                                   2019.8445,
                                   93,
                                   34002,
                                   303,
                                   {15, 12, 3},
                                   {14, 13, 2}},
                                  {5251662.1416,
                                   68.0977,
                                   -7.1972,
                                   3,
                                   2378.5644,
                                   99,
                                   34002,
                                   303,
                                   {16, 14, 3},
                                   {16, 11, 6}},
                                  {5272524.9526,
                                   0.4145,
                                   -5.9819,
                                   3,
                                   875.6318,
                                   21,
                                   34002,
                                   303,
                                   {10, 9, 1},
                                   {10, 4, 6}},
                                  {5284660.5057,
                                   14.9896,
                                   -1.3417,
                                   3,
                                   281.6023,
                                   45,
                                   -34002,
                                   303,
                                   {7, 3, 4},
                                   {6, 2, 5}},
                                  {5285074.5190,
                                   14.9896,
                                   -0.8229,
                                   3,
                                   680.4646,
                                   57,
                                   -34002,
                                   303,
                                   {9, 8, 1},
                                   {8, 7, 2}},
                                  {5298518.7118,
                                   14.9896,
                                   -1.6659,
                                   3,
                                   458.6686,
                                   17,
                                   -34002,
                                   303,
                                   {8, 5, 3},
                                   {7, 4, 4}},
                                  {5308544.3712,
                                   14.9896,
                                   -1.8199,
                                   3,
                                   281.5945,
                                   15,
                                   -34002,
                                   303,
                                   {7, 4, 4},
                                   {6, 1, 5}},
                                  {5324204.3300,
                                   14.9896,
                                   -1.8303,
                                   3,
                                   1129.7641,
                                   75,
                                   -34002,
                                   303,
                                   {12, 7, 6},
                                   {11, 6, 5}},
                                  {5330675.0505,
                                   14.9896,
                                   -1.0039,
                                   3,
                                   646.4593,
                                   57,
                                   -34002,
                                   303,
                                   {9, 7, 2},
                                   {8, 6, 3}},
                                  {5339491.5987,
                                   6.1434,
                                   -5.3643,
                                   3,
                                   1682.7300,
                                   87,
                                   34002,
                                   303,
                                   {14, 10, 5},
                                   {14, 7, 8}},
                                  {5339838.9223,
                                   0.7580,
                                   -4.4609,
                                   3,
                                   951.6463,
                                   69,
                                   34002,
                                   303,
                                   {11, 6, 5},
                                   {11, 3, 8}},
                                  {5345704.9210,
                                   664.2493,
                                   -3.9859,
                                   3,
                                   1880.8019,
                                   117,
                                   34002,
                                   303,
                                   {19, 1, 18},
                                   {19, 0, 19}},
                                  {5345704.9210,
                                   664.2493,
                                   -4.4630,
                                   3,
                                   1880.8019,
                                   39,
                                   34002,
                                   303,
                                   {19, 2, 18},
                                   {19, 1, 19}},
                                  {5354113.1928,
                                   2.1055,
                                   -5.4691,
                                   3,
                                   1421.9064,
                                   27,
                                   34002,
                                   303,
                                   {13, 9, 5},
                                   {13, 6, 8}},
                                  {5357379.6109,
                                   56.8724,
                                   -6.5331,
                                   3,
                                   2179.0136,
                                   33,
                                   34002,
                                   303,
                                   {16, 10, 6},
                                   {16, 7, 9}},
                                  {5364189.6112,
                                   18.7516,
                                   -6.2940,
                                   3,
                                   1960.2475,
                                   31,
                                   34002,
                                   303,
                                   {15, 11, 5},
                                   {15, 8, 8}},
                                  {5373807.9901,
                                   14.9896,
                                   -1.2210,
                                   3,
                                   716.9203,
                                   19,
                                   -34002,
                                   303,
                                   {9, 9, 1},
                                   {8, 8, 0}},
                                  {5376205.7302,
                                   14.9896,
                                   -0.7436,
                                   3,
                                   716.8650,
                                   57,
                                   -34002,
                                   303,
                                   {9, 9, 0},
                                   {8, 8, 1}},
                                  {5381289.6669,
                                   43.7871,
                                   -5.8145,
                                   3,
                                   2071.1854,
                                   31,
                                   34002,
                                   303,
                                   {15, 13, 3},
                                   {14, 14, 0}},
                                  {5384285.7115,
                                   43.9977,
                                   -5.3360,
                                   3,
                                   2071.1849,
                                   93,
                                   34002,
                                   303,
                                   {15, 13, 2},
                                   {14, 14, 1}},
                                  {5397025.4963,
                                   1.3273,
                                   -4.7008,
                                   3,
                                   1178.1039,
                                   75,
                                   34002,
                                   303,
                                   {12, 8, 5},
                                   {12, 5, 8}},
                                  {5398465.1073,
                                   19.8443,
                                   -4.4547,
                                   3,
                                   1965.5240,
                                   93,
                                   34002,
                                   303,
                                   {15, 11, 4},
                                   {14, 12, 3}},
                                  {5437022.5296,
                                   45.8551,
                                   -6.3504,
                                   3,
                                   2254.1506,
                                   99,
                                   34002,
                                   303,
                                   {16, 12, 5},
                                   {16, 9, 8}},
                                  {5440504.6170,
                                   14.9896,
                                   -2.6617,
                                   3,
                                   375.4308,
                                   15,
                                   -34002,
                                   303,
                                   {7, 7, 1},
                                   {6, 4, 2}},
                                  {5444905.5703,
                                   14.9896,
                                   -3.2205,
                                   3,
                                   210.2648,
                                   13,
                                   -34002,
                                   303,
                                   {6, 5, 1},
                                   {5, 2, 4}},
                                  {5449769.1959,
                                   1.5266,
                                   -7.1651,
                                   3,
                                   1129.7641,
                                   69,
                                   34002,
                                   303,
                                   {11, 11, 0},
                                   {11, 6, 5}},
                                  {5454189.7015,
                                   0.1194,
                                   -4.6502,
                                   3,
                                   114.1722,
                                   11,
                                   34002,
                                   303,
                                   {5, 5, 1},
                                   {4, 0, 4}},
                                  {5456880.5783,
                                   0.7166,
                                   -4.9729,
                                   3,
                                   951.6482,
                                   23,
                                   34002,
                                   303,
                                   {11, 7, 5},
                                   {11, 4, 8}},
                                  {5465224.4246,
                                   2.6810,
                                   -2.7711,
                                   3,
                                   1404.1081,
                                   81,
                                   34002,
                                   303,
                                   {13, 8, 5},
                                   {12, 9, 4}},
                                  {5473789.6442,
                                   0.3513,
                                   -4.8543,
                                   3,
                                   742.8526,
                                   21,
                                   34002,
                                   303,
                                   {10, 5, 5},
                                   {10, 2, 8}},
                                  {5478342.3225,
                                   14.9896,
                                   -2.1014,
                                   3,
                                   1307.3605,
                                   81,
                                   -34002,
                                   303,
                                   {13, 6, 7},
                                   {12, 7, 6}},
                                  {5479479.8981,
                                   347.7638,
                                   -2.4051,
                                   3,
                                   1698.0261,
                                   117,
                                   34002,
                                   303,
                                   {19, 0, 19},
                                   {18, 1, 18}},
                                  {5479479.8981,
                                   347.7638,
                                   -2.8823,
                                   3,
                                   1698.0261,
                                   39,
                                   34002,
                                   303,
                                   {19, 1, 19},
                                   {18, 0, 18}},
                                  {5487707.1614,
                                   3.1702,
                                   -2.2960,
                                   3,
                                   1516.9382,
                                   93,
                                   34002,
                                   303,
                                   {15, 4, 11},
                                   {14, 5, 10}},
                                  {5487729.4647,
                                   3.1604,
                                   -2.7731,
                                   3,
                                   1516.9376,
                                   31,
                                   34002,
                                   303,
                                   {15, 5, 11},
                                   {14, 4, 10}},
                                  {5488987.9701,
                                   9.3133,
                                   -2.8483,
                                   3,
                                   1592.4247,
                                   33,
                                   34002,
                                   303,
                                   {16, 3, 13},
                                   {15, 4, 12}},
                                  {5488988.2124,
                                   9.3131,
                                   -2.3712,
                                   3,
                                   1592.4247,
                                   99,
                                   34002,
                                   303,
                                   {16, 4, 13},
                                   {15, 3, 12}},
                                  {5489122.2597,
                                   14.9896,
                                   -2.6753,
                                   3,
                                   1421.9064,
                                   29,
                                   -34002,
                                   303,
                                   {14, 5, 9},
                                   {13, 6, 8}},
                                  {5489195.3292,
                                   64.2722,
                                   -2.4294,
                                   3,
                                   1683.7465,
                                   111,
                                   34002,
                                   303,
                                   {18, 2, 17},
                                   {17, 1, 16}},
                                  {5489195.3292,
                                   64.2722,
                                   -2.9065,
                                   3,
                                   1683.7465,
                                   37,
                                   34002,
                                   303,
                                   {18, 1, 17},
                                   {17, 2, 16}},
                                  {5490403.6954,
                                   24.4768,
                                   -2.4170,
                                   3,
                                   1648.2046,
                                   105,
                                   34002,
                                   303,
                                   {17, 2, 15},
                                   {16, 3, 14}},
                                  {5490403.6972,
                                   24.4768,
                                   -2.8941,
                                   3,
                                   1648.2046,
                                   35,
                                   34002,
                                   303,
                                   {17, 3, 15},
                                   {16, 2, 14}},
                                  {5490518.9928,
                                   14.9896,
                                   -2.1978,
                                   3,
                                   1421.8697,
                                   87,
                                   -34002,
                                   303,
                                   {14, 6, 9},
                                   {13, 5, 8}},
                                  {5493313.7499,
                                   1.3572,
                                   -5.9498,
                                   3,
                                   1082.3184,
                                   69,
                                   34002,
                                   303,
                                   {11, 10, 1},
                                   {11, 5, 6}},
                                  {5503439.0029,
                                   24.7060,
                                   -4.8461,
                                   3,
                                   2194.9895,
                                   99,
                                   34002,
                                   303,
                                   {16, 11, 6},
                                   {15, 12, 3}},
                                  {5510017.5060,
                                   128.1058,
                                   -7.2544,
                                   3,
                                   2976.4738,
                                   39,
                                   34002,
                                   303,
                                   {19, 11, 9},
                                   {18, 12, 6}},
                                  {5517658.3079,
                                   658.1394,
                                   -4.8974,
                                   3,
                                   2471.0482,
                                   129,
                                   34002,
                                   303,
                                   {21, 2, 19},
                                   {21, 1, 20}},
                                  {5517658.3081,
                                   658.1394,
                                   -4.8974,
                                   3,
                                   2471.0482,
                                   129,
                                   34002,
                                   303,
                                   {21, 2, 19},
                                   {21, 1, 20}},
                                  {5517658.3095,
                                   658.1394,
                                   -5.3745,
                                   3,
                                   2471.0482,
                                   43,
                                   34002,
                                   303,
                                   {21, 3, 19},
                                   {21, 2, 20}},
                                  {5523653.9355,
                                   0.3483,
                                   -4.3915,
                                   3,
                                   742.8529,
                                   63,
                                   34002,
                                   303,
                                   {10, 6, 5},
                                   {10, 3, 8}},
                                  {5530954.6997,
                                   14.9896,
                                   -2.5639,
                                   3,
                                   1306.0224,
                                   27,
                                   -34002,
                                   303,
                                   {13, 7, 7},
                                   {12, 6, 6}},
                                  {5537910.1175,
                                   37.9387,
                                   -5.0798,
                                   3,
                                   2250.7853,
                                   99,
                                   34002,
                                   303,
                                   {16, 12, 5},
                                   {15, 13, 2}},
                                  {5571342.4399,
                                   14.9896,
                                   -1.1713,
                                   3,
                                   824.2715,
                                   63,
                                   -34002,
                                   303,
                                   {10, 8, 3},
                                   {9, 7, 2}},
                                  {5571549.1973,
                                   0.2222,
                                   -4.4187,
                                   3,
                                   552.0138,
                                   57,
                                   34002,
                                   303,
                                   {9, 4, 5},
                                   {9, 1, 8}},
                                  {5588694.6925,
                                   66.1574,
                                   -5.4231,
                                   3,
                                   2307.4749,
                                   99,
                                   34002,
                                   303,
                                   {16, 13, 4},
                                   {15, 14, 1}},
                                  {5589737.4105,
                                   0.2280,
                                   -4.9010,
                                   3,
                                   552.0139,
                                   19,
                                   34002,
                                   303,
                                   {9, 5, 5},
                                   {9, 2, 8}},
                                  {5603586.9685,
                                   67.9799,
                                   -5.8945,
                                   3,
                                   2307.4682,
                                   33,
                                   34002,
                                   303,
                                   {16, 13, 3},
                                   {15, 14, 2}},
                                  {5622072.4203,
                                   14.9896,
                                   -1.6597,
                                   3,
                                   227.9450,
                                   45,
                                   -34002,
                                   303,
                                   {7, 2, 5},
                                   {6, 1, 6}},
                                  {5623415.7903,
                                   14.9896,
                                   -2.1368,
                                   3,
                                   227.9449,
                                   15,
                                   -34002,
                                   303,
                                   {7, 3, 5},
                                   {6, 0, 6}},
                                  {5633977.7783,
                                   29.9792,
                                   -2.3339,
                                   3,
                                   1358.1293,
                                   81,
                                   -34002,
                                   303,
                                   {13, 7, 6},
                                   {12, 8, 5}},
                                  {5644771.9330,
                                   0.2298,
                                   -5.1739,
                                   3,
                                   379.4073,
                                   17,
                                   34002,
                                   303,
                                   {8, 3, 5},
                                   {8, 0, 8}},
                                  {5647576.5659,
                                   46.1878,
                                   -5.5180,
                                   3,
                                   2250.6859,
                                   33,
                                   34002,
                                   303,
                                   {16, 12, 4},
                                   {15, 13, 3}},
                                  {5648034.8993,
                                   21.7976,
                                   -5.7003,
                                   3,
                                   1883.8150,
                                   93,
                                   34002,
                                   303,
                                   {15, 9, 6},
                                   {15, 6, 9}},
                                  {5650314.6168,
                                   0.2294,
                                   -4.6984,
                                   3,
                                   379.4073,
                                   51,
                                   34002,
                                   303,
                                   {8, 4, 5},
                                   {8, 1, 8}},
                                  {5653475.0806,
                                   14.9896,
                                   -1.3108,
                                   3,
                                   457.8796,
                                   51,
                                   -34002,
                                   303,
                                   {8, 6, 3},
                                   {7, 3, 4}},
                                  {5655004.6809,
                                   0.1200,
                                   -7.3102,
                                   3,
                                   227.9450,
                                   39,
                                   34002,
                                   303,
                                   {6, 6, 1},
                                   {6, 1, 6}},
                                  {5688155.9908,
                                   7.3509,
                                   -3.8477,
                                   3,
                                   1648.9115,
                                   29,
                                   34002,
                                   303,
                                   {14, 9, 5},
                                   {13, 10, 4}},
                                  {5702402.5698,
                                   40.7506,
                                   -5.7909,
                                   3,
                                   2439.0688,
                                   35,
                                   34002,
                                   303,
                                   {17, 11, 7},
                                   {16, 12, 4}},
                                  {5712868.7633,
                                   14.9896,
                                   -1.2676,
                                   3,
                                   610.7241,
                                   57,
                                   -34002,
                                   303,
                                   {9, 6, 3},
                                   {8, 5, 4}},
                                  {5717816.2620,
                                   66.7432,
                                   -5.9627,
                                   3,
                                   2699.4331,
                                   111,
                                   34002,
                                   303,
                                   {18, 11, 8},
                                   {17, 12, 5}},
                                  {5752629.6250,
                                   578.0227,
                                   -2.7287,
                                   3,
                                   1880.8019,
                                   123,
                                   34002,
                                   303,
                                   {20, 1, 20},
                                   {19, 0, 19}},
                                  {5752629.6250,
                                   578.0227,
                                   -3.2058,
                                   3,
                                   1880.8019,
                                   41,
                                   34002,
                                   303,
                                   {20, 0, 20},
                                   {19, 1, 19}},
                                  {5758654.2666,
                                   14.9896,
                                   -2.4956,
                                   3,
                                   1605.0138,
                                   93,
                                   -34002,
                                   303,
                                   {15, 5, 10},
                                   {14, 6, 9}},
                                  {5758712.4263,
                                   14.9896,
                                   -2.8516,
                                   3,
                                   1490.5152,
                                   29,
                                   -34002,
                                   303,
                                   {14, 6, 8},
                                   {13, 7, 7}},
                                  {5759037.7012,
                                   14.9896,
                                   -2.9726,
                                   3,
                                   1605.0038,
                                   31,
                                   -34002,
                                   303,
                                   {15, 6, 10},
                                   {14, 5, 9}},
                                  {5759191.1399,
                                   6.8224,
                                   -3.0802,
                                   3,
                                   1699.9886,
                                   33,
                                   34002,
                                   303,
                                   {16, 4, 12},
                                   {15, 5, 11}},
                                  {5759197.0025,
                                   6.8172,
                                   -2.6031,
                                   3,
                                   1699.9884,
                                   99,
                                   34002,
                                   303,
                                   {16, 5, 12},
                                   {15, 4, 11}},
                                  {5761991.8757,
                                   20.4120,
                                   -2.6846,
                                   3,
                                   1775.5177,
                                   105,
                                   34002,
                                   303,
                                   {17, 3, 14},
                                   {16, 4, 13}},
                                  {5761991.9390,
                                   20.4120,
                                   -3.1617,
                                   3,
                                   1775.5177,
                                   35,
                                   34002,
                                   303,
                                   {17, 4, 14},
                                   {16, 3, 13}},
                                  {5764080.0920,
                                   123.1495,
                                   -2.7507,
                                   3,
                                   1866.8464,
                                   117,
                                   34002,
                                   303,
                                   {19, 1, 18},
                                   {18, 2, 17}},
                                  {5764080.0920,
                                   123.1495,
                                   -3.2278,
                                   3,
                                   1866.8464,
                                   39,
                                   34002,
                                   303,
                                   {19, 2, 18},
                                   {18, 1, 17}},
                                  {5764599.9308,
                                   49.5545,
                                   -3.2121,
                                   3,
                                   1831.3448,
                                   37,
                                   34002,
                                   303,
                                   {18, 2, 16},
                                   {17, 3, 15}},
                                  {5764599.9313,
                                   49.5545,
                                   -2.7349,
                                   3,
                                   1831.3448,
                                   111,
                                   34002,
                                   303,
                                   {18, 3, 16},
                                   {17, 2, 15}},
                                  {5771533.2058,
                                   4.1746,
                                   -7.0871,
                                   3,
                                   1306.0224,
                                   25,
                                   34002,
                                   303,
                                   {12, 11, 1},
                                   {12, 6, 6}},
                                  {5772795.1771,
                                   14.9896,
                                   -2.0297,
                                   3,
                                   985.3515,
                                   23,
                                   -34002,
                                   303,
                                   {11, 8, 4},
                                   {10, 7, 3}},
                                  {5774953.0832,
                                   14.9896,
                                   -2.3704,
                                   3,
                                   1490.0984,
                                   87,
                                   -34002,
                                   303,
                                   {14, 7, 8},
                                   {13, 6, 7}},
                                  {5787290.4422,
                                   14.9896,
                                   -1.7889,
                                   3,
                                   415.5219,
                                   17,
                                   -34002,
                                   303,
                                   {8, 4, 4},
                                   {7, 3, 5}},
                                  {5792872.2780,
                                   14.9896,
                                   -1.6803,
                                   3,
                                   487.2351,
                                   51,
                                   -34002,
                                   303,
                                   {8, 7, 2},
                                   {7, 4, 3}},
                                  {5827947.6723,
                                   109.0733,
                                   -5.8672,
                                   3,
                                   2359.3416,
                                   99,
                                   34002,
                                   303,
                                   {16, 14, 3},
                                   {15, 15, 0}},
                                  {5829368.4983,
                                   109.2952,
                                   -6.3436,
                                   3,
                                   2359.3413,
                                   33,
                                   34002,
                                   303,
                                   {16, 14, 2},
                                   {15, 15, 1}},
                                  {5829608.5460,
                                   14.9896,
                                   -1.0462,
                                   3,
                                   856.7557,
                                   63,
                                   -34002,
                                   303,
                                   {10, 9, 2},
                                   {9, 8, 1}},
                                  {5835267.7282,
                                   14.9896,
                                   -1.6394,
                                   3,
                                   818.6985,
                                   21,
                                   -34002,
                                   303,
                                   {10, 8, 2},
                                   {9, 7, 3}},
                                  {5853341.0906,
                                   62.7011,
                                   -5.8830,
                                   3,
                                   2494.3838,
                                   35,
                                   34002,
                                   303,
                                   {17, 12, 6},
                                   {16, 13, 3}},
                                  {5853351.5091,
                                   14.9896,
                                   -1.3174,
                                   3,
                                   415.4772,
                                   51,
                                   -34002,
                                   303,
                                   {8, 5, 4},
                                   {7, 2, 5}},
                                  {5856391.9865,
                                   19.7340,
                                   -3.9993,
                                   3,
                                   1912.2573,
                                   93,
                                   34002,
                                   303,
                                   {15, 10, 5},
                                   {14, 11, 4}},
                                  {5856704.9875,
                                   14.9896,
                                   -1.5207,
                                   3,
                                   856.1456,
                                   21,
                                   -34002,
                                   303,
                                   {10, 9, 1},
                                   {9, 8, 2}},
                                  {5874380.1216,
                                   88.1189,
                                   -6.1507,
                                   3,
                                   2553.7881,
                                   35,
                                   34002,
                                   303,
                                   {17, 13, 5},
                                   {16, 14, 2}},
                                  {5875636.2816,
                                   14.9896,
                                   -1.9084,
                                   3,
                                   1162.1390,
                                   75,
                                   -34002,
                                   303,
                                   {12, 8, 5},
                                   {11, 7, 4}},
                                  {5895330.0306,
                                   7.0852,
                                   -5.8542,
                                   3,
                                   1605.0038,
                                   29,
                                   34002,
                                   303,
                                   {14, 8, 6},
                                   {14, 5, 9}},
                                  {5925806.8493,
                                   14.9896,
                                   -2.5211,
                                   3,
                                   359.3662,
                                   45,
                                   -34002,
                                   303,
                                   {7, 7, 0},
                                   {6, 4, 3}},
                                  {5931556.6729,
                                   98.4701,
                                   -5.6537,
                                   3,
                                   2553.7410,
                                   105,
                                   34002,
                                   303,
                                   {17, 13, 4},
                                   {16, 14, 3}},
                                  {5945324.8373,
                                   14.9896,
                                   -0.9949,
                                   3,
                                   896.1960,
                                   63,
                                   -34002,
                                   303,
                                   {10, 10, 1},
                                   {9, 9, 0}},
                                  {5946398.6938,
                                   14.9896,
                                   -1.4720,
                                   3,
                                   896.1713,
                                   21,
                                   -34002,
                                   303,
                                   {10, 10, 0},
                                   {9, 9, 1}},
                                  {5962429.8951,
                                   0.3747,
                                   -6.1873,
                                   3,
                                   814.4569,
                                   21,
                                   34002,
                                   303,
                                   {10, 8, 2},
                                   {10, 3, 7}},
                                  {5967935.8304,
                                   136.1952,
                                   -6.4904,
                                   3,
                                   2612.5902,
                                   35,
                                   34002,
                                   303,
                                   {17, 14, 4},
                                   {16, 15, 1}},
                                  {5968228.0817,
                                   14.9896,
                                   -2.7004,
                                   3,
                                   1350.1911,
                                   27,
                                   -34002,
                                   303,
                                   {13, 8, 6},
                                   {12, 7, 5}},
                                  {5969420.2120,
                                   91.3831,
                                   -7.2885,
                                   3,
                                   2490.5118,
                                   35,
                                   34002,
                                   303,
                                   {17, 12, 6},
                                   {17, 9, 9}},
                                  {5975135.5152,
                                   138.2327,
                                   -6.0104,
                                   3,
                                   2612.5870,
                                   105,
                                   34002,
                                   303,
                                   {17, 14, 3},
                                   {16, 15, 2}},
                                  {5978753.0539,
                                   39.4228,
                                   -6.2844,
                                   3,
                                   2179.1347,
                                   99,
                                   34002,
                                   303,
                                   {16, 11, 6},
                                   {16, 8, 9}},
                                  {5989546.4580,
                                   3.5794,
                                   -3.0429,
                                   3,
                                   1549.2698,
                                   29,
                                   34002,
                                   303,
                                   {14, 7, 7},
                                   {13, 8, 6}},
                                  {5991611.8602,
                                   0.2514,
                                   -5.8120,
                                   3,
                                   624.4128,
                                   57,
                                   34002,
                                   303,
                                   {9, 7, 2},
                                   {9, 2, 7}},
                                  {5997846.5744,
                                   0.6265,
                                   -5.8317,
                                   3,
                                   1022.3267,
                                   69,
                                   34002,
                                   303,
                                   {11, 9, 2},
                                   {11, 4, 7}},
                                  {6002860.1338,
                                   46.1712,
                                   -5.1192,
                                   3,
                                   2193.9950,
                                   33,
                                   34002,
                                   303,
                                   {16, 11, 5},
                                   {15, 12, 4}},
                                  {6023892.1438,
                                   937.0248,
                                   -3.0745,
                                   3,
                                   2072.6889,
                                   129,
                                   34002,
                                   303,
                                   {21, 0, 21},
                                   {20, 1, 20}},
                                  {6023892.1438,
                                   937.0248,
                                   -3.5516,
                                   3,
                                   2072.6889,
                                   43,
                                   34002,
                                   303,
                                   {21, 1, 21},
                                   {20, 0, 20}},
                                  {6025930.1271,
                                   13.9675,
                                   -6.3058,
                                   3,
                                   1883.8520,
                                   31,
                                   34002,
                                   303,
                                   {15, 10, 6},
                                   {15, 7, 9}},
                                  {6027298.2356,
                                   5.9670,
                                   -3.2963,
                                   3,
                                   1797.1046,
                                   33,
                                   34002,
                                   303,
                                   {16, 5, 11},
                                   {15, 6, 10}},
                                  {6027403.5683,
                                   5.9307,
                                   -2.8191,
                                   3,
                                   1797.1019,
                                   99,
                                   34002,
                                   303,
                                   {16, 6, 11},
                                   {15, 5, 10}},
                                  {6028376.0421,
                                   14.9896,
                                   -2.6832,
                                   3,
                                   1682.7300,
                                   93,
                                   -34002,
                                   303,
                                   {15, 6, 9},
                                   {14, 7, 8}},
                                  {6029806.6173,
                                   15.0116,
                                   -2.9345,
                                   3,
                                   1892.0945,
                                   105,
                                   34002,
                                   303,
                                   {17, 4, 13},
                                   {16, 5, 12}},
                                  {6029808.2011,
                                   15.0092,
                                   -3.4116,
                                   3,
                                   1892.0945,
                                   35,
                                   34002,
                                   303,
                                   {17, 5, 13},
                                   {16, 4, 12}},
                                  {6030338.5204,
                                   6.0833,
                                   -3.3419,
                                   3,
                                   1600.5004,
                                   29,
                                   34002,
                                   303,
                                   {14, 8, 6},
                                   {13, 9, 5}},
                                  {6033229.0824,
                                   14.9896,
                                   -3.1592,
                                   3,
                                   1682.6052,
                                   31,
                                   -34002,
                                   303,
                                   {15, 7, 9},
                                   {14, 6, 8}},
                                  {6034091.2131,
                                   42.0029,
                                   -3.4985,
                                   3,
                                   1967.7170,
                                   37,
                                   34002,
                                   303,
                                   {18, 3, 15},
                                   {17, 4, 14}},
                                  {6034091.2302,
                                   42.0028,
                                   -3.0214,
                                   3,
                                   1967.7170,
                                   111,
                                   34002,
                                   303,
                                   {18, 4, 15},
                                   {17, 3, 14}},
                                  {6037976.7753,
                                   94.6154,
                                   -3.0758,
                                   3,
                                   2023.6312,
                                   117,
                                   34002,
                                   303,
                                   {19, 2, 17},
                                   {18, 3, 16}},
                                  {6037976.7754,
                                   94.6154,
                                   -3.5529,
                                   3,
                                   2023.6312,
                                   39,
                                   34002,
                                   303,
                                   {19, 3, 17},
                                   {18, 2, 16}},
                                  {6038125.1439,
                                   224.9038,
                                   -3.0945,
                                   3,
                                   2059.1154,
                                   123,
                                   34002,
                                   303,
                                   {20, 2, 19},
                                   {19, 1, 18}},
                                  {6038125.1439,
                                   224.9038,
                                   -3.5716,
                                   3,
                                   2059.1154,
                                   41,
                                   34002,
                                   303,
                                   {20, 1, 19},
                                   {19, 2, 18}},
                                  {6041026.6843,
                                   14.9896,
                                   -2.4694,
                                   3,
                                   325.1463,
                                   45,
                                   -34002,
                                   303,
                                   {7, 6, 1},
                                   {6, 3, 4}},
                                  {6042853.3061,
                                   0.1924,
                                   -7.2713,
                                   3,
                                   515.2971,
                                   51,
                                   34002,
                                   303,
                                   {8, 8, 1},
                                   {8, 3, 6}},
                                  {6076790.6349,
                                   0.1922,
                                   -6.6509,
                                   3,
                                   452.4943,
                                   17,
                                   34002,
                                   303,
                                   {8, 6, 2},
                                   {8, 1, 7}},
                                  {6088539.6475,
                                   2.7454,
                                   -5.0972,
                                   3,
                                   1342.9668,
                                   81,
                                   34002,
                                   303,
                                   {13, 7, 6},
                                   {13, 4, 9}},
                                  {6098532.7881,
                                   1.0831,
                                   -6.6277,
                                   3,
                                   1247.6784,
                                   25,
                                   34002,
                                   303,
                                   {12, 10, 2},
                                   {12, 5, 7}},
                                  {6098544.1977,
                                   4.2445,
                                   -5.4425,
                                   3,
                                   1605.0138,
                                   87,
                                   34002,
                                   303,
                                   {14, 9, 6},
                                   {14, 6, 9}},
                                  {6112644.9997,
                                   98.9553,
                                   -5.8464,
                                   3,
                                   2751.5965,
                                   111,
                                   34002,
                                   303,
                                   {18, 12, 7},
                                   {17, 13, 4}},
                                  {6118270.6094,
                                   14.9896,
                                   -1.8735,
                                   3,
                                   781.2679,
                                   21,
                                   -34002,
                                   303,
                                   {10, 7, 3},
                                   {9, 6, 4}},
                                  {6119823.5343,
                                   14.9896,
                                   -2.5305,
                                   3,
                                   1546.0586,
                                   87,
                                   -34002,
                                   303,
                                   {14, 8, 7},
                                   {13, 7, 6}},
                                  {6156053.4529,
                                   14.9896,
                                   -1.9335,
                                   3,
                                   362.3528,
                                   17,
                                   -34002,
                                   303,
                                   {8, 3, 5},
                                   {7, 2, 6}},
                                  {6161634.6891,
                                   14.9896,
                                   -1.4566,
                                   3,
                                   362.3515,
                                   51,
                                   -34002,
                                   303,
                                   {8, 4, 5},
                                   {7, 1, 6}},
                                  {6161914.9007,
                                   95.9701,
                                   -5.2956,
                                   3,
                                   2493.8937,
                                   105,
                                   34002,
                                   303,
                                   {17, 12, 5},
                                   {16, 13, 4}},
                                  {6181330.7246,
                                   141.3958,
                                   -6.0236,
                                   3,
                                   2811.8961,
                                   111,
                                   34002,
                                   303,
                                   {18, 13, 6},
                                   {17, 14, 3}},
                                  {6182018.2808,
                                   211.7292,
                                   -7.1732,
                                   3,
                                   3311.9532,
                                   123,
                                   34002,
                                   303,
                                   {20, 12, 9},
                                   {19, 13, 6}},
                                  {6184740.9426,
                                   2.0706,
                                   -5.6047,
                                   3,
                                   1342.9691,
                                   27,
                                   34002,
                                   303,
                                   {13, 8, 6},
                                   {13, 5, 9}},
                                  {6186469.2178,
                                   0.1356,
                                   -3.8000,
                                   3,
                                   210.2173,
                                   39,
                                   34002,
                                   303,
                                   {6, 6, 1},
                                   {5, 1, 4}},
                                  {6199311.0162,
                                   0.1521,
                                   -6.8823,
                                   3,
                                   298.9696,
                                   45,
                                   34002,
                                   303,
                                   {7, 5, 2},
                                   {7, 0, 7}},
                                  {6215176.7320,
                                   185.1704,
                                   -6.3069,
                                   3,
                                   2874.3526,
                                   111,
                                   34002,
                                   303,
                                   {18, 14, 5},
                                   {17, 15, 2}},
                                  {6215203.1044,
                                   14.9896,
                                   -1.8830,
                                   3,
                                   1013.3421,
                                   23,
                                   -34002,
                                   303,
                                   {11, 9, 3},
                                   {10, 8, 2}},
                                  {6234699.1578,
                                   1.3838,
                                   -5.3552,
                                   3,
                                   1098.0552,
                                   25,
                                   34002,
                                   303,
                                   {12, 6, 6},
                                   {12, 3, 9}},
                                  {6243866.0964,
                                   196.4754,
                                   -6.7740,
                                   3,
                                   2874.3302,
                                   37,
                                   34002,
                                   303,
                                   {18, 14, 4},
                                   {17, 15, 3}},
                                  {6245012.2442,
                                   143.2144,
                                   -6.9052,
                                   3,
                                   3023.7034,
                                   39,
                                   34002,
                                   303,
                                   {19, 12, 8},
                                   {18, 13, 5}},
                                  {6250528.8489,
                                   14.9896,
                                   -1.3984,
                                   3,
                                   567.8815,
                                   57,
                                   -34002,
                                   303,
                                   {9, 5, 4},
                                   {8, 4, 5}},
                                  {6263666.3376,
                                   3.9901,
                                   -6.6616,
                                   3,
                                   1490.0984,
                                   81,
                                   34002,
                                   303,
                                   {13, 11, 2},
                                   {13, 6, 7}},
                                  {6268259.0263,
                                   0.1508,
                                   -4.3940,
                                   3,
                                   166.3442,
                                   13,
                                   34002,
                                   303,
                                   {6, 4, 2},
                                   {5, 1, 5}},
                                  {6274802.4437,
                                   1.2035,
                                   -4.8907,
                                   3,
                                   1098.0557,
                                   75,
                                   34002,
                                   303,
                                   {12, 7, 6},
                                   {12, 4, 9}},
                                  {6285993.3973,
                                   14.9896,
                                   -2.8681,
                                   3,
                                   1750.1938,
                                   93,
                                   -34002,
                                   303,
                                   {15, 7, 8},
                                   {14, 8, 7}},
                                  {6294474.3303,
                                   7.9277,
                                   -3.4981,
                                   3,
                                   1883.8520,
                                   33,
                                   34002,
                                   303,
                                   {16, 6, 10},
                                   {15, 7, 9}},
                                  {6294805.3204,
                                   241.5505,
                                   -6.9050,
                                   3,
                                   2664.3581,
                                   35,
                                   34002,
                                   303,
                                   {17, 15, 3},
                                   {16, 16, 0}},
                                  {6295093.5779,
                                   12.7331,
                                   -3.1675,
                                   3,
                                   1998.1544,
                                   105,
                                   34002,
                                   303,
                                   {17, 5, 12},
                                   {16, 6, 11}},
                                  {6295123.2412,
                                   12.7100,
                                   -3.6446,
                                   3,
                                   1998.1537,
                                   35,
                                   34002,
                                   303,
                                   {17, 6, 12},
                                   {16, 5, 11}},
                                  {6295480.1098,
                                   241.7684,
                                   -6.4275,
                                   3,
                                   2664.3580,
                                   105,
                                   34002,
                                   303,
                                   {17, 15, 2},
                                   {16, 16, 1}},
                                  {6295917.1273,
                                   14.9896,
                                   -3.0207,
                                   3,
                                   1883.8150,
                                   99,
                                   -34002,
                                   303,
                                   {16, 7, 10},
                                   {15, 6, 9}},
                                  {6299459.2901,
                                   31.6473,
                                   -3.7664,
                                   3,
                                   2093.2272,
                                   37,
                                   34002,
                                   303,
                                   {18, 4, 14},
                                   {17, 5, 13}},
                                  {6299459.7290,
                                   31.6463,
                                   -3.2893,
                                   3,
                                   2093.2272,
                                   111,
                                   34002,
                                   303,
                                   {18, 5, 14},
                                   {17, 4, 13}},
                                  {6299633.0548,
                                   14.9896,
                                   -2.0060,
                                   3,
                                   608.5652,
                                   19,
                                   -34002,
                                   303,
                                   {9, 7, 3},
                                   {8, 4, 4}},
                                  {6305235.5604,
                                   81.4015,
                                   -3.3810,
                                   3,
                                   2168.9926,
                                   117,
                                   34002,
                                   303,
                                   {19, 3, 16},
                                   {18, 4, 15}},
                                  {6305235.5650,
                                   81.4015,
                                   -3.8581,
                                   3,
                                   2168.9926,
                                   39,
                                   34002,
                                   303,
                                   {19, 4, 16},
                                   {18, 3, 15}},
                                  {6310546.5990,
                                   171.9946,
                                   -3.9161,
                                   3,
                                   2225.0364,
                                   41,
                                   34002,
                                   303,
                                   {20, 2, 18},
                                   {19, 3, 17}},
                                  {6310546.5991,
                                   171.9946,
                                   -3.4390,
                                   3,
                                   2225.0364,
                                   123,
                                   34002,
                                   303,
                                   {20, 3, 18},
                                   {19, 2, 17}},
                                  {6311310.2413,
                                   393.9727,
                                   -3.9376,
                                   3,
                                   2260.5256,
                                   43,
                                   34002,
                                   303,
                                   {21, 2, 20},
                                   {20, 1, 19}},
                                  {6311310.2414,
                                   393.9727,
                                   -3.4604,
                                   3,
                                   2260.5256,
                                   129,
                                   34002,
                                   303,
                                   {21, 1, 20},
                                   {20, 2, 19}},
                                  {6326885.5636,
                                   104.4715,
                                   -6.6969,
                                   3,
                                   2406.7002,
                                   105,
                                   34002,
                                   303,
                                   {17, 10, 7},
                                   {17, 7, 10}},
                                  {6328876.6099,
                                   14.9896,
                                   -2.5238,
                                   3,
                                   505.7564,
                                   51,
                                   -34002,
                                   303,
                                   {8, 8, 1},
                                   {7, 5, 2}},
                                  {6331268.9537,
                                   14.9896,
                                   -3.3342,
                                   3,
                                   1749.0596,
                                   31,
                                   -34002,
                                   303,
                                   {15, 8, 8},
                                   {14, 7, 7}},
                                  {6336919.1784,
                                   17.8758,
                                   -3.4732,
                                   3,
                                   1860.8363,
                                   93,
                                   34002,
                                   303,
                                   {15, 9, 6},
                                   {14, 10, 5}},
                                  {6347274.9486,
                                   0.6120,
                                   -4.7441,
                                   3,
                                   870.5961,
                                   69,
                                   34002,
                                   303,
                                   {11, 5, 6},
                                   {11, 2, 9}},
                                  {6356925.0891,
                                   184.1962,
                                   -6.4426,
                                   3,
                                   2811.6592,
                                   37,
                                   34002,
                                   303,
                                   {18, 13, 5},
                                   {17, 14, 4}},
                                  {6358443.4981,
                                   264.2117,
                                   -6.6386,
                                   3,
                                   2934.8614,
                                   111,
                                   34002,
                                   303,
                                   {18, 15, 4},
                                   {17, 16, 1}},
                                  {6361888.3290,
                                   266.2465,
                                   -7.1143,
                                   3,
                                   2934.8599,
                                   37,
                                   34002,
                                   303,
                                   {18, 15, 3},
                                   {17, 16, 2}},
                                  {6361935.9388,
                                   0.5520,
                                   -5.2259,
                                   3,
                                   870.5962,
                                   23,
                                   34002,
                                   303,
                                   {11, 6, 6},
                                   {11, 3, 9}},
                                  {6364047.0619,
                                   14.9896,
                                   -1.3964,
                                   3,
                                   1010.1115,
                                   69,
                                   -34002,
                                   303,
                                   {11, 9, 2},
                                   {10, 8, 3}},
                                  {6402702.9010,
                                   14.9896,
                                   -1.8963,
                                   3,
                                   567.6967,
                                   19,
                                   -34002,
                                   303,
                                   {9, 6, 4},
                                   {8, 3, 5}},
                                  {6412944.1112,
                                   14.9896,
                                   -1.8036,
                                   3,
                                   1051.5043,
                                   23,
                                   -34002,
                                   303,
                                   {11, 10, 2},
                                   {10, 9, 1}},
                                  {6425902.6402,
                                   14.9896,
                                   -1.3253,
                                   3,
                                   1051.2106,
                                   69,
                                   -34002,
                                   303,
                                   {11, 10, 1},
                                   {10, 9, 2}},
                                  {6437597.4036,
                                   0.2990,
                                   -5.2152,
                                   3,
                                   660.8967,
                                   21,
                                   34002,
                                   303,
                                   {10, 4, 6},
                                   {10, 1, 9}},
                                  {6442226.0657,
                                   0.2918,
                                   -4.7395,
                                   3,
                                   660.8967,
                                   63,
                                   34002,
                                   303,
                                   {10, 5, 6},
                                   {10, 2, 9}},
                                  {6458508.3113,
                                   10.6222,
                                   -3.0915,
                                   3,
                                   1808.4393,
                                   93,
                                   34002,
                                   303,
                                   {15, 8, 7},
                                   {14, 9, 6}},
                                  {6469046.4304,
                                   219.4789,
                                   -6.9400,
                                   3,
                                   3082.6032,
                                   39,
                                   34002,
                                   303,
                                   {19, 13, 7},
                                   {18, 14, 4}},
                                  {6471690.2421,
                                   14.9896,
                                   -1.8238,
                                   3,
                                   1188.2356,
                                   75,
                                   -34002,
                                   303,
                                   {12, 9, 4},
                                   {11, 8, 3}},
                                  {6485077.7741,
                                   14.9896,
                                   -2.2390,
                                   3,
                                   298.9696,
                                   17,
                                   -34002,
                                   303,
                                   {8, 2, 6},
                                   {7, 1, 7}},
                                  {6485332.5977,
                                   14.9896,
                                   -1.7619,
                                   3,
                                   298.9696,
                                   51,
                                   -34002,
                                   303,
                                   {8, 3, 6},
                                   {7, 0, 7}},
                                  {6499775.5270,
                                   288.8268,
                                   -7.1660,
                                   3,
                                   3147.0696,
                                   39,
                                   34002,
                                   303,
                                   {19, 14, 6},
                                   {18, 15, 3}},
                                  {6506154.6828,
                                   14.9896,
                                   -1.7774,
                                   3,
                                   1094.5219,
                                   23,
                                   -34002,
                                   303,
                                   {11, 11, 1},
                                   {10, 10, 0}},
                                  {6506638.8477,
                                   14.9896,
                                   -1.3002,
                                   3,
                                   1094.5108,
                                   69,
                                   -34002,
                                   303,
                                   {11, 11, 0},
                                   {10, 10, 1}},
                                  {6512762.3034,
                                   0.5097,
                                   -4.9699,
                                   3,
                                   469.2461,
                                   57,
                                   34002,
                                   303,
                                   {9, 3, 6},
                                   {9, 0, 9}},
                                  {6513985.3320,
                                   0.5097,
                                   -5.4474,
                                   3,
                                   469.2461,
                                   19,
                                   34002,
                                   303,
                                   {9, 4, 6},
                                   {9, 1, 9}},
                                  {6533125.5113,
                                   14.9896,
                                   -1.5770,
                                   3,
                                   970.3139,
                                   69,
                                   -34002,
                                   303,
                                   {11, 8, 3},
                                   {10, 7, 4}},
                                  {6551633.3546,
                                   46.6895,
                                   -4.6207,
                                   3,
                                   2139.1776,
                                   33,
                                   34002,
                                   303,
                                   {16, 10, 6},
                                   {15, 11, 5}},
                                  {6558443.3548,
                                   11.2027,
                                   -3.6900,
                                   3,
                                   1960.2475,
                                   33,
                                   34002,
                                   303,
                                   {16, 7, 9},
                                   {15, 8, 8}},
                                  {6559101.5415,
                                   16.1858,
                                   -3.3848,
                                   3,
                                   2093.8240,
                                   105,
                                   34002,
                                   303,
                                   {17, 6, 11},
                                   {16, 7, 10}},
                                  {6559529.4583,
                                   16.1373,
                                   -3.8618,
                                   3,
                                   2093.8130,
                                   35,
                                   34002,
                                   303,
                                   {17, 7, 11},
                                   {16, 6, 10}},
                                  {6561948.9257,
                                   26.3355,
                                   -4.0167,
                                   3,
                                   2208.1364,
                                   37,
                                   34002,
                                   303,
                                   {18, 5, 13},
                                   {17, 6, 12}},
                                  {6561957.4500,
                                   26.3227,
                                   -3.5396,
                                   3,
                                   2208.1362,
                                   111,
                                   34002,
                                   303,
                                   {18, 6, 13},
                                   {17, 5, 12}},
                                  {6568052.8603,
                                   62.9833,
                                   -3.6670,
                                   3,
                                   2303.3546,
                                   117,
                                   34002,
                                   303,
                                   {19, 4, 15},
                                   {18, 5, 14}},
                                  {6568052.9846,
                                   62.9829,
                                   -4.1441,
                                   3,
                                   2303.3546,
                                   39,
                                   34002,
                                   303,
                                   {19, 5, 15},
                                   {18, 4, 14}},
                                  {6573334.1104,
                                   11.4277,
                                   -3.2094,
                                   3,
                                   1959.8719,
                                   99,
                                   34002,
                                   303,
                                   {16, 8, 9},
                                   {15, 7, 8}},
                                  {6575390.4255,
                                   149.7318,
                                   -4.2400,
                                   3,
                                   2379.3127,
                                   41,
                                   34002,
                                   303,
                                   {20, 3, 17},
                                   {19, 4, 16}},
                                  {6575390.4268,
                                   149.7318,
                                   -3.7629,
                                   3,
                                   2379.3127,
                                   123,
                                   34002,
                                   303,
                                   {20, 4, 17},
                                   {19, 3, 16}},
                                  {6577782.1551,
                                   371.9254,
                                   -6.9723,
                                   3,
                                   3212.0891,
                                   117,
                                   34002,
                                   303,
                                   {19, 15, 4},
                                   {18, 16, 3}},
                                  {6582361.4273,
                                   299.9109,
                                   -3.8242,
                                   3,
                                   2435.5336,
                                   129,
                                   34002,
                                   303,
                                   {21, 2, 19},
                                   {20, 3, 18}},
                                  {6582361.4286,
                                   299.9109,
                                   -4.3013,
                                   3,
                                   2435.5336,
                                   43,
                                   34002,
                                   303,
                                   {21, 3, 19},
                                   {20, 2, 18}},
                                  {6591855.7496,
                                   39.0668,
                                   -6.7583,
                                   3,
                                   2093.8130,
                                   33,
                                   34002,
                                   303,
                                   {16, 9, 7},
                                   {16, 6, 10}},
                                  {6593425.2696,
                                   336.6095,
                                   -6.6587,
                                   3,
                                   3146.9563,
                                   117,
                                   34002,
                                   303,
                                   {19, 14, 5},
                                   {18, 15, 4}},
                                  {6595093.8116,
                                   14.9896,
                                   -2.7292,
                                   3,
                                   1380.5116,
                                   27,
                                   -34002,
                                   303,
                                   {13, 9, 5},
                                   {12, 8, 4}},
                                  {6617538.6733,
                                   14.9896,
                                   -2.5385,
                                   3,
                                   635.4082,
                                   19,
                                   -34002,
                                   303,
                                   {9, 8, 2},
                                   {8, 5, 3}},
                                  {6641640.7011,
                                   0.1530,
                                   -4.2231,
                                   3,
                                   166.3435,
                                   39,
                                   34002,
                                   303,
                                   {6, 5, 2},
                                   {5, 0, 5}},
                                  {6656324.0228,
                                   14.9896,
                                   -2.6461,
                                   3,
                                   1586.4084,
                                   87,
                                   -34002,
                                   303,
                                   {14, 9, 6},
                                   {13, 8, 5}},
                                  {6668363.0883,
                                   14.9896,
                                   -2.0367,
                                   3,
                                   738.4675,
                                   21,
                                   -34002,
                                   303,
                                   {10, 6, 4},
                                   {9, 5, 5}},
                                  {6672289.4701,
                                   14.9896,
                                   -1.4694,
                                   3,
                                   515.2971,
                                   57,
                                   -34002,
                                   303,
                                   {9, 4, 5},
                                   {8, 3, 6}},
                                  {6681999.9772,
                                   313.4036,
                                   -7.0067,
                                   3,
                                   3366.8893,
                                   123,
                                   34002,
                                   303,
                                   {20, 13, 8},
                                   {19, 14, 5}},
                                  {6685912.9388,
                                   14.9896,
                                   -2.9021,
                                   3,
                                   458.6686,
                                   17,
                                   -34002,
                                   303,
                                   {8, 7, 1},
                                   {7, 4, 4}},
                                  {6690734.2011,
                                   14.9896,
                                   -1.9474,
                                   3,
                                   515.2886,
                                   19,
                                   -34002,
                                   303,
                                   {9, 5, 5},
                                   {8, 2, 6}},
                                  {6693562.9604,
                                   104.5890,
                                   -4.8520,
                                   3,
                                   2435.5101,
                                   105,
                                   34002,
                                   303,
                                   {17, 11, 6},
                                   {16, 12, 5}},
                                  {6701914.9608,
                                   14.9896,
                                   -3.1822,
                                   3,
                                   493.3685,
                                   17,
                                   -34002,
                                   303,
                                   {8, 8, 0},
                                   {7, 5, 3}},
                                  {6719970.8610,
                                   14.9896,
                                   -3.2815,
                                   3,
                                   281.6023,
                                   45,
                                   -34002,
                                   303,
                                   {7, 5, 2},
                                   {6, 2, 5}},
                                  {6744592.2253,
                                   8.5077,
                                   -3.4901,
                                   3,
                                   1801.6509,
                                   31,
                                   34002,
                                   303,
                                   {15, 9, 7},
                                   {14, 8, 6}},
                                  {6770712.9528,
                                   29.0744,
                                   -6.3409,
                                   3,
                                   2093.8240,
                                   99,
                                   34002,
                                   303,
                                   {16, 10, 7},
                                   {16, 7, 10}},
                                  {6781246.5277,
                                   488.7929,
                                   -7.0187,
                                   3,
                                   2985.8911,
                                   111,
                                   34002,
                                   303,
                                   {18, 16, 3},
                                   {17, 17, 0}},
                                  {6790610.3531,
                                   17.4744,
                                   -3.8888,
                                   3,
                                   2026.6263,
                                   33,
                                   34002,
                                   303,
                                   {16, 8, 8},
                                   {15, 9, 7}},
                                  {6791370.8784,
                                   442.0566,
                                   -7.1435,
                                   3,
                                   3431.5003,
                                   123,
                                   34002,
                                   303,
                                   {20, 14, 7},
                                   {19, 15, 4}},
                                  {6797417.4449,
                                   205.9550,
                                   -6.0556,
                                   3,
                                   2749.7364,
                                   37,
                                   34002,
                                   303,
                                   {18, 12, 6},
                                   {17, 13, 5}},
                                  {6798396.1046,
                                   12.0486,
                                   -5.9016,
                                   3,
                                   1797.1019,
                                   93,
                                   34002,
                                   303,
                                   {15, 8, 7},
                                   {15, 5, 10}},
                                  {6822241.4769,
                                   22.6165,
                                   -3.5888,
                                   3,
                                   2179.1347,
                                   105,
                                   34002,
                                   303,
                                   {17, 7, 10},
                                   {16, 8, 9}},
                                  {6822724.7559,
                                   31.3254,
                                   -4.2502,
                                   3,
                                   2312.6154,
                                   37,
                                   34002,
                                   303,
                                   {18, 6, 12},
                                   {17, 7, 11}},
                                  {6822853.5399,
                                   31.2764,
                                   -3.7730,
                                   3,
                                   2312.6121,
                                   111,
                                   34002,
                                   303,
                                   {18, 7, 12},
                                   {17, 6, 11}},
                                  {6827034.0831,
                                   22.7213,
                                   -4.0649,
                                   3,
                                   2179.0136,
                                   35,
                                   34002,
                                   303,
                                   {17, 8, 10},
                                   {16, 7, 9}},
                                  {6827735.3498,
                                   51.7742,
                                   -3.9346,
                                   3,
                                   2427.0195,
                                   117,
                                   34002,
                                   303,
                                   {19, 5, 14},
                                   {18, 6, 13}},
                                  {6827737.8456,
                                   51.7676,
                                   -4.4117,
                                   3,
                                   2427.0194,
                                   39,
                                   34002,
                                   303,
                                   {19, 6, 14},
                                   {18, 5, 13}},
                                  {6830090.9264,
                                   29.9792,
                                   -1.7044,
                                   3,
                                   1222.3933,
                                   75,
                                   -34002,
                                   303,
                                   {12, 10, 3},
                                   {11, 9, 2}},
                                  {6835489.4917,
                                   118.7122,
                                   -4.5440,
                                   3,
                                   2522.4413,
                                   41,
                                   34002,
                                   303,
                                   {20, 4, 16},
                                   {19, 5, 15}},
                                  {6835489.5274,
                                   118.7121,
                                   -4.0669,
                                   3,
                                   2522.4413,
                                   123,
                                   34002,
                                   303,
                                   {20, 5, 16},
                                   {19, 4, 15}},
                                  {6844549.9426,
                                   263.4165,
                                   -4.6438,
                                   3,
                                   2598.6441,
                                   43,
                                   34002,
                                   303,
                                   {21, 4, 18},
                                   {20, 3, 17}},
                                  {6844549.9571,
                                   263.4165,
                                   -4.1667,
                                   3,
                                   2598.6441,
                                   129,
                                   34002,
                                   303,
                                   {21, 3, 18},
                                   {20, 4, 17}},
                                  {6860399.9528,
                                   32.5732,
                                   -4.1640,
                                   3,
                                   2084.8554,
                                   33,
                                   34002,
                                   303,
                                   {16, 9, 7},
                                   {15, 10, 6}},
                                  {6880884.6682,
                                   9.2956,
                                   -6.4066,
                                   3,
                                   1797.1046,
                                   31,
                                   34002,
                                   303,
                                   {15, 9, 7},
                                   {15, 6, 10}},
                                  {6903577.9100,
                                   16.4390,
                                   -3.3848,
                                   3,
                                   2023.8720,
                                   99,
                                   34002,
                                   303,
                                   {16, 9, 8},
                                   {15, 8, 7}},
                                  {6903760.2567,
                                   367.5800,
                                   -6.3150,
                                   3,
                                   3081.6686,
                                   117,
                                   34002,
                                   303,
                                   {19, 13, 6},
                                   {18, 14, 5}},
                                  {6908551.7092,
                                   29.9792,
                                   -2.1755,
                                   3,
                                   1220.6591,
                                   25,
                                   -34002,
                                   303,
                                   {12, 10, 2},
                                   {11, 9, 3}},
                                  {6916644.4816,
                                   3.4746,
                                   -6.7569,
                                   3,
                                   1421.8697,
                                   81,
                                   34002,
                                   303,
                                   {13, 10, 3},
                                   {13, 5, 8}},
                                  {6954845.0743,
                                   0.1575,
                                   -4.2216,
                                   3,
                                   324.9179,
                                   15,
                                   34002,
                                   303,
                                   {7, 7, 1},
                                   {6, 2, 4}},
                                  {6958842.5957,
                                   3.8372,
                                   -6.0477,
                                   3,
                                   1516.9376,
                                   29,
                                   34002,
                                   303,
                                   {14, 7, 7},
                                   {14, 4, 10}},
                                  {6968660.8461,
                                   1.4051,
                                   -7.0947,
                                   3,
                                   1178.0946,
                                   25,
                                   34002,
                                   303,
                                   {12, 9, 3},
                                   {12, 4, 8}},
                                  {6968769.8210,
                                   14.9896,
                                   -1.6214,
                                   3,
                                   737.8607,
                                   63,
                                   -34002,
                                   303,
                                   {10, 7, 4},
                                   {9, 4, 5}},
                                  {6974153.1942,
                                   29.9792,
                                   -2.2958,
                                   3,
                                   1177.9114,
                                   25,
                                   -34002,
                                   303,
                                   {12, 9, 3},
                                   {11, 8, 4}},
                                  {6982757.8373,
                                   14.9896,
                                   -1.6630,
                                   3,
                                   1265.5556,
                                   75,
                                   -34002,
                                   303,
                                   {12, 11, 2},
                                   {11, 10, 1}},
                                  {6988852.9178,
                                   14.9896,
                                   -2.1396,
                                   3,
                                   1265.4171,
                                   25,
                                   -34002,
                                   303,
                                   {12, 11, 1},
                                   {11, 10, 2}},
                                  {6992826.5913,
                                   3.3587,
                                   -5.5823,
                                   3,
                                   1516.9382,
                                   87,
                                   34002,
                                   303,
                                   {14, 8, 7},
                                   {14, 5, 10}},
                                  {7007189.4237,
                                   14.9896,
                                   -1.8655,
                                   3,
                                   776.3768,
                                   63,
                                   -34002,
                                   303,
                                   {10, 8, 3},
                                   {9, 5, 4}},
                                  {7014960.3440,
                                   14.9896,
                                   -1.6012,
                                   3,
                                   452.4945,
                                   57,
                                   -34002,
                                   303,
                                   {9, 3, 6},
                                   {8, 2, 7}},
                                  {7016189.4931,
                                   14.9896,
                                   -2.0783,
                                   3,
                                   452.4943,
                                   19,
                                   -34002,
                                   303,
                                   {9, 4, 6},
                                   {8, 1, 7}},
                                  {7046225.0999,
                                   20.9855,
                                   -1.7696,
                                   3,
                                   927.1022,
                                   69,
                                   -34002,
                                   303,
                                   {11, 7, 4},
                                   {10, 6, 5}},
                                  {7056053.1960,
                                   20.9855,
                                   -1.6564,
                                   3,
                                   1311.5489,
                                   75,
                                   -34002,
                                   303,
                                   {12, 12, 1},
                                   {11, 11, 0}},
                                  {7056274.1431,
                                   20.9855,
                                   -2.1335,
                                   3,
                                   1311.5438,
                                   25,
                                   -34002,
                                   303,
                                   {12, 12, 0},
                                   {11, 11, 1}},
                                  {7075245.9710,
                                   31.3518,
                                   -3.7868,
                                   3,
                                   2254.1506,
                                   105,
                                   34002,
                                   303,
                                   {17, 8, 9},
                                   {16, 9, 8}},
                                  {7082868.1190,
                                   42.8316,
                                   -4.4684,
                                   3,
                                   2406.7389,
                                   37,
                                   34002,
                                   303,
                                   {18, 7, 11},
                                   {17, 8, 10}},
                                  {7084402.2069,
                                   42.8189,
                                   -3.9910,
                                   3,
                                   2406.7002,
                                   111,
                                   34002,
                                   303,
                                   {18, 8, 11},
                                   {17, 7, 10}},
                                  {7085338.1648,
                                   57.5029,
                                   -4.1845,
                                   3,
                                   2540.1980,
                                   117,
                                   34002,
                                   303,
                                   {19, 6, 13},
                                   {18, 7, 12}},
                                  {7085377.4308,
                                   57.4660,
                                   -4.6616,
                                   3,
                                   2540.1970,
                                   39,
                                   34002,
                                   303,
                                   {19, 7, 13},
                                   {18, 6, 12}},
                                  {7087984.0634,
                                   1.9785,
                                   -5.3041,
                                   3,
                                   1253.6687,
                                   81,
                                   34002,
                                   303,
                                   {13, 6, 7},
                                   {13, 3, 10}},
                                  {7092305.7725,
                                   96.9868,
                                   -4.8290,
                                   3,
                                   2654.7683,
                                   41,
                                   34002,
                                   303,
                                   {20, 5, 15},
                                   {19, 6, 14}},
                                  {7092306.5140,
                                   96.9836,
                                   -4.3519,
                                   3,
                                   2654.7682,
                                   123,
                                   34002,
                                   303,
                                   {20, 6, 15},
                                   {19, 5, 14}},
                                  {7092770.0260,
                                   0.7781,
                                   -6.7020,
                                   3,
                                   951.6463,
                                   69,
                                   34002,
                                   303,
                                   {11, 8, 3},
                                   {11, 3, 8}},
                                  {7100476.3280,
                                   1.8688,
                                   -5.7856,
                                   3,
                                   1253.6688,
                                   27,
                                   34002,
                                   303,
                                   {13, 7, 7},
                                   {13, 4, 10}},
                                  {7101674.0891,
                                   213.3748,
                                   -4.4886,
                                   3,
                                   2750.4486,
                                   129,
                                   34002,
                                   303,
                                   {21, 4, 17},
                                   {20, 5, 16}},
                                  {7101674.2304,
                                   213.3750,
                                   -4.9658,
                                   3,
                                   2750.4486,
                                   43,
                                   34002,
                                   303,
                                   {21, 5, 17},
                                   {20, 4, 16}},
                                  {7116328.0757,
                                   31.1322,
                                   -4.2548,
                                   3,
                                   2253.1366,
                                   35,
                                   34002,
                                   303,
                                   {17, 9, 9},
                                   {16, 8, 8}},
                                  {7146070.9780,
                                   14.9896,
                                   -2.6183,
                                   3,
                                   1410.5441,
                                   27,
                                   -34002,
                                   303,
                                   {13, 10, 4},
                                   {12, 9, 3}},
                                  {7162320.0290,
                                   14.9896,
                                   -2.0785,
                                   3,
                                   686.5293,
                                   21,
                                   -34002,
                                   303,
                                   {10, 5, 5},
                                   {9, 4, 6}},
                                  {7170373.9866,
                                   87.0149,
                                   -4.3567,
                                   3,
                                   2378.5644,
                                   105,
                                   34002,
                                   303,
                                   {17, 10, 7},
                                   {16, 11, 6}},
                                  {7185195.1017,
                                   0.2255,
                                   -3.3986,
                                   3,
                                   415.5219,
                                   17,
                                   34002,
                                   303,
                                   {8, 6, 2},
                                   {7, 3, 5}},
                                  {7196573.8621,
                                   1.0258,
                                   -5.6020,
                                   3,
                                   1007.6266,
                                   25,
                                   34002,
                                   303,
                                   {12, 5, 7},
                                   {12, 2, 10}},
                                  {7200629.0130,
                                   0.9677,
                                   -5.1263,
                                   3,
                                   1007.6266,
                                   75,
                                   34002,
                                   303,
                                   {12, 6, 7},
                                   {12, 3, 10}},
                                  {7213417.5549,
                                   14.9896,
                                   -1.6047,
                                   3,
                                   686.4885,
                                   63,
                                   -34002,
                                   303,
                                   {10, 6, 5},
                                   {9, 3, 6}},
                                  {7224310.2115,
                                   0.3345,
                                   -3.3614,
                                   3,
                                   655.1942,
                                   19,
                                   34002,
                                   303,
                                   {9, 9, 1},
                                   {8, 6, 2}},
                                  {7256045.1342,
                                   50.7665,
                                   -4.0100,
                                   3,
                                   2319.6707,
                                   105,
                                   34002,
                                   303,
                                   {17, 9, 8},
                                   {16, 10, 7}},
                                  {7277462.8047,
                                   0.1721,
                                   -4.2613,
                                   3,
                                   281.5945,
                                   15,
                                   34002,
                                   303,
                                   {7, 6, 2},
                                   {6, 1, 5}},
                                  {7290934.2346,
                                   0.4317,
                                   -5.0747,
                                   3,
                                   779.1273,
                                   69,
                                   34002,
                                   303,
                                   {11, 4, 7},
                                   {11, 1, 10}},
                                  {7292072.7156,
                                   0.4179,
                                   -5.5522,
                                   3,
                                   779.1273,
                                   23,
                                   34002,
                                   303,
                                   {11, 5, 7},
                                   {11, 2, 10}},
                                  {7318769.1216,
                                   14.9896,
                                   -2.6194,
                                   3,
                                   1616.7083,
                                   87,
                                   -34002,
                                   303,
                                   {14, 10, 5},
                                   {13, 9, 4}},
                                  {7340355.1662,
                                   57.2452,
                                   -4.6751,
                                   3,
                                   2490.5118,
                                   37,
                                   34002,
                                   303,
                                   {18, 8, 10},
                                   {17, 9, 9}},
                                  {7341981.9392,
                                   76.1682,
                                   -4.4181,
                                   3,
                                   2643.0104,
                                   117,
                                   34002,
                                   303,
                                   {19, 7, 12},
                                   {18, 8, 11}},
                                  {7342474.2947,
                                   76.1013,
                                   -4.8951,
                                   3,
                                   2642.9980,
                                   39,
                                   34002,
                                   303,
                                   {19, 8, 12},
                                   {18, 7, 11}},
                                  {7345082.2054,
                                   14.9896,
                                   -1.8918,
                                   3,
                                   379.4073,
                                   57,
                                   -34002,
                                   303,
                                   {9, 2, 7},
                                   {8, 1, 8}},
                                  {7345128.7008,
                                   0.2706,
                                   -2.3689,
                                   3,
                                   379.4073,
                                   19,
                                   34002,
                                   303,
                                   {9, 3, 7},
                                   {8, 0, 8}},
                                  {7346798.3068,
                                   101.1493,
                                   -5.0956,
                                   3,
                                   2776.5397,
                                   41,
                                   34002,
                                   303,
                                   {20, 6, 14},
                                   {19, 7, 13}},
                                  {7346810.4211,
                                   101.1255,
                                   -4.6185,
                                   3,
                                   2776.5394,
                                   123,
                                   34002,
                                   303,
                                   {20, 7, 14},
                                   {19, 6, 13}},
                                  {7354721.7111,
                                   57.2470,
                                   -4.1950,
                                   3,
                                   2490.1554,
                                   111,
                                   34002,
                                   303,
                                   {18, 9, 10},
                                   {17, 8, 9}},
                                  {7355493.4624,
                                   174.1520,
                                   -5.2680,
                                   3,
                                   2891.3421,
                                   43,
                                   34002,
                                   303,
                                   {21, 6, 16},
                                   {20, 5, 15}},
                                  {7355494.1512,
                                   174.1543,
                                   -4.7909,
                                   3,
                                   2891.3421,
                                   129,
                                   34002,
                                   303,
                                   {21, 5, 16},
                                   {20, 6, 15}},
                                  {7374428.1483,
                                   1.1075,
                                   -5.7397,
                                   3,
                                   568.4724,
                                   21,
                                   34002,
                                   303,
                                   {10, 3, 7},
                                   {10, 0, 10}},
                                  {7374695.0770,
                                   1.1090,
                                   -5.2627,
                                   3,
                                   568.4724,
                                   63,
                                   34002,
                                   303,
                                   {10, 4, 7},
                                   {10, 1, 10}},
                                  {7375842.1108,
                                   14.9896,
                                   -2.5474,
                                   3,
                                   610.7241,
                                   57,
                                   -34002,
                                   303,
                                   {9, 8, 1},
                                   {8, 5, 4}},
                                  {7377780.1446,
                                   199.7919,
                                   -5.5700,
                                   3,
                                   2689.6302,
                                   37,
                                   34002,
                                   303,
                                   {18, 11, 7},
                                   {17, 12, 6}},
                                  {7381111.0389,
                                   10.1310,
                                   -3.5662,
                                   3,
                                   1838.6480,
                                   31,
                                   34002,
                                   303,
                                   {15, 10, 6},
                                   {14, 9, 5}},
                                  {7400116.9057,
                                   29.9792,
                                   -2.4926,
                                   3,
                                   1133.6702,
                                   25,
                                   -34002,
                                   303,
                                   {12, 8, 4},
                                   {11, 7, 5}},
                                  {7418589.8531,
                                   22.2867,
                                   -3.5262,
                                   3,
                                   2072.2132,
                                   99,
                                   34002,
                                   303,
                                   {16, 10, 7},
                                   {15, 9, 6}},
                                  {7419692.1540,
                                   29.9792,
                                   -2.5408,
                                   3,
                                   1451.1036,
                                   27,
                                   -34002,
                                   303,
                                   {13, 11, 3},
                                   {12, 10, 2}},
                                  {7449121.2806,
                                   29.9792,
                                   -2.1281,
                                   3,
                                   1404.1081,
                                   81,
                                   -34002,
                                   303,
                                   {13, 10, 3},
                                   {12, 9, 4}},
                                  {7459149.0386,
                                   29.9792,
                                   -2.0605,
                                   3,
                                   1450.2212,
                                   81,
                                   -34002,
                                   303,
                                   {13, 11, 2},
                                   {12, 10, 3}},
                                  {7467656.5456,
                                   63.8056,
                                   -6.8558,
                                   3,
                                   2312.6121,
                                   105,
                                   34002,
                                   303,
                                   {17, 9, 8},
                                   {17, 6, 11}},
                                  {7486915.8160,
                                   14.9896,
                                   -2.9749,
                                   3,
                                   646.4593,
                                   57,
                                   -34002,
                                   303,
                                   {9, 9, 0},
                                   {8, 6, 3}},
                                  {7492581.8935,
                                   14.9896,
                                   -2.4985,
                                   3,
                                   801.2849,
                                   63,
                                   -34002,
                                   303,
                                   {10, 9, 2},
                                   {9, 6, 3}},
                                  {7496403.4436,
                                   400.3652,
                                   -5.8740,
                                   3,
                                   3018.0831,
                                   117,
                                   34002,
                                   303,
                                   {19, 12, 7},
                                   {18, 13, 6}},
                                  {7509394.4761,
                                   42.2747,
                                   -4.4267,
                                   3,
                                   2313.6937,
                                   35,
                                   34002,
                                   303,
                                   {17, 10, 8},
                                   {16, 9, 7}},
                                  {7509862.7895,
                                   0.1609,
                                   -6.2436,
                                   3,
                                   166.3442,
                                   13,
                                   34002,
                                   303,
                                   {6, 6, 0},
                                   {5, 1, 5}},
                                  {7531306.0853,
                                   14.9896,
                                   -2.1328,
                                   3,
                                   624.4145,
                                   21,
                                   -34002,
                                   303,
                                   {10, 4, 6},
                                   {9, 3, 7}},
                                  {7535984.0468,
                                   14.9896,
                                   -1.6559,
                                   3,
                                   624.4128,
                                   63,
                                   -34002,
                                   303,
                                   {10, 5, 6},
                                   {9, 2, 7}},
                                  {7540510.6131,
                                   14.9896,
                                   -2.5287,
                                   3,
                                   1498.5400,
                                   27,
                                   -34002,
                                   303,
                                   {13, 12, 2},
                                   {12, 11, 1}},
                                  {7543357.7421,
                                   14.9896,
                                   -2.0513,
                                   3,
                                   1498.4753,
                                   81,
                                   -34002,
                                   303,
                                   {13, 12, 1},
                                   {12, 11, 2}},
                                  {7568939.6321,
                                   14.9896,
                                   -2.3957,
                                   3,
                                   925.4386,
                                   23,
                                   -34002,
                                   303,
                                   {11, 8, 4},
                                   {10, 5, 5}},
                                  {7570320.4358,
                                   80.0705,
                                   -4.8837,
                                   3,
                                   2564.1801,
                                   37,
                                   34002,
                                   303,
                                   {18, 9, 9},
                                   {17, 10, 8}},
                                  {7594800.6289,
                                   14.9896,
                                   -2.5376,
                                   3,
                                   1546.9158,
                                   27,
                                   -34002,
                                   303,
                                   {13, 13, 1},
                                   {12, 12, 0}},
                                  {7594903.7575,
                                   14.9896,
                                   -2.0604,
                                   3,
                                   1546.9134,
                                   81,
                                   -34002,
                                   303,
                                   {13, 13, 0},
                                   {12, 12, 1}},
                                  {7598021.3093,
                                   100.3579,
                                   -4.6373,
                                   3,
                                   2735.4825,
                                   117,
                                   34002,
                                   303,
                                   {19, 8, 11},
                                   {18, 9, 10}},
                                  {7599937.6679,
                                   128.8563,
                                   -5.3449,
                                   3,
                                   2887.9166,
                                   41,
                                   34002,
                                   303,
                                   {20, 7, 13},
                                   {19, 8, 12}},
                                  {7600096.5757,
                                   128.7818,
                                   -4.8678,
                                   3,
                                   2887.9125,
                                   123,
                                   34002,
                                   303,
                                   {20, 8, 13},
                                   {19, 7, 12}},
                                  {7602961.0266,
                                   100.3153,
                                   -5.1134,
                                   3,
                                   2735.3597,
                                   39,
                                   34002,
                                   303,
                                   {19, 9, 11},
                                   {18, 8, 10}},
                                  {7606903.8833,
                                   172.1075,
                                   -5.0743,
                                   3,
                                   3021.6026,
                                   129,
                                   34002,
                                   303,
                                   {21, 6, 15},
                                   {20, 7, 14}},
                                  {7606912.8261,
                                   172.0937,
                                   -5.5514,
                                   3,
                                   3021.6026,
                                   43,
                                   34002,
                                   303,
                                   {21, 7, 15},
                                   {20, 6, 14}},
                                  {7614058.6968,
                                   14.9896,
                                   -1.8169,
                                   3,
                                   875.7862,
                                   69,
                                   -34002,
                                   303,
                                   {11, 6, 5},
                                   {10, 5, 6}},
                                  {7644196.7857,
                                   21.1128,
                                   -6.9052,
                                   3,
                                   1998.1537,
                                   33,
                                   34002,
                                   303,
                                   {16, 8, 8},
                                   {16, 5, 11}},
                                  {7657451.3945,
                                   132.9415,
                                   -5.1485,
                                   3,
                                   2629.2805,
                                   37,
                                   34002,
                                   303,
                                   {18, 10, 8},
                                   {17, 11, 7}},
                                  {7673030.4895,
                                   74.8747,
                                   -4.3845,
                                   3,
                                   2561.7063,
                                   111,
                                   34002,
                                   303,
                                   {18, 10, 9},
                                   {17, 9, 8}},
                                  {7674570.4462,
                                   20.2154,
                                   -6.4394,
                                   3,
                                   1998.1544,
                                   99,
                                   34002,
                                   303,
                                   {16, 9, 8},
                                   {16, 6, 11}},
                                  {7686375.2332,
                                   14.9896,
                                   -2.7567,
                                   3,
                                   567.8815,
                                   57,
                                   -34002,
                                   303,
                                   {9, 7, 2},
                                   {8, 4, 5}},
                                  {7735793.6215,
                                   14.9896,
                                   -2.3056,
                                   3,
                                   875.6318,
                                   23,
                                   -34002,
                                   303,
                                   {11, 7, 5},
                                   {10, 4, 6}},
                                  {7751998.0035,
                                   14.9896,
                                   -2.2966,
                                   3,
                                   1358.1293,
                                   81,
                                   -34002,
                                   303,
                                   {13, 9, 4},
                                   {12, 8, 5}},
                                  {7764188.2344,
                                   0.2239,
                                   -3.9008,
                                   3,
                                   457.8796,
                                   51,
                                   34002,
                                   303,
                                   {8, 8, 1},
                                   {7, 3, 4}},
                                  {7773320.6665,
                                   0.1809,
                                   -4.3949,
                                   3,
                                   227.9450,
                                   45,
                                   34002,
                                   303,
                                   {7, 4, 3},
                                   {6, 1, 6}},
                                  {7784794.0992,
                                   14.9896,
                                   -2.5146,
                                   3,
                                   1652.5842,
                                   87,
                                   -34002,
                                   303,
                                   {14, 11, 4},
                                   {13, 10, 3}},
                                  {7787379.2096,
                                   14.9896,
                                   -2.7824,
                                   3,
                                   960.9002,
                                   23,
                                   -34002,
                                   303,
                                   {11, 9, 3},
                                   {10, 6, 4}},
                                  {7791110.2420,
                                   6.6189,
                                   -6.0522,
                                   3,
                                   1699.9884,
                                   93,
                                   34002,
                                   303,
                                   {15, 7, 8},
                                   {15, 4, 11}},
                                  {7802365.5140,
                                   6.9656,
                                   -6.5337,
                                   3,
                                   1699.9886,
                                   31,
                                   34002,
                                   303,
                                   {15, 8, 8},
                                   {15, 5, 11}},
                                  {7845066.8504,
                                   131.2303,
                                   -4.8484,
                                   3,
                                   2817.6511,
                                   117,
                                   34002,
                                   303,
                                   {19, 9, 10},
                                   {18, 10, 9}},
                                  {7852682.8600,
                                   167.7288,
                                   -5.5784,
                                   3,
                                   2988.9672,
                                   41,
                                   34002,
                                   303,
                                   {20, 8, 12},
                                   {19, 9, 11}},
                                  {7854370.7021,
                                   167.6108,
                                   -5.1009,
                                   3,
                                   2988.9252,
                                   123,
                                   34002,
                                   303,
                                   {20, 9, 12},
                                   {19, 8, 11}},
                                  {7856689.1195,
                                   209.7517,
                                   -5.3396,
                                   3,
                                   3141.4245,
                                   129,
                                   34002,
                                   303,
                                   {21, 7, 14},
                                   {20, 8, 13}},
                                  {7856716.1325,
                                   209.6983,
                                   -5.8167,
                                   3,
                                   3141.4232,
                                   43,
                                   34002,
                                   303,
                                   {21, 8, 14},
                                   {20, 7, 13}},
                                  {7867842.9695,
                                   0.3034,
                                   -2.2500,
                                   3,
                                   552.0139,
                                   21,
                                   34002,
                                   303,
                                   {10, 3, 7},
                                   {9, 2, 8}},
                                  {7868111.0327,
                                   0.3025,
                                   -1.7729,
                                   3,
                                   552.0138,
                                   63,
                                   34002,
                                   303,
                                   {10, 4, 7},
                                   {9, 1, 8}},
                                  {7884212.5966,
                                   128.5055,
                                   -5.3176,
                                   3,
                                   2816.6988,
                                   39,
                                   34002,
                                   303,
                                   {19, 10, 10},
                                   {18, 9, 9}},
                                  {7918681.2925,
                                   2.9597,
                                   -6.2198,
                                   3,
                                   1418.4664,
                                   29,
                                   34002,
                                   303,
                                   {14, 6, 8},
                                   {14, 3, 11}},
                                  {7922423.2605,
                                   3.0668,
                                   -5.7442,
                                   3,
                                   1418.4664,
                                   87,
                                   34002,
                                   303,
                                   {14, 7, 8},
                                   {14, 4, 11}},
                                  {7944120.5242,
                                   0.2372,
                                   -3.6492,
                                   3,
                                   415.4772,
                                   51,
                                   34002,
                                   303,
                                   {8, 7, 2},
                                   {7, 2, 5}},
                                  {7953570.7485,
                                   8.1312,
                                   -2.9813,
                                   3,
                                   1648.9115,
                                   29,
                                   34002,
                                   303,
                                   {14, 11, 3},
                                   {13, 10, 4}},
                                  {7957196.9328,
                                   0.1804,
                                   -4.9616,
                                   3,
                                   227.9449,
                                   15,
                                   34002,
                                   303,
                                   {7, 5, 3},
                                   {6, 0, 6}},
                                  {7966499.1638,
                                   313.4102,
                                   -5.4028,
                                   3,
                                   2955.4923,
                                   117,
                                   34002,
                                   303,
                                   {19, 11, 8},
                                   {18, 12, 7}},
                                  {7989233.7593,
                                   9.0455,
                                   -2.4783,
                                   3,
                                   1699.0318,
                                   87,
                                   34002,
                                   303,
                                   {14, 12, 3},
                                   {13, 11, 2}},
                                  {8008496.3908,
                                   9.3564,
                                   -2.9539,
                                   3,
                                   1698.5981,
                                   29,
                                   34002,
                                   303,
                                   {14, 12, 2},
                                   {13, 11, 3}},
                                  {8015959.6130,
                                   1.7164,
                                   -2.5724,
                                   3,
                                   1082.8075,
                                   25,
                                   34002,
                                   303,
                                   {12, 7, 5},
                                   {11, 6, 6}},
                                  {8023956.2278,
                                   12.4625,
                                   -3.4971,
                                   3,
                                   1871.5272,
                                   31,
                                   34002,
                                   303,
                                   {15, 11, 5},
                                   {14, 10, 4}},
                                  {8028100.2160,
                                   190.9195,
                                   -5.0764,
                                   3,
                                   2890.1589,
                                   117,
                                   34002,
                                   303,
                                   {19, 10, 9},
                                   {18, 11, 8}},
                                  {8030019.4688,
                                   0.7193,
                                   -1.8283,
                                   3,
                                   814.4658,
                                   69,
                                   34002,
                                   303,
                                   {11, 5, 6},
                                   {10, 4, 7}},
                                  {8032788.0200,
                                   1.6901,
                                   -5.5214,
                                   3,
                                   1153.9248,
                                   81,
                                   34002,
                                   303,
                                   {13, 5, 8},
                                   {13, 2, 11}},
                                  {8033888.0526,
                                   1.6637,
                                   -5.9990,
                                   3,
                                   1153.9248,
                                   27,
                                   34002,
                                   303,
                                   {13, 6, 8},
                                   {13, 3, 11}},
                                  {8044949.4088,
                                   0.6584,
                                   -2.3061,
                                   3,
                                   814.4569,
                                   23,
                                   34002,
                                   303,
                                   {11, 6, 6},
                                   {10, 3, 7}},
                                  {8086903.8910,
                                   16.1809,
                                   -2.4890,
                                   3,
                                   1750.0944,
                                   87,
                                   34002,
                                   303,
                                   {14, 13, 2},
                                   {13, 12, 1}},
                                  {8088234.9126,
                                   16.1832,
                                   -2.9660,
                                   3,
                                   1750.0642,
                                   29,
                                   34002,
                                   303,
                                   {14, 13, 1},
                                   {13, 12, 2}},
                                  {8101484.1755,
                                   0.4770,
                                   -3.2692,
                                   3,
                                   781.2679,
                                   21,
                                   34002,
                                   303,
                                   {10, 9, 1},
                                   {9, 6, 4}},
                                  {8101570.2662,
                                   0.7209,
                                   -3.2667,
                                   3,
                                   824.2715,
                                   63,
                                   34002,
                                   303,
                                   {10, 10, 1},
                                   {9, 7, 2}},
                                  {8103079.0831,
                                   214.0342,
                                   -5.7992,
                                   3,
                                   3079.6878,
                                   41,
                                   34002,
                                   303,
                                   {20, 9, 11},
                                   {19, 10, 10}},
                                  {8105718.5691,
                                   269.0386,
                                   -5.5880,
                                   3,
                                   3250.9188,
                                   129,
                                   34002,
                                   303,
                                   {21, 8, 13},
                                   {20, 9, 12}},
                                  {8105718.5691,
                                   269.0386,
                                   -5.5880,
                                   3,
                                   3250.9188,
                                   129,
                                   34002,
                                   303,
                                   {21, 8, 13},
                                   {20, 9, 12}},
                                  {8106391.9475,
                                   268.8515,
                                   -6.0651,
                                   3,
                                   3250.9045,
                                   43,
                                   34002,
                                   303,
                                   {21, 9, 13},
                                   {20, 8, 12}},
                                  {8117540.5984,
                                   212.6247,
                                   -5.3193,
                                   3,
                                   3079.3343,
                                   123,
                                   34002,
                                   303,
                                   {20, 10, 11},
                                   {19, 9, 10}},
                                  {8122343.0093,
                                   25.2553,
                                   -2.5101,
                                   3,
                                   1800.2527,
                                   87,
                                   34002,
                                   303,
                                   {14, 14, 1},
                                   {13, 13, 0}},
                                  {8122392.4394,
                                   25.2540,
                                   -2.9872,
                                   3,
                                   1800.2516,
                                   29,
                                   34002,
                                   303,
                                   {14, 14, 0},
                                   {13, 13, 1}},
                                  {8123144.7166,
                                   26.0904,
                                   -3.5492,
                                   3,
                                   2107.6055,
                                   99,
                                   34002,
                                   303,
                                   {16, 11, 6},
                                   {15, 10, 5}},
                                  {8125177.6988,
                                   9.1643,
                                   -3.0998,
                                   3,
                                   1600.5004,
                                   29,
                                   34002,
                                   303,
                                   {14, 10, 4},
                                   {13, 9, 5}},
                                  {8136545.6380,
                                   0.8321,
                                   -5.9063,
                                   3,
                                   906.6886,
                                   25,
                                   34002,
                                   303,
                                   {12, 4, 8},
                                   {12, 1, 11}},
                                  {8136824.9496,
                                   0.8234,
                                   -5.4292,
                                   3,
                                   906.6886,
                                   75,
                                   34002,
                                   303,
                                   {12, 5, 8},
                                   {12, 2, 11}},
                                  {8141282.2580,
                                   52.6840,
                                   -4.5373,
                                   3,
                                   2357.7165,
                                   35,
                                   34002,
                                   303,
                                   {17, 11, 7},
                                   {16, 10, 6}},
                                  {8165582.4131,
                                   659.6131,
                                   -6.6787,
                                   3,
                                   3298.3873,
                                   41,
                                   34002,
                                   303,
                                   {20, 12, 8},
                                   {19, 13, 7}},
                                  {8166842.9233,
                                   95.9964,
                                   -4.5454,
                                   3,
                                   2617.7424,
                                   111,
                                   34002,
                                   303,
                                   {18, 11, 8},
                                   {17, 10, 7}},
                                  {8185997.0683,
                                   0.2472,
                                   -4.3217,
                                   3,
                                   362.3528,
                                   17,
                                   34002,
                                   303,
                                   {8, 5, 3},
                                   {7, 2, 6}},
                                  {8202516.7723,
                                   0.5584,
                                   -2.5243,
                                   3,
                                   469.2461,
                                   21,
                                   34002,
                                   303,
                                   {10, 2, 8},
                                   {9, 1, 9}},
                                  {8202526.5639,
                                   0.5584,
                                   -2.0472,
                                   3,
                                   469.2461,
                                   63,
                                   34002,
                                   303,
                                   {10, 3, 8},
                                   {9, 0, 9}},
                                  {8224625.4916,
                                   1.4383,
                                   -2.3137,
                                   3,
                                   1129.7641,
                                   75,
                                   34002,
                                   303,
                                   {12, 9, 4},
                                   {11, 6, 5}},
                                  {8231550.4249,
                                   2.5570,
                                   -5.5746,
                                   3,
                                   677.0713,
                                   69,
                                   34002,
                                   303,
                                   {11, 3, 8},
                                   {11, 0, 11}},
                                  {8231609.4712,
                                   2.5588,
                                   -6.0518,
                                   3,
                                   677.0713,
                                   23,
                                   34002,
                                   303,
                                   {11, 4, 8},
                                   {11, 1, 11}},
                                  {8240535.4540,
                                   0.4471,
                                   -3.2452,
                                   3,
                                   738.4675,
                                   21,
                                   34002,
                                   303,
                                   {10, 8, 2},
                                   {9, 5, 5}},
                                  {8257112.0680,
                                   0.1828,
                                   -5.3940,
                                   3,
                                   281.6023,
                                   45,
                                   34002,
                                   303,
                                   {7, 7, 0},
                                   {6, 2, 5}},
                                  {8261160.5200,
                                   161.7042,
                                   -5.5038,
                                   3,
                                   2884.7056,
                                   39,
                                   34002,
                                   303,
                                   {19, 11, 9},
                                   {18, 10, 8}},
                                  {8268601.9954,
                                   1.5215,
                                   -2.1302,
                                   3,
                                   1082.3184,
                                   75,
                                   34002,
                                   303,
                                   {12, 8, 5},
                                   {11, 5, 6}},
                                  {8268976.4783,
                                   0.7212,
                                   -3.7885,
                                   3,
                                   818.6985,
                                   21,
                                   34002,
                                   303,
                                   {10, 10, 0},
                                   {9, 7, 3}},
                                  {8328458.3249,
                                   282.3733,
                                   -6.0186,
                                   3,
                                   3160.2682,
                                   41,
                                   34002,
                                   303,
                                   {20, 10, 10},
                                   {19, 11, 9}},
                                  {8354654.3273,
                                   339.5047,
                                   -5.8216,
                                   3,
                                   3350.1063,
                                   129,
                                   34002,
                                   303,
                                   {21, 9, 12},
                                   {20, 10, 11}},
                                  {8359577.1639,
                                   338.7981,
                                   -6.2977,
                                   3,
                                   3349.9775,
                                   43,
                                   34002,
                                   303,
                                   {21, 10, 12},
                                   {20, 9, 11}},
                                  {8365643.1370,
                                   3.3919,
                                   -2.4183,
                                   3,
                                   1307.3605,
                                   81,
                                   34002,
                                   303,
                                   {13, 8, 5},
                                   {12, 7, 6}},
                                  {8378413.1084,
                                   0.4921,
                                   -1.8684,
                                   3,
                                   742.8529,
                                   69,
                                   34002,
                                   303,
                                   {11, 4, 7},
                                   {10, 3, 8}},
                                  {8379561.4112,
                                   0.4777,
                                   -2.3456,
                                   3,
                                   742.8526,
                                   23,
                                   34002,
                                   303,
                                   {11, 5, 7},
                                   {10, 2, 8}},
                                  {8387621.4322,
                                   18.4214,
                                   -3.4239,
                                   3,
                                   1914.2141,
                                   31,
                                   34002,
                                   303,
                                   {15, 12, 4},
                                   {14, 11, 3}},
                                  {8396154.2372,
                                   1.3714,
                                   -3.4417,
                                   3,
                                   985.3515,
                                   23,
                                   34002,
                                   303,
                                   {11, 10, 2},
                                   {10, 7, 3}},
                                  {8424065.0952,
                                   432.8707,
                                   -6.2810,
                                   3,
                                   3232.0145,
                                   41,
                                   34002,
                                   303,
                                   {20, 11, 9},
                                   {19, 12, 8}},
                                  {8425968.1835,
                                   263.2853,
                                   -5.5223,
                                   3,
                                   3157.9475,
                                   123,
                                   34002,
                                   303,
                                   {20, 11, 10},
                                   {19, 10, 9}},
                                  {8454722.8393,
                                   41.9219,
                                   -6.9613,
                                   3,
                                   2208.1362,
                                   105,
                                   34002,
                                   303,
                                   {17, 8, 9},
                                   {17, 5, 12}},
                                  {8476097.7002,
                                   21.5879,
                                   -2.9405,
                                   3,
                                   1912.2573,
                                   93,
                                   34002,
                                   303,
                                   {15, 12, 3},
                                   {14, 11, 4}},
                                  {8503843.9354,
                                   1.6581,
                                   -2.5621,
                                   3,
                                   1022.3647,
                                   25,
                                   34002,
                                   303,
                                   {12, 6, 6},
                                   {11, 5, 7}},
                                  {8517336.2914,
                                   0.2518,
                                   -4.0336,
                                   3,
                                   362.3515,
                                   51,
                                   34002,
                                   303,
                                   {8, 6, 3},
                                   {7, 1, 6}},
                                  {8536923.2451,
                                   23.1663,
                                   -3.0057,
                                   3,
                                   1860.8363,
                                   93,
                                   34002,
                                   303,
                                   {15, 11, 4},
                                   {14, 10, 5}},
                                  {8542679.7422,
                                   25.1106,
                                   -3.4210,
                                   3,
                                   1965.7328,
                                   31,
                                   34002,
                                   303,
                                   {15, 13, 3},
                                   {14, 12, 2}},
                                  {8545099.1519,
                                   1.4555,
                                   -2.0875,
                                   3,
                                   1022.3267,
                                   75,
                                   34002,
                                   303,
                                   {12, 7, 6},
                                   {11, 4, 7}},
                                  {8551919.2349,
                                   25.5106,
                                   -2.9432,
                                   3,
                                   1965.5240,
                                   93,
                                   34002,
                                   303,
                                   {15, 13, 2},
                                   {14, 12, 3}},
                                  {8573698.2588,
                                   0.3500,
                                   -3.5271,
                                   3,
                                   515.2971,
                                   57,
                                   34002,
                                   303,
                                   {9, 6, 3},
                                   {8, 3, 6}},
                                  {8594517.4940,
                                   428.0825,
                                   -6.0455,
                                   3,
                                   3439.0076,
                                   129,
                                   34002,
                                   303,
                                   {21, 10, 11},
                                   {20, 11, 10}},
                                  {8594517.4941,
                                   428.0825,
                                   -6.0455,
                                   3,
                                   3439.0076,
                                   129,
                                   34002,
                                   303,
                                   {21, 10, 11},
                                   {20, 11, 10}},
                                  {8605247.3499,
                                   16.8826,
                                   -6.5461,
                                   3,
                                   1892.0945,
                                   99,
                                   34002,
                                   303,
                                   {16, 8, 9},
                                   {16, 5, 12}},
                                  {8622213.4276,
                                   0.3589,
                                   -4.7177,
                                   3,
                                   608.5652,
                                   19,
                                   34002,
                                   303,
                                   {9, 9, 1},
                                   {8, 4, 4}},
                                  {8622317.8254,
                                   37.6660,
                                   -3.4500,
                                   3,
                                   2019.8587,
                                   31,
                                   34002,
                                   303,
                                   {15, 14, 2},
                                   {14, 13, 1}},
                                  {8622944.1259,
                                   37.6771,
                                   -2.9728,
                                   3,
                                   2019.8445,
                                   93,
                                   34002,
                                   303,
                                   {15, 14, 1},
                                   {14, 13, 2}},
                                  {8634280.9152,
                                   416.6872,
                                   -6.5155,
                                   3,
                                   3438.0757,
                                   43,
                                   34002,
                                   303,
                                   {21, 11, 11},
                                   {20, 10, 10}},
                                  {8636486.8739,
                                   1.2348,
                                   -2.8189,
                                   3,
                                   1162.1390,
                                   75,
                                   34002,
                                   303,
                                   {12, 10, 3},
                                   {11, 7, 4}},
                                  {8638697.1463,
                                   61.6924,
                                   -3.4806,
                                   3,
                                   2071.1854,
                                   31,
                                   34002,
                                   303,
                                   {15, 15, 1},
                                   {14, 14, 0}},
                                  {8638721.5318,
                                   61.6907,
                                   -3.0035,
                                   3,
                                   2071.1849,
                                   93,
                                   34002,
                                   303,
                                   {15, 15, 0},
                                   {14, 14, 1}},
                                  {8647482.6182,
                                   0.3178,
                                   -4.1881,
                                   3,
                                   567.6967,
                                   19,
                                   34002,
                                   303,
                                   {9, 8, 2},
                                   {8, 3, 5}},
                                  {8675339.0689,
                                   9.4113,
                                   -3.2488,
                                   3,
                                   1549.2698,
                                   29,
                                   34002,
                                   303,
                                   {14, 9, 5},
                                   {13, 8, 6}},
                                  {8691364.2452,
                                   35.2006,
                                   -3.4668,
                                   3,
                                   2145.5974,
                                   99,
                                   34002,
                                   303,
                                   {16, 12, 5},
                                   {15, 11, 4}},
                                  {8716453.1199,
                                   0.4087,
                                   -1.9695,
                                   3,
                                   660.8967,
                                   69,
                                   34002,
                                   303,
                                   {11, 3, 8},
                                   {10, 2, 9}},
                                  {8716512.3781,
                                   0.4070,
                                   -2.4467,
                                   3,
                                   660.8967,
                                   23,
                                   34002,
                                   303,
                                   {11, 4, 8},
                                   {10, 1, 9}},
                                  {8728009.3585,
                                   927.9592,
                                   -6.5969,
                                   3,
                                   3589.7768,
                                   129,
                                   34002,
                                   303,
                                   {21, 12, 9},
                                   {20, 13, 8}},
                                  {8728009.3586,
                                   927.9592,
                                   -6.5969,
                                   3,
                                   3589.7768,
                                   129,
                                   34002,
                                   303,
                                   {21, 12, 9},
                                   {20, 13, 8}},
                                  {8735660.7061,
                                   6.8349,
                                   -6.1950,
                                   3,
                                   1592.4247,
                                   93,
                                   34002,
                                   303,
                                   {15, 6, 9},
                                   {15, 3, 12}},
                                  {8736768.8271,
                                   6.9976,
                                   -6.6726,
                                   3,
                                   1592.4247,
                                   31,
                                   34002,
                                   303,
                                   {15, 7, 9},
                                   {15, 4, 12}},
                                  {8777944.6470,
                                   587.0697,
                                   -6.2816,
                                   3,
                                   3518.1632,
                                   129,
                                   34002,
                                   303,
                                   {21, 11, 10},
                                   {20, 12, 9}},
                                  {8828229.6708,
                                   2.3173,
                                   -2.9856,
                                   3,
                                   1306.0224,
                                   27,
                                   34002,
                                   303,
                                   {13, 9, 5},
                                   {12, 6, 6}},
                                  {8851123.0772,
                                   1.3580,
                                   -3.1283,
                                   3,
                                   970.3139,
                                   69,
                                   34002,
                                   303,
                                   {11, 10, 1},
                                   {10, 7, 4}},
                                  {8852605.7473,
                                   0.7759,
                                   -2.9352,
                                   3,
                                   927.1022,
                                   69,
                                   34002,
                                   303,
                                   {11, 9, 2},
                                   {10, 6, 5}},
                                  {8855908.5194,
                                   63.5326,
                                   -4.5097,
                                   3,
                                   2394.2289,
                                   35,
                                   34002,
                                   303,
                                   {17, 12, 6},
                                   {16, 11, 5}},
                                  {8859898.4518,
                                   3.3193,
                                   -6.4107,
                                   3,
                                   1309.4694,
                                   29,
                                   34002,
                                   303,
                                   {14, 5, 9},
                                   {14, 2, 12}},
                                  {8860197.3435,
                                   3.3127,
                                   -5.9337,
                                   3,
                                   1309.4694,
                                   87,
                                   34002,
                                   303,
                                   {14, 6, 9},
                                   {14, 3, 12}},
                                  {8874762.5611,
                                   1.1857,
                                   -2.5584,
                                   3,
                                   951.6482,
                                   25,
                                   34002,
                                   303,
                                   {12, 5, 7},
                                   {11, 4, 8}},
                                  {8878877.1938,
                                   1.1233,
                                   -2.0814,
                                   3,
                                   951.6463,
                                   75,
                                   34002,
                                   303,
                                   {12, 6, 7},
                                   {11, 3, 8}},
                                  {8882492.8435,
                                   197.1029,
                                   -5.6397,
                                   3,
                                   2935.7265,
                                   39,
                                   34002,
                                   303,
                                   {19, 12, 8},
                                   {18, 11, 7}},
                                  {8895112.0873,
                                   113.9943,
                                   -4.6080,
                                   3,
                                   2658.7833,
                                   111,
                                   34002,
                                   303,
                                   {18, 12, 7},
                                   {17, 11, 6}},
                                  {8901947.4670,
                                   320.2713,
                                   -5.6990,
                                   3,
                                   3221.2261,
                                   123,
                                   34002,
                                   303,
                                   {20, 12, 9},
                                   {19, 11, 8}},
                                  {8939861.3684,
                                   1.4734,
                                   -4.1575,
                                   3,
                                   1013.3421,
                                   23,
                                   34002,
                                   303,
                                   {11, 11, 1},
                                   {10, 8, 2}},
                                  {8941156.1762,
                                   3.1853,
                                   -2.4089,
                                   3,
                                   1247.8137,
                                   81,
                                   34002,
                                   303,
                                   {13, 7, 6},
                                   {12, 6, 7}},
                                  {8955049.2462,
                                   0.1950,
                                   -6.0887,
                                   3,
                                   227.9450,
                                   45,
                                   34002,
                                   303,
                                   {7, 6, 1},
                                   {6, 1, 6}},
                                  {8955412.5173,
                                   2.3287,
                                   -3.2840,
                                   3,
                                   1350.1911,
                                   27,
                                   34002,
                                   303,
                                   {13, 10, 4},
                                   {12, 7, 5}},
                                  {8958465.4135,
                                   0.4926,
                                   -3.8222,
                                   3,
                                   686.5293,
                                   21,
                                   34002,
                                   303,
                                   {10, 7, 3},
                                   {9, 4, 6}},
                                  {8960924.7500,
                                   47.6581,
                                   -3.4342,
                                   3,
                                   2194.9895,
                                   99,
                                   34002,
                                   303,
                                   {16, 13, 4},
                                   {15, 12, 3}},
                                  {8968775.2679,
                                   25.7193,
                                   -3.1507,
                                   3,
                                   1808.4393,
                                   93,
                                   34002,
                                   303,
                                   {15, 10, 5},
                                   {14, 9, 6}},
                                  {8975934.8426,
                                   1.9353,
                                   -5.8007,
                                   3,
                                   1043.5618,
                                   81,
                                   34002,
                                   303,
                                   {13, 4, 9},
                                   {13, 1, 12}},
                                  {8976004.3285,
                                   1.9308,
                                   -6.2778,
                                   3,
                                   1043.5618,
                                   27,
                                   34002,
                                   303,
                                   {13, 5, 9},
                                   {13, 2, 12}},
                                  {8990513.0427,
                                   51.7964,
                                   -3.9288,
                                   3,
                                   2139.1776,
                                   33,
                                   34002,
                                   303,
                                   {16, 12, 4},
                                   {15, 11, 5}},
                                  {8997773.4614,
                                   500.6360,
                                   -6.7147,
                                   3,
                                   3513.0111,
                                   43,
                                   34002,
                                   303,
                                   {21, 12, 10},
                                   {20, 11, 9}},
                                  {9005427.5626,
                                   51.4098,
                                   -3.9081,
                                   3,
                                   2193.9950,
                                   33,
                                   34002,
                                   303,
                                   {16, 13, 3},
                                   {15, 12, 4}},
                                  {9035695.7357,
                                   0.2568,
                                   -5.8438,
                                   3,
                                   415.5219,
                                   17,
                                   34002,
                                   303,
                                   {8, 8, 0},
                                   {7, 3, 5}},
                                  {9036864.5832,
                                   1.4684,
                                   -3.7040,
                                   3,
                                   1010.1115,
                                   69,
                                   34002,
                                   303,
                                   {11, 11, 0},
                                   {10, 8, 3}},
                                  {9041482.5451,
                                   2.4962,
                                   -2.8937,
                                   3,
                                   1247.6784,
                                   27,
                                   34002,
                                   303,
                                   {13, 8, 6},
                                   {12, 5, 7}},
                                  {9057439.5972,
                                   1.1405,
                                   -2.2267,
                                   3,
                                   568.4724,
                                   69,
                                   34002,
                                   303,
                                   {11, 2, 9},
                                   {10, 1, 10}},
                                  {9057441.6183,
                                   1.1406,
                                   -2.7039,
                                   3,
                                   568.4724,
                                   23,
                                   34002,
                                   303,
                                   {11, 3, 9},
                                   {10, 0, 10}},
                                  {9082383.4926,
                                   60.2217,
                                   -3.4572,
                                   3,
                                   2250.7853,
                                   99,
                                   34002,
                                   303,
                                   {16, 14, 3},
                                   {15, 13, 2}},
                                  {9084586.2574,
                                   6.0663,
                                   -6.3821,
                                   3,
                                   795.0260,
                                   25,
                                   34002,
                                   303,
                                   {12, 3, 9},
                                   {12, 0, 12}},
                                  {9084599.6651,
                                   6.0675,
                                   -5.9049,
                                   3,
                                   795.0260,
                                   75,
                                   34002,
                                   303,
                                   {12, 4, 9},
                                   {12, 1, 12}},
                                  {9086775.9778,
                                   60.6457,
                                   -3.9340,
                                   3,
                                   2250.6859,
                                   33,
                                   34002,
                                   303,
                                   {16, 14, 2},
                                   {15, 13, 3}},
                                  {9096000.1857,
                                   0.3731,
                                   -4.3446,
                                   3,
                                   515.2886,
                                   19,
                                   34002,
                                   303,
                                   {9, 7, 3},
                                   {8, 2, 6}},
                                  {9144160.5965,
                                   135.3559,
                                   -3.5387,
                                   3,
                                   2359.3416,
                                   99,
                                   34002,
                                   303,
                                   {16, 16, 1},
                                   {15, 15, 0}},
                                  {9144173.0393,
                                   135.3538,
                                   -4.0158,
                                   3,
                                   2359.3413,
                                   33,
                                   34002,
                                   303,
                                   {16, 16, 0},
                                   {15, 15, 1}},
                                  {9147029.9020,
                                   81.9181,
                                   -3.5010,
                                   3,
                                   2307.4749,
                                   99,
                                   34002,
                                   303,
                                   {16, 15, 2},
                                   {15, 14, 1}},
                                  {9147327.8836,
                                   81.9332,
                                   -3.9781,
                                   3,
                                   2307.4682,
                                   33,
                                   34002,
                                   303,
                                   {16, 15, 1},
                                   {15, 14, 2}},
                                  {9218570.1083,
                                   0.8136,
                                   -2.5829,
                                   3,
                                   870.5962,
                                   25,
                                   34002,
                                   303,
                                   {12, 4, 8},
                                   {11, 3, 9}},
                                  {9218851.4477,
                                   0.8023,
                                   -2.1058,
                                   3,
                                   870.5961,
                                   75,
                                   34002,
                                   303,
                                   {12, 5, 8},
                                   {11, 2, 9}},
                                  {9274785.2848,
                                   59.6411,
                                   -4.0391,
                                   3,
                                   2084.8554,
                                   33,
                                   34002,
                                   303,
                                   {16, 11, 5},
                                   {15, 10, 6}},
                                  {9281441.7734,
                                   0.2447,
                                   -5.2632,
                                   3,
                                   298.9696,
                                   17,
                                   34002,
                                   303,
                                   {8, 4, 4},
                                   {7, 1, 7}},
                                  {9300750.9710,
                                   3.9499,
                                   -3.4492,
                                   3,
                                   1188.2356,
                                   75,
                                   34002,
                                   303,
                                   {12, 11, 2},
                                   {11, 8, 3}},
                                  {9313579.5335,
                                   86.0934,
                                   -4.4453,
                                   3,
                                   2439.0688,
                                   35,
                                   34002,
                                   303,
                                   {17, 13, 5},
                                   {16, 12, 4}},
                                  {9327612.2067,
                                   7.2865,
                                   -3.2652,
                                   3,
                                   1490.5152,
                                   29,
                                   34002,
                                   303,
                                   {14, 8, 6},
                                   {13, 7, 7}},
                                  {9346163.1903,
                                   0.2495,
                                   -4.8104,
                                   3,
                                   298.9696,
                                   51,
                                   34002,
                                   303,
                                   {8, 5, 4},
                                   {7, 0, 7}},
                                  {9353360.9916,
                                   2.5623,
                                   -2.3774,
                                   3,
                                   1178.1039,
                                   81,
                                   34002,
                                   303,
                                   {13, 6, 7},
                                   {12, 5, 8}},
                                  {9366135.7218,
                                   2.3633,
                                   -2.8551,
                                   3,
                                   1178.0946,
                                   27,
                                   34002,
                                   303,
                                   {13, 7, 7},
                                   {12, 4, 8}},
                                  {9366997.0802,
                                   0.8969,
                                   -3.2843,
                                   3,
                                   875.7862,
                                   69,
                                   34002,
                                   303,
                                   {11, 8, 3},
                                   {10, 5, 6}},
                                  {9393992.1598,
                                   0.5294,
                                   -3.9347,
                                   3,
                                   737.8607,
                                   63,
                                   34002,
                                   303,
                                   {10, 9, 2},
                                   {9, 4, 5}},
                                  {9436798.1713,
                                   5.7498,
                                   -2.9688,
                                   3,
                                   1546.0586,
                                   87,
                                   34002,
                                   303,
                                   {14, 10, 5},
                                   {13, 7, 6}},
                                  {9476030.0479,
                                   106.9057,
                                   -3.9580,
                                   3,
                                   2435.5101,
                                   105,
                                   34002,
                                   303,
                                   {17, 13, 4},
                                   {16, 12, 5}},
                                  {9511676.7455,
                                   107.6725,
                                   -4.4495,
                                   3,
                                   2494.3838,
                                   35,
                                   34002,
                                   303,
                                   {17, 14, 4},
                                   {16, 13, 3}},
                                  {9516414.7709,
                                   1.2850,
                                   -3.7062,
                                   3,
                                   1133.6702,
                                   25,
                                   34002,
                                   303,
                                   {12, 10, 2},
                                   {11, 7, 5}},
                                  {9533470.7247,
                                   111.5392,
                                   -3.9708,
                                   3,
                                   2493.8937,
                                   105,
                                   34002,
                                   303,
                                   {17, 14, 3},
                                   {16, 13, 4}},
                                  {9535993.6483,
                                   3.0996,
                                   -3.8522,
                                   3,
                                   1380.5116,
                                   27,
                                   34002,
                                   303,
                                   {13, 11, 3},
                                   {12, 8, 4}},
                                  {9537416.9105,
                                   0.7372,
                                   -4.7595,
                                   3,
                                   776.3768,
                                   63,
                                   34002,
                                   303,
                                   {10, 10, 1},
                                   {9, 5, 4}},
                                  {9542584.2933,
                                   17.6056,
                                   -6.6608,
                                   3,
                                   1775.5177,
                                   99,
                                   34002,
                                   303,
                                   {16, 7, 10},
                                   {16, 4, 13}},
                                  {9543620.6985,
                                   4.6053,
                                   -2.8106,
                                   3,
                                   1490.0984,
                                   87,
                                   34002,
                                   303,
                                   {14, 9, 6},
                                   {13, 6, 7}},
                                  {9552886.5486,
                                   143.8028,
                                   -4.5523,
                                   3,
                                   2699.4331,
                                   111,
                                   34002,
                                   303,
                                   {18, 13, 6},
                                   {17, 12, 5}},
                                  {9561217.4932,
                                   0.6800,
                                   -2.6671,
                                   3,
                                   779.1273,
                                   25,
                                   34002,
                                   303,
                                   {12, 3, 9},
                                   {11, 2, 10}},
                                  {9561230.9428,
                                   0.6786,
                                   -2.1900,
                                   3,
                                   779.1273,
                                   75,
                                   34002,
                                   303,
                                   {12, 4, 9},
                                   {11, 1, 10}},
                                  {9573389.5274,
                                   0.2605,
                                   -6.0330,
                                   3,
                                   362.3528,
                                   17,
                                   34002,
                                   303,
                                   {8, 7, 1},
                                   {7, 2, 6}},
                                  {9609609.8614,
                                   129.8264,
                                   -4.4926,
                                   3,
                                   2553.7881,
                                   35,
                                   34002,
                                   303,
                                   {17, 15, 3},
                                   {16, 14, 2}},
                                  {9611693.0339,
                                   130.2283,
                                   -4.0153,
                                   3,
                                   2553.7410,
                                   105,
                                   34002,
                                   303,
                                   {17, 15, 2},
                                   {16, 14, 3}},
                                  {9612205.5859,
                                   3.9168,
                                   -4.0129,
                                   3,
                                   1177.9114,
                                   25,
                                   34002,
                                   303,
                                   {12, 11, 1},
                                   {11, 8, 4}},
                                  {9613411.6404,
                                   592.6252,
                                   -6.8713,
                                   3,
                                   3570.7618,
                                   43,
                                   34002,
                                   303,
                                   {21, 13, 9},
                                   {20, 12, 8}},
                                  {9619400.6479,
                                   120.7921,
                                   -4.0190,
                                   3,
                                   2378.5644,
                                   105,
                                   34002,
                                   303,
                                   {17, 12, 5},
                                   {16, 11, 6}},
                                  {9639313.0865,
                                   273.3974,
                                   -4.5913,
                                   3,
                                   2664.3581,
                                   35,
                                   34002,
                                   303,
                                   {17, 17, 1},
                                   {16, 16, 0}},
                                  {9639319.6739,
                                   273.3950,
                                   -4.1142,
                                   3,
                                   2664.3580,
                                   105,
                                   34002,
                                   303,
                                   {17, 17, 0},
                                   {16, 16, 1}},
                                  {9642542.9850,
                                   376.7095,
                                   -5.7926,
                                   3,
                                   3268.1362,
                                   123,
                                   34002,
                                   303,
                                   {20, 13, 8},
                                   {19, 12, 7}},
                                  {9650724.8217,
                                   232.7925,
                                   -5.6493,
                                   3,
                                   2976.4738,
                                   39,
                                   34002,
                                   303,
                                   {19, 13, 7},
                                   {18, 12, 6}},
                                  {9653898.4309,
                                   21.8535,
                                   -3.2102,
                                   3,
                                   1750.1938,
                                   93,
                                   34002,
                                   303,
                                   {15, 9, 6},
                                   {14, 8, 7}},
                                  {9661400.0691,
                                   166.7886,
                                   -4.5488,
                                   3,
                                   2612.5902,
                                   35,
                                   34002,
                                   303,
                                   {17, 16, 2},
                                   {16, 15, 1}},
                                  {9661543.9234,
                                   166.8040,
                                   -4.0717,
                                   3,
                                   2612.5870,
                                   105,
                                   34002,
                                   303,
                                   {17, 16, 1},
                                   {16, 15, 2}},
                                  {9679229.4887,
                                   7.9946,
                                   -6.3605,
                                   3,
                                   1474.2376,
                                   93,
                                   34002,
                                   303,
                                   {15, 5, 10},
                                   {15, 2, 13}},
                                  {9679311.7515,
                                   7.9912,
                                   -6.8376,
                                   3,
                                   1474.2376,
                                   31,
                                   34002,
                                   303,
                                   {15, 6, 10},
                                   {15, 3, 13}},
                                  {9701974.3543,
                                   0.5641,
                                   -3.8813,
                                   3,
                                   686.4885,
                                   63,
                                   34002,
                                   303,
                                   {10, 8, 3},
                                   {9, 3, 6}},
                                  {9707702.0975,
                                   1.8587,
                                   -2.3585,
                                   3,
                                   1098.0557,
                                   81,
                                   34002,
                                   303,
                                   {13, 5, 8},
                                   {12, 4, 9}},
                                  {9708815.6358,
                                   1.8264,
                                   -2.8356,
                                   3,
                                   1098.0552,
                                   27,
                                   34002,
                                   303,
                                   {13, 6, 8},
                                   {12, 3, 9}},
                                  {9709745.1989,
                                   0.3643,
                                   -4.3219,
                                   3,
                                   452.4945,
                                   57,
                                   34002,
                                   303,
                                   {9, 5, 4},
                                   {8, 2, 7}},
                                  {9728868.0578,
                                   3.5402,
                                   -4.1336,
                                   3,
                                   1222.3933,
                                   75,
                                   34002,
                                   303,
                                   {12, 12, 1},
                                   {11, 9, 2}},
                                  {9768704.7236,
                                   6.6654,
                                   -3.3835,
                                   3,
                                   1586.4084,
                                   87,
                                   34002,
                                   303,
                                   {14, 11, 4},
                                   {13, 8, 5}},
                                  {9780930.1609,
                                   3.5234,
                                   -4.6243,
                                   3,
                                   1220.6591,
                                   25,
                                   34002,
                                   303,
                                   {12, 12, 0},
                                   {11, 9, 3}},
                                  {9807805.1253,
                                   4.8601,
                                   -3.2175,
                                   3,
                                   1421.9064,
                                   29,
                                   34002,
                                   303,
                                   {14, 7, 7},
                                   {13, 6, 8}},
                                  {9809537.1186,
                                   4.7442,
                                   -6.6652,
                                   3,
                                   1189.7267,
                                   29,
                                   34002,
                                   303,
                                   {14, 4, 10},
                                   {14, 1, 13}},
                                  {9809554.8003,
                                   4.7417,
                                   -6.1881,
                                   3,
                                   1189.7267,
                                   87,
                                   34002,
                                   303,
                                   {14, 5, 10},
                                   {14, 2, 13}},
                                  {9825295.0156,
                                   1.5902,
                                   -3.8325,
                                   3,
                                   1082.8075,
                                   25,
                                   34002,
                                   303,
                                   {12, 9, 3},
                                   {11, 6, 6}},
                                  {9842619.7877,
                                   0.3946,
                                   -5.5709,
                                   3,
                                   567.8815,
                                   57,
                                   34002,
                                   303,
                                   {9, 9, 0},
                                   {8, 4, 5}},
                                  {9842906.9336,
                                   4.0828,
                                   -2.7425,
                                   3,
                                   1421.8697,
                                   87,
                                   34002,
                                   303,
                                   {14, 8, 7},
                                   {13, 5, 8}},
                                  {9856383.3612,
                                   0.3833,
                                   -4.8593,
                                   3,
                                   452.4943,
                                   19,
                                   34002,
                                   303,
                                   {9, 6, 4},
                                   {8, 1, 7}},
                                  {9895313.0930,
                                   186.2769,
                                   -4.5243,
                                   3,
                                   2751.5965,
                                   111,
                                   34002,
                                   303,
                                   {18, 14, 5},
                                   {17, 13, 4}},
                                  {9909798.1702,
                                   2.5427,
                                   -2.9067,
                                   3,
                                   677.0713,
                                   25,
                                   34002,
                                   303,
                                   {12, 2, 10},
                                   {11, 1, 11}},
                                  {9909798.6057,
                                   2.5427,
                                   -2.4296,
                                   3,
                                   677.0713,
                                   75,
                                   34002,
                                   303,
                                   {12, 3, 10},
                                   {11, 0, 11}},
                                  {9925836.5355,
                                   58.5899,
                                   -4.1385,
                                   3,
                                   2026.6263,
                                   33,
                                   34002,
                                   303,
                                   {16, 10, 6},
                                   {15, 9, 7}},
                                  {9933638.3354,
                                   14.0158,
                                   -6.2519,
                                   3,
                                   922.3182,
                                   81,
                                   34002,
                                   303,
                                   {13, 3, 10},
                                   {13, 0, 13}},
                                  {9933641.4807,
                                   14.0164,
                                   -6.7290,
                                   3,
                                   922.3182,
                                   27,
                                   34002,
                                   303,
                                   {13, 4, 10},
                                   {13, 1, 13}},
                                  {9979095.8362,
                                   208.7064,
                                   -4.9958,
                                   3,
                                   2749.7364,
                                   37,
                                   34002,
                                   303,
                                   {18, 14, 4},
                                   {17, 13, 5}}};
#endif  // H2S
